package sync4j.foundation.engine.source;

import java.io.Serializable;
import java.security.Principal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.logging.*;

import sync4j.framework.core.*;
import sync4j.framework.engine.*;
import sync4j.framework.engine.source.*;
import sync4j.framework.security.*;
import sync4j.framework.tools.beans.*;
import sync4j.zimbra.util.Def;

// TODO: Update this class to reflect Funambol v3+ data model and to fetch
// jndiDataSource string from a user config panel, rather than hardcoded
// From Funambol cvs, use ./3x/modules/domino/src/java/com/funambol/domino/engine/cache/CachedSyncSource.java
// as a guide.
// Does that mean we need to change class package name??

/**
 * This class represents a SyncSource that read and store the state of the items
 * in a support table.
 */

public abstract class CachedSyncSource extends AbstractSyncSource
implements SyncSource, Serializable, LazyInitBean
{


    private String jndiDataSourceName = "jdbc/sync4j";

    private ItemStateDAO itemStateDao;

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    private SyncItemInfo[] localItems;
    private SyncItemInfo[] serverItems;

    private SyncItemKey[] newItemKeys;
    private SyncItemKey[] deleteItemKeys;
    private SyncItemKey[] updateItemKeys;

    private Principal principal = null;

    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Init " + getClass().getName() + " with sourceUri: " + sourceURI);
        }

        itemStateDao = new ItemStateDAO(jndiDataSourceName);
        try {
            itemStateDao.init();
        } catch (Exception e) {
            throw new BeanInitializationException("Error initializing ItemStateDAO", e);
        }
    }

    /**
     * @param jndiDataSourceName String
     */
    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    /**
     * @return String
     */
    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }


    /**
     * BeginSync
     *
     * @param principal Principal
     * @param syncMode  int
     * @throws SyncSourceException
     */
    public void beginSync(Principal principal, int syncMode) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource beginSync [" + sourceURI + "]");
        }

        this.principal = principal;
        String principalId = ((Sync4jPrincipal) principal).getId();

        try {

            if ((syncMode != AlertCode.SLOW) &&
                    (syncMode != AlertCode.REFRESH_FROM_SERVER))
            {
                this.serverItems = getAllSyncItemInfo(principal);
                this.localItems = itemStateDao.getLocalItems(this.sourceURI,
                        principalId);
                if (serverItems != null)
                    if (localItems != null) {
                        log.finest("Found " + serverItems.length + " serverItems and " + localItems.length + " localitems! ");

                        for (int i = 0; i < serverItems.length; i++) {
                            log.finest("ServerItemKey[" + i + "]=" + serverItems[i].getKey().getKeyAsString());
                        }
                        for (int i = 0; i < localItems.length; i++) {
                            log.finest("LocalItemKey[" + i + "]=" + localItems[i].getKey().getKeyAsString());
                        }
                    } else {
                        log.finest("LOCALITEMS NULL");
                    }
                else {
                    log.finest("SERVERITEMS NULL");
                }


                this.newItemKeys = findNewItemIds();

                if (newItemKeys != null) {
                    log.finest("Found new " + newItemKeys.length + " Items ");
                    for (int i = 0; i < newItemKeys.length; i++) {
                        log.finest("newItemKey[" + i + "]=" + newItemKeys[i].getKeyAsString());
                    }
                }

                this.updateItemKeys = findUpdateItemIds();
                this.deleteItemKeys = findDeleteItemIds();
            }

        } catch (Exception e) {
            log.finest("CachedSyncSource beginSync [" + sourceURI + "] failed");
            throw new SyncSourceException("Error accessing items: " +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource beginSync done [" + sourceURI + "]");
        }

    }

    /**
     * EndSync
     * <p/>
     * by read only update / add items
     * to setting lastModified
     * with addItems[exist], deleteItems[exist],
     * updateItems[implements] array
     *
     * @param principal Principal
     * @throws SyncSourceException
     */
    public void endSync(Principal principal) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource endSync [" + sourceURI + "]");
        }

        String principalId = ((Sync4jPrincipal) principal).getId();

        try {

            serverItems = getAllSyncItemInfo(principal);
            log.finest("Starting updateLocalItems");
            try{
            itemStateDao.updateLocalItems(this.serverItems,
                                          this.sourceURI,
                                          principalId);
            }catch(Exception e){
                log.finest("updateLocal... failes");
            }

            log.finest("updateLocalItems complete ");
        } catch (Exception e) {
            //throw new SyncSourceException("Error caching items:" + e.getMessage());
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource endSync done [" + sourceURI + "]");
        }

    }

    /**
     * @see SyncSource
     */
    public SyncItemKey[] getUpdatedSyncItemKeys(Principal principal, Timestamp timestamp) throws
            SyncSourceException {
        return updateItemKeys;
    }

    /**
     * @see SyncSource
     */
    public SyncItem[] getUpdatedSyncItems(Principal principal, Timestamp since) throws
            SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getUpdatedSyncItems [" + sourceURI + "]");
        }
        return getSyncItems(updateItemKeys, SyncItemState.UPDATED);
    }


    /**
     * @see SyncSource
     */
    public SyncItemKey[] getNewSyncItemKeys(Principal principal, Timestamp since) throws
            SyncSourceException {
        return newItemKeys;
    }

    /**
     * @see SyncSource
     */
    public SyncItem[] getNewSyncItems(Principal principal,
                                      Timestamp since)
            throws SyncSourceException {
        if (log.isLoggable(Level.FINEST)) {
            log.finest("getNewSyncItems [" + sourceURI + "]");
        }
        return getSyncItems(newItemKeys, SyncItemState.NEW);
    }

    /**
     * @see SyncSource
     */
    public SyncItemKey[] getDeletedSyncItemKeys(Principal principal,
                                                Timestamp since)
    throws  SyncSourceException
    {
        if (log.isLoggable(Level.FINEST))
            log.finest("CachedSyncSource.getDeletedSyncItemKeys [" + sourceURI + "]");
        return deleteItemKeys;
    }

    /**
     * @see SyncSource
     */
    public SyncItem[] getDeletedSyncItems(Principal principal,
                                          Timestamp since)
    throws  SyncSourceException
    {
        if (log.isLoggable(Level.FINEST))
            log.finest("CachedSyncSource.getDeletedSyncItems [" + sourceURI + "] started");
        int num = deleteItemKeys.length;
        SyncItem[] items = new SyncItem[num];
        for (int i = 0; i < num; i++) {
            items[i] = new SyncItemImpl(this,
                    deleteItemKeys[i].getKeyValue(),
                    SyncItemState.DELETED);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CachedSyncSource.getDeletedSyncItems [" + sourceURI + "] ended");

        return items;
    }


    /**
     * @param principal Principal
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public abstract SyncItemInfo[] getAllSyncItemInfo(Principal principal)
            throws SyncSourceException;



    // ---------------------------------------------------------- Private Methods

    /**
     * Creates new items id array
     *
     * @return new items id array
     */
    private SyncItemKey[] findNewItemIds() {

        ArrayList newRows;
        SyncItemInfo serverItem;

        newRows = new ArrayList();

        for (int i = 0, l = this.serverItems.length; i < l; i++) {

            serverItem = this.serverItems[i];

            if (!isItemInArray(this.localItems, serverItem)) {
                newRows.add(serverItem.getKey());
            }

        }

        return (SyncItemKey[]) newRows.toArray(new SyncItemKey[0]);
    }

    /**
     * Creates deleted items id array
     *
     * @return deleted items id array
     */
    private SyncItemKey[] findDeleteItemIds() {

        ArrayList deleteRows;
        SyncItemInfo localItem;

        deleteRows = new ArrayList();

        for (int i = 0, l = this.localItems.length; i < l; i++) {

            localItem = localItems[i];

            if (!this.isItemInArray(this.serverItems, localItem)) {
                deleteRows.add(localItem.getKey());
            }

        }

        return (SyncItemKey[]) deleteRows.toArray(new SyncItemKey[0]);
    }

    /**
     * Creates updated items id array
     *
     * @return updated items id array
     */
    private SyncItemKey[] findUpdateItemIds() {

        ArrayList updateRows = new ArrayList();

        SyncItemInfo serverItem;
        SyncItemInfo localItem;

        Date serverItemLastModified;
        Date localItemLastModified;

        for (int i = 0, l = this.serverItems.length; i < l; i++) {

            serverItem = this.serverItems[i];
            localItem = this.getItemById(this.localItems,
                    serverItem.getKey().getKeyAsString());

            if (localItem != null) {
                serverItemLastModified = serverItem.getLastModified();
                localItemLastModified = localItem.getLastModified();

                if (!serverItemLastModified.equals(localItemLastModified)) {
                    updateRows.add(serverItem.getKey());
                }
            }
        }

        return (SyncItemKey[]) updateRows.toArray(new SyncItemKey[0]);
    }


    /**
     * Checks if item id is included in ids from array of items
     *
     * @param items items array
     * @param id    id to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     */
    private boolean isIdInArray(SyncItemInfo[] items,
                                String id) {

        boolean isFound = false;

        for (int i = 0, l = items.length; i < l; i++) {

            if (items[i].getKey().getKeyAsString().equals(id)) {
                isFound = true;
                break;
            }
        }

        return isFound;
    }

    /**
     * Checks if item is included in array of items
     *
     * @param items items array
     * @param item  item to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     */
    private boolean isItemInArray(SyncItemInfo[] items,
                                  SyncItemInfo item) {

        return isIdInArray(items, item.getKey().getKeyAsString());

    }

    /**
     * Gets item by id
     *
     * @param items array
     * @param id
     * @return <p>item</p> if found
     *         <p>null</p> if not found
     */
    private SyncItemInfo getItemById(SyncItemInfo[] items,
                                     String id) {

        SyncItemInfo item = null;

        for (int i = 0, l = items.length; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                item = items[i];
                break;
            }
        }
        return item;
    }


    /**
     * @param keys  SyncItemKey[]
     * @param state char
     * @return SyncItem[]
     * @throws SyncSourceException
     */
    private SyncItem[] getSyncItems(SyncItemKey[] keys, char state) throws SyncSourceException
    {

        SyncItem[] items = getSyncItemsFromIds(principal, keys);

        int num = items.length;
        for (int i = 0; i < num; i++) {
            items[i].setState(state);
        }
        return items;
    }


}
