package sync4j.foundation.engine.source;

import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.util.Date;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.Timestamp;

import sync4j.framework.tools.DBTools;

import java.sql.ResultSet;
import java.util.ArrayList;

import sync4j.zimbra.util.Def;

// TODO:
// Update this class to reflect minor changes in the Funambol v3+ data model
// From the Funambol cvs, use 
// /3x/modules/domino/src/java/com/funambol/domino/engine/cache/ItemStateDAO.java
// as a guide.

/**
 * Contains methods to access to the table with the state of the items.
 */
public class ItemStateDAO {

    // --------------------------------------------------------------- Constants
    private static final String SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE =
            "select id, last_modified " +
            "from ss_item_status where source_uri = ? and principal = ?";

    private static final String SQL_ADD_LOCAL_ITEM =
            "insert into ss_item_status " +
            "(id, last_modified, source_uri, principal) values(?, ?, ?, ?)";

    private static final String SQL_DELETE_LOCAL_ITEMS =
            "delete from ss_item_status where source_uri = ? and principal = ? ";


    private Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    private String jndiDataSourceName;

    private DataSource dataSource;

    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }

    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    // ------------------------------------------------------------- Constructor
    protected ItemStateDAO() {
    }

    public ItemStateDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // ---------------------------------------------------------- Public Methods
    public void init() throws Exception {
        lookupDataSource();
    }


    /**
     * Adds local items
     *
     * @param items
     * @param sourceURI
     * @param principal
     * @throws Exception
     */
    public void addLocalItems(SyncItemInfo[] items,
                              String sourceURI,
                              String principal) throws Exception {

        Connection connection = null;
        PreparedStatement ps = null;

        String id;
        Date lastModified;

        if (items != null)
            try {
                connection = dataSource.getConnection();
                for (int i = 0; i < items.length; i++) {
                    ps = connection.prepareStatement(SQL_ADD_LOCAL_ITEM);

                    id = items[i].getKey().getKeyAsString();
                    lastModified = items[i].getLastModified();

                    ps.setString(1, id);
                    ps.setTimestamp(2, new Timestamp(lastModified.getTime()));
                    ps.setString(3, sourceURI);
                    ps.setString(4, principal);
                    ps.executeUpdate();

                    ps.close();
                    ps = null;
                }
            } catch (Exception e) {
                throw new Exception("Error adding local items", e);
            } finally {
                DBTools.close(connection, ps, null);
            }
    }

    /**
     * Delete local items
     *
     * @param sourceURI
     * @param principal
     * @throws Exception
     */
    public void deleteLocalItems(String sourceURI, String principal)
            throws Exception {

        Connection connection = null;
        PreparedStatement ps = null;

        try {

            connection = dataSource.getConnection();

            ps = connection.prepareStatement(SQL_DELETE_LOCAL_ITEMS);

            ps.setString(1, sourceURI);
            ps.setString(2, principal);

            ps.executeUpdate();

        } catch (Exception e) {
            throw new Exception("Error deleting local items", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     * @throws Exception
     */
    public void updateLocalItems(SyncItemInfo[] localItems,
                                 String sourceURI,
                                 String principal)
            throws Exception {
        this.deleteLocalItems(sourceURI, principal);
        this.addLocalItems(localItems, sourceURI, principal);
    }

    /**
     * Gets local items
     *
     * @param sourceURI
     * @param principalId
     * @return array of find local items
     * @throws Exception
     */
    public SyncItemInfo[] getLocalItems(String sourceURI, String principalId)
            throws Exception {

        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs = null;

        ArrayList rows;

        SyncItemInfo[] items = null;

        try {

            connection = dataSource.getConnection();

            rows = new ArrayList();

            ps = connection.prepareStatement(SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

            ps.setString(1, sourceURI);
            ps.setString(2, principalId);

            rs = ps.executeQuery();

            while (rs.next()) {
                rows.add(new SyncItemInfo(rs.getString(1),
                        rs.getTimestamp(2)));
            }

            items = (SyncItemInfo[]) rows.toArray(new SyncItemInfo[0]);

        } catch (Exception e) {
            throw new Exception("Error getting local items", e);
        } finally {
            DBTools.close(connection, ps, rs);
        }

        return items;
    }



    // --------------------------------------------------------- Private Methods

    /**
     * Lookups the datasource
     * <p/>
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     * <p/>
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    private void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();
        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource: datasource lookup");
        }

        try {
            // try tomcat  i.e. java:comp/env/jdbc/sync4j
            dataSource = (DataSource) ctx.lookup("java:comp/env/" + jndiDataSourceName);
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: datasource lookup done using java:comp/env/ prefix");
            }
        } catch (NamingException ne) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No tomcat datasource lookup!");
            }
            try {
                // try jboss i.e. java:jdbc/sync4j
                dataSource = (DataSource) ctx.lookup("java:" + jndiDataSourceName);
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("CachedSyncSource: datasource lookup done using java: prefix");
                }
            } catch (Exception ex) {
                if (log.isLoggable(Level.FINEST)) {
                    log.finest("CachedSyncSource: No jboss datasource lookup!");
                }
                try {
                    // try other j2ee server jdbc/sync4j
                    dataSource = (DataSource) ctx.lookup("" + jndiDataSourceName);
                    if (log.isLoggable(Level.FINEST)) {
                        log.finest("CachedSyncSource: datasource lookup done using - prefix");
                    }
                } catch (Exception e) {
                    if (log.isLoggable(Level.FINEST)) {
                        log.finest("CachedSyncSource: No j2ee server datasource lookup!");
                    }
                    throw new Exception("Data source " + jndiDataSourceName
                            + " not found " + e.toString());
                }
            }
        }
    }

}
