package sync4j.zimbra.engine.source;

import java.io.ByteArrayInputStream;
import java.io.Serializable;

import java.security.Principal;

import java.sql.Timestamp;

import java.util.*;
import java.util.logging.Level;

import sync4j.foundation.engine.source.SyncItemInfo;
import sync4j.foundation.pdi.event.Calendar;
import sync4j.foundation.pdi.converter.*;
import sync4j.foundation.pdi.parser.*;
import sync4j.foundation.pdi.common.*;

import sync4j.framework.engine.*;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceException;
import sync4j.framework.tools.Base64;
import sync4j.framework.tools.beans.BeanInitializationException;
import sync4j.framework.tools.beans.LazyInitBean;
import sync4j.framework.security.Sync4jPrincipal;

import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.items.manager.CalendarManager;
import sync4j.zimbra.items.manager.ContactManager;
import sync4j.zimbra.util.Def;
import sync4j.zimbra.util.Utility;
import sync4j.zimbra.util.ICalParser;

/**
 * This class define SyncSource
 * between SyncServer and Zimbra Server calendars items
 */
public final class ZimbraCalendarSyncSource
        extends ZimbraSyncSource
        implements SyncSource, Serializable, LazyInitBean {

    // ------------------------------------------------------------ Private data

    private CalendarManager calendarManager = null;

    // ---------------------------------------------------------- Public Methods

    /**
     * @param principal Principal
     * @param syncMode  int
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void beginSync(Principal principal,
                          int syncMode)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.beginSync[" + sourceURI + "] with mode" + syncMode);
        String credentials;
        credentials = ((Sync4jPrincipal) principal).getEncodedCredentials();
        String username = ((Sync4jPrincipal) principal).getUsername();
        String password = Utility.getPassword(username, credentials);

        HashMap calendarManagerProperties = new HashMap();
        calendarManagerProperties.put(CalendarManager.PROPERTY_NAME_SERVER_URL, getZimbraUrl());
        calendarManagerProperties.put(CalendarManager.PROPERTY_NAME_USER_NAME, username);
        calendarManagerProperties.put(CalendarManager.PROPERTY_NAME_PASSWORD, password);

        String deviceId = ((Sync4jPrincipal) principal).getDeviceId();
        log.warning("Device ID: " + deviceId);
        phone = getPhone(deviceId);
        log.warning("Detected Phone: " + phone);

        calendarManagerProperties.put(ContactManager.PHONE, phone);

        try {
            calendarManager = new CalendarManager(calendarManagerProperties);

        } catch (EntityException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.beginSync failed");
            throw new SyncSourceException("Error creation CalendarManager ["
                    + ex.getMessage() + "]", ex);
        }

        super.beginSync(principal, syncMode);

        // set the device timezone to use with the CalenadarManager
        calendarManager.setDeviceTimezoneDescr( deviceTimezoneDescr );

        if (log.isLoggable(Level.FINEST))
            log.finest("ZimbraCalendarSyncSource.beginSync  ended");
    }

    /**
     * @param principal Principal
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void endSync(Principal principal) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.endSync[" + sourceURI + "]");

        if (log.isLoggable(Level.FINEST))
            log.finest("ZimbraCalendarSyncSource.endSync started [" + sourceURI + "]");
        super.endSync(principal);

        if (calendarManager != null) {
            // close Zimbra connection
            try {
                this.calendarManager.releaseSession();
            } catch (EntityException e) {
                if (log.isLoggable(Level.SEVERE))
                    log.severe("ZimbraCalendarSyncSource.endSync failed");
                throw new SyncSourceException(e);
            }
        }
        if (log.isLoggable(Level.FINEST)) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ZimbraCalendarSyncSource.endSync ended");
        }
    }


    /**
     * The same as <i>getSyncItemFromId()</i>
     *
     * @param principal   Principal
     * @param syncItemExt SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem getSyncItemFromTwin(Principal principal,
                                        SyncItem syncItemExt)
            throws SyncSourceException {
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getSyncItemFromTwin[" +
                    syncItemExt.getKey().getKeyAsString() + "]");

        SyncItem syncItem = null;

        Calendar calendar;
        Calendar calendarTwin;

        // get content of the item
        try {
            calendar = getFoundationCalendarFromSyncItem(syncItemExt);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.getSyncItemFromTwin failed: " + e.toString());
            throw new SyncSourceException("Error getting the item: " +
                    syncItemExt, e);
        }

        try {
            calendarTwin = this.calendarManager.getCalendarTwin(calendar, this.getType());
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.getSyncItemFromTwin failed");
            throw new SyncSourceException("Error reading items", e);
        }

        if (calendarTwin != null) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Document not Null");
            }

            try {
                syncItem = getSyncItemFromCalendar(calendarTwin,
                        SyncItemState.NEW);
            } catch (EntityException ex) {
                if (log.isLoggable(Level.SEVERE))
                    log.severe("ZimbraCalendarSyncSource.getSyncItemFromTwin failed");
                throw new SyncSourceException("Error converting calendar into SyncItem", ex);
            }

        }

        if (log.isLoggable(Level.FINEST))
            log.finest("ZimbraCalendarSyncSource.getSyncItemFromTwin ended");

        return syncItem;
    }


    /**
     * The same as <i>getSyncItemFromIds()</i>
     *
     * @param principal Principal
     * @param syncItems SyncItem[]
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] getSyncItemsFromTwins(Principal principal,
                                            SyncItem[] syncItems)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getSyncItemsFromTwins");

        ArrayList listItem = new ArrayList();
        SyncItem syncItem;

        int num = syncItems.length;

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Number of items: " + num);
        }

        for (int i = 0; i < num; i++) {
            syncItem = getSyncItemFromTwin(principal, syncItems[i]);
            listItem.add(syncItem);
        }

        SyncItem[] toReturn;
        toReturn = (SyncItem[]) (listItem.toArray(new SyncItem[0]));

        if (log.isLoggable(Level.FINEST))
            log.finest("ZimbraCalendarSyncSource.getSyncItemsFromTwins ended");

        return toReturn;
    }

    /**
     * @param principal Principal
     * @param syncItems SyncItem[]
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] setSyncItems(Principal principal,
                                   SyncItem[] syncItems)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.setSyncItems");

        ArrayList syncItemsInError = new ArrayList();
        ArrayList ret = new ArrayList();
        SyncItem syncItem;

        for (int i = 0, l = syncItems.length; i < l; ++i) {
            try {
                syncItem = setSyncItem(principal, syncItems[i]);
                ret.add(syncItem);
            } catch (SyncSourceException e) {
                syncItemsInError.add(syncItems[i]);
            }
        } // next i

        if (syncItemsInError.size() > 0) {
            throw new SyncSourceException("Error setting the following items: "
                    + syncItemsInError.toString());
        }
        if (log.isLoggable(Level.SEVERE)) {
            log.severe("Zimbra SyncSource " + sourceURI + " set syncItems");
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.setSyncItems");

        return (SyncItem[]) ret.toArray(new SyncItem[]{});

    }


    /**
     * @param principal   Principal
     * @param syncItemExt SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem setSyncItem(Principal principal,
                                SyncItem syncItemExt) throws
            SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.setSyncItem");

        SyncItem syncItem;

        String id;

        Calendar calendarToSet;
        char status;

        id = syncItemExt.getKey().getKeyAsString();
        if (log.isLoggable(Level.FINEST)) {
            Object value = syncItemExt.getPropertyValue(SyncItem.PROPERTY_TIMESTAMP);
            log.finest("TIMESTAMP: " + value.getClass().getName());
        }
        Timestamp time = (Timestamp) syncItemExt.getPropertyValue(SyncItem.PROPERTY_TIMESTAMP);

        try {
            calendarToSet = getFoundationCalendarFromSyncItem(syncItemExt);
        } catch (EntityException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.setSyncItem failed:" + ex);
            throw new SyncSourceException("Error converting item in calendar", ex);
        }

        try {
            //
            // check if there is a mapped key. If this happend,
            // we must do an update
            //
            if (syncItemExt.getMappedKey() != null) {

                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Update item with id: " + id);
                }
                status = SyncItemState.UPDATED;
                this.calendarManager.updateCalendar(calendarToSet, this.getType(), time );

            } else {

                if (log.isLoggable(Level.FINEST)) {
                    log.finest("Insert item with id: " + id);
                }
                status = SyncItemState.NEW;
                calendarToSet = this.calendarManager.insertCalendar(calendarToSet,
                                                                    this.getType(),
                                                                    time );
            }

            // create the output syncitem
            syncItem = getSyncItemFromCalendar(calendarToSet,
                    status);

        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItemExt;
            if (log.isLoggable(Level.SEVERE)) {
                log.severe(errorMessage);
            }
            throw new SyncSourceException(errorMessage, e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.setSyncItem");

        return syncItem;
    }

    /**
     * @param principal Principal
     * @param syncItems SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void removeSyncItems(Principal principal, SyncItem[] syncItems)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.removeSyncItems");

        ArrayList syncItemsInError = new ArrayList();

        for (int i = 0, l = syncItems.length; i < l; ++i) {
            try {
                removeSyncItem(principal, syncItems[i]);
            } catch (SyncSourceException e) {
                syncItemsInError.add(syncItems[i]);
            }
        } // next i

        if (syncItemsInError.size() > 0) {
            String errMsg = "Error deleting the following items: " +
                    syncItemsInError.toString();
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraCalendarSyncSource.removeSyncItems:" + errMsg);
            throw new SyncSourceException(errMsg);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.removeSyncItems");
    }


    /**
     * @param principal Principal
     * @param syncItem  SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void removeSyncItem(Principal principal,
                               SyncItem syncItem)
            throws SyncSourceException {

        String id = syncItem.getKey().getKeyAsString();

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.removeSyncItem for id " + id);

        try {
            if (log.isLoggable(Level.FINEST))
                log.finest("MESSAGE: remove item with UID: " + id);
            this.calendarManager.removeCalendar(id);
        } catch (EntityException e) {
            if (log.isLoggable(Level.WARNING))
                log.warning("ERROR: ZimbraCalendarSyncSource.removeSyncItem for id " + id + ": " + e);
            throw new SyncSourceException(e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.removeSyncItem for id " + id);
    }


    /**
     * @param principal     Principal
     * @param syncItemsKeys SyncItemKey[]
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] getSyncItemsFromIds(Principal principal,
                                          SyncItemKey[] syncItemsKeys)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getSyncItemFromIds");

        ArrayList listItem = new ArrayList();
        SyncItem syncItem;

        int num = syncItemsKeys.length;

        for (int i = 0; i < num; i++) {
            syncItem = getSyncItemFromId(principal, syncItemsKeys[i]);
            listItem.add(syncItem);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getSyncItemFromIds");

        return (SyncItem[]) (listItem.toArray(new SyncItem[0]));

    }


    /**
     * @param principal   Principal
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem getSyncItemFromId(Principal principal,
                                      SyncItemKey syncItemKey)
            throws SyncSourceException {

        SyncItem syncItem;

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getSyncItemFromId");

        Calendar calendar;
        String id;

        id = syncItemKey.getKeyAsString();

        try {
            calendar = this.calendarManager.getCalendarByUID(id, this.getType());
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.getSyncItemFromId failed: " + e);
            throw new SyncSourceException("Error reading items", e);
        }

        try {
            if (calendar != null)
                syncItem = getSyncItemFromCalendar(calendar,
                        SyncItemState.NEW);
            else {
                if (log.isLoggable(Level.FINEST))
                    log.finest("calendar with key " + syncItemKey.getKeyAsString() + " not found");
                syncItem = null;
            }
        } catch (EntityException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.getSyncItemFromId failed: " + ex);
            throw new SyncSourceException("Error converting calendar into SyncItem", ex);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getSyncItemFromId");

        return syncItem;
    }


    /**
     * @param principal Principal
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] getAllSyncItems(Principal principal)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getAllSyncItems");
        SyncItem[] syncItems;
        Calendar[] calendars;
        char status = SyncItemState.UNKNOWN;

        try {
            calendars = this.calendarManager.getAllCalendars(this.getType());
            syncItems = getSyncItemsFromCalendars(calendars, status);
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ZimbraCalendarSyncSource.getAllSyncItems failed: " + e);
	    e.printStackTrace();
            throw new SyncSourceException("Error reading calendars [" +
                    e.getMessage() + "]", e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getAllSyncItems");

        return syncItems;
    }

    /**
     * Get Data from ICal message
     * converting the ical item into a Calendar object
     * <p/>
     * the calendar object is a sync4j.foundation.pdi.event.Calendar
     *
     * @param content String
     * @return Calendar
     * @throws EntityException
     */

    // This is where we have to compensate for devices that incorrectly interpret
    // date fields in vcalendars

    public Calendar getFoundationCalendarFromICal(String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getFoundationCalendarFromICal");

        ByteArrayInputStream buffer;
        ICalendarParser parser;
        sync4j.foundation.pdi.event.Calendar calendar;

        content = ICalParser.remove13(content);

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: iCal to convert:\n"
                    + content);

        try {
            calendar = new sync4j.foundation.pdi.event.Calendar();
            byte[] filteredCotent = ICalParser.remove13(ICalParser.adjustDates(content)).getBytes();
            buffer = new ByteArrayInputStream(filteredCotent);
            if ((content.getBytes()).length > 0) {
                parser = new ICalendarParser(buffer, deviceTimezoneDescr, deviceCharset);
                calendar = parser.iCalendar();
            }
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraCalendarSyncSource.getFoundationCalendarFromICal: " + e);
            throw new EntityException(e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: obtained Calendar:\n" + calendar.toString());
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getFoundationCalendarFromICal");

        /**
         * workaround
         *
         */
        // convert in to XML
        //String xml = this.getXMLFromFoundationCalendar(calendar);

        // get content from XML
        //calendar = this.getFoundationCalendarFromXML(xml);

        return calendar;

    }

    /**
     * Get Data from sync4j.foundation.pdi.event.Calendar
     * converting the Calendar object into an xml item
     *
     * @param calendar Calendar
     * @return String
     * @throws EntityException
     */
    public String getXMLFromFoundationCalendar(Calendar calendar)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getFoundationCalendarFromXML");
        String xml;
        try {
            CalendarToXML c2xml = new CalendarToXML(deviceTimezone, deviceCharset);
            xml = c2xml.convert(calendar);
            log.finest("XML to be sent to device: " + xml);
        } catch (ConverterException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraCalendarSyncSource.getFoundationCalendarFromXML: " + ex);
            throw new EntityException("Error converting calendar in xml", ex);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getFoundationCalendarFromXML");
        return xml;
    }

    /**
     * Get Data from XML message
     * converting the xml item into a Calendar object
     * <p/>
     * the calendar object is a sync4j.foundation.pdi.event.Calendar
     *
     * @param content String
     * @return Calendar
     * @throws EntityException
     */
    public Calendar getFoundationCalendarFromXML(String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getFoundationCalendarFromXML");

        if (log.isLoggable(Level.FINEST))
            log.finest("SIF-E to convert:\n" + content);

        ByteArrayInputStream buffer;
        XMLEventParser parser;
        Calendar calendar;
        try {
            calendar = new sync4j.foundation.pdi.event.Calendar();
            buffer = new ByteArrayInputStream(content.getBytes());
            if ((content.getBytes()).length > 0) {
                parser = new XMLEventParser(buffer);
                calendar = parser.parse();
            }
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraCalendarSyncSource.getFoundationCalendarFromXML: " + e);
            throw new EntityException(e.toString(), e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("Obtained Calendar:\n" + calendar);
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getFoundationCalendarFromXML");
        return calendar;
    }


    // Convert an iCal or XML representation of a calendar  item into
    // the internal format used by sync4j. This method is always
    // called before comparing, inserting, or updating a calendar item
    // on the server. 

    public Calendar getFoundationCalendarFromSyncItem(SyncItem item)
            throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getFoundationCalendarFromSyncItem " +
                    "for item with id: " + item.getKey().getKeyAsString());
        sync4j.foundation.pdi.event.Calendar foundationCalendar;

        String content = Utility.getContentFromSyncItem(item, this.isEncode());

        if (Def.MSG_TYPE_ICAL.equals(this.getType()))
            foundationCalendar = getFoundationCalendarFromICal(content);
        else
            foundationCalendar = getFoundationCalendarFromXML(content);
        foundationCalendar.getEvent().getUid().setPropertyValue(item.getKey().getKeyAsString());
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getFoundationCalendarFromSyncItem " +
                    "for item with id: " + item.getKey().getKeyAsString());

        // - kal -
        // set the organizer to the principal username / (check if it exists ??)
        foundationCalendar.getEvent().getOrganizer().setPropertyValue( getUsername() );
        // - kal -

        return foundationCalendar;
    }

    protected SyncItem getSyncItemFromCalendar(Calendar calendar,
                                               char status)
            throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getSyncItemFromCalendar");

        SyncItem syncItem;

        String content;

        if (Def.MSG_TYPE_ICAL.equals(this.getType())) {
            content = getICalFromFoundationCalendar(calendar);
        } else {
            content = getXMLFromFoundationCalendar(calendar);
        }

        syncItem = new SyncItemImpl(this,
                calendar.getEvent().getUid().getPropertyValue(),
                status);

        if (this.isEncode()) {
            syncItem.setProperty(new SyncProperty(SyncItem.PROPERTY_BINARY_CONTENT,
                    Base64.encode(content.getBytes())));
        } else {
            syncItem.setProperty(new SyncProperty(SyncItem.PROPERTY_BINARY_CONTENT,
                    content.getBytes()));
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getSyncItemFromCalendar");

        return syncItem;

    }


    public SyncItem[] getSyncItemsFromCalendars(Calendar[] calendars, char status)
            throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getSyncItemsFromCalendars");
        int size = calendars.length;
        SyncItem[] items = new SyncItem[size];
        for (int i = 0; i < size; i++) {
            items[i] = getSyncItemFromCalendar(calendars[i], status);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getSyncItemsFromCalendars");
        return items;
    }

    /**
     * Get Data from sync4j.foundation.pdi.event.Calendar
     * converting the Calendar object into a v-card item
     *
     * @param calendar Calendar
     * @return String
     * @throws EntityException
     */
    public String getICalFromFoundationCalendar(Calendar calendar)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getICalFromFoundationCalendar");
        String ical;
        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: Calendar to convert:\n" + calendar);
        try {
            CalendarToIcalendar c2ical = new CalendarToIcalendar(deviceTimezone, deviceCharset);
	    if (calendar.getVersion().getPropertyValue() == null)
	    {
            	if (log.isLoggable(Level.FINEST))
            		log.finest("MESSAGE: Version is Null, so setting it to " + Def.MSG_VCALENDAR_VERSION);
		calendar.setVersion(new Property(Def.MSG_VCALENDAR_VERSION));
	    }
            ical = c2ical.convert(calendar);
        } catch (ConverterException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraCalendarSyncSource.getICalFromFoundationCalendar: " + ex);
            throw new EntityException("Error converting calendar in iCal", ex);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: Obtained iCal:\n" + ical);
        return ical;
    }

    /**
     * Gets all the SyncItemInfo (Key, Lastmodified values) from all the Calendars for the given principal
     *
     * @param principal Principal
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public SyncItemInfo[] getAllSyncItemInfo(Principal principal)
            throws SyncSourceException {
        SyncItemInfo[] itemInfos;

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraCalendarSyncSource.getAllSyncItemInfo");
        try {
            itemInfos = this.calendarManager.getAllSyncItemInfo(principal);
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraCalendarSyncSource.getAllSyncItemInfo: " + e);
            throw new SyncSourceException("Error getting all calendars: " + e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraCalendarSyncSource.getAllSyncItemInfo");
        return itemInfos;
    }

}
