package sync4j.zimbra.engine.source;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.StringReader;
import java.io.Serializable;
import java.security.Principal;
import java.sql.Timestamp;
import java.util.*;
import java.util.logging.Level;

import sync4j.foundation.engine.source.SyncItemInfo;
import sync4j.foundation.pdi.contact.Contact;
import sync4j.foundation.pdi.converter.*;
import sync4j.foundation.pdi.parser.VcardParser;
import sync4j.foundation.pdi.utils.SourceUtils;

import sync4j.framework.engine.*;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceException;
import sync4j.framework.tools.Base64;
import sync4j.framework.tools.beans.BeanInitializationException;
import sync4j.framework.tools.beans.LazyInitBean;
import sync4j.framework.security.Sync4jPrincipal;

import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.items.manager.ContactManager;
import sync4j.zimbra.util.Def;
import sync4j.zimbra.util.Utility;
import sync4j.zimbra.util.UtilityContact;
import sync4j.zimbra.util.ICalParser;
import sync4j.foundation.pdi.parser.XMLContactParser;

/**
 * This class define SyncSource
 * between SyncServer and Zimbra Server contacts items
 */
public final class ZimbraContactSyncSource
        extends ZimbraSyncSource
        implements SyncSource, Serializable, LazyInitBean {

    // ------------------------------------------------------------ Private data

    private ContactManager contactManager = null;

    // ---------------------------------------------------------- Public Methods

    /**
     * @param principal Principal
     * @param syncMode  int
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void beginSync(Principal principal, int syncMode)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.beginSync[" + sourceURI +
                    "] with mode: " + syncMode);

        String credentials;
        credentials = ((Sync4jPrincipal) principal).getEncodedCredentials();
        String username = ((Sync4jPrincipal) principal).getUsername();
        String password = Utility.getPassword(username, credentials);

        HashMap contactManagerProperties = new HashMap();
        contactManagerProperties.put(ContactManager.PROPERTY_NAME_SERVER_URL, getZimbraUrl());
        contactManagerProperties.put(ContactManager.PROPERTY_NAME_USER_NAME, username);
        contactManagerProperties.put(ContactManager.PROPERTY_NAME_PASSWORD, password);

        String deviceId = ((Sync4jPrincipal) principal).getDeviceId();
        log.warning("Device ID: " + deviceId);
        phone = getPhone(deviceId);
        log.warning("Detected Phone: " + phone);

        contactManagerProperties.put(ContactManager.PHONE, phone);

        try {
            contactManager = new ContactManager(contactManagerProperties);

        } catch (EntityException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.beginSync[" + sourceURI + "]: " + ex);
            throw new SyncSourceException("Error creation ContactManager ["
                    + ex.getMessage() + "]", ex);
        }

        super.beginSync(principal, syncMode);

        if (log.isLoggable(Level.FINEST))
            log.finest("END ZimbraContactSyncSource.beginSync[" + sourceURI +
                    "] with mode: " + syncMode);
    }

    /**
     * @param principal Principal
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void endSync(Principal principal) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.endSync[" + sourceURI + "]");
        super.endSync(principal);

        if (contactManager != null) {
            // close Zimbra connection
            try {
                contactManager.releaseSession();
            } catch (EntityException e) {
                if (log.isLoggable(Level.SEVERE))
                    log.severe("ERROR: ZimbraContactSyncSource.endSync[" + sourceURI + "]");
                throw new SyncSourceException(e);
            }
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.endSync[" + sourceURI + "]");
    }

    /**
     * @param principal   Principal
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem getSyncItemFromId(Principal principal,
                                      SyncItemKey syncItemKey) throws
            SyncSourceException {

        SyncItem syncItem;

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getSyncItemFromId[" + syncItemKey.getKeyAsString() + "]");

        Contact contact;
        String id;

        id = syncItemKey.getKeyAsString();

        try {
            contact = this.contactManager.getContactByUID(id, this.getType());
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getSyncItemFromId[" +
                        syncItemKey.getKeyAsString() + "]: " + e.getMessage());
            throw new SyncSourceException("Error reading item " + id + ":" + e.getMessage(), e);
        }

        try {
            if (contact != null)
                syncItem = getSyncItemFromContact(contact,
                        SyncItemState.NEW);
            else {
                if (log.isLoggable(Level.FINEST))
                    log.finest("MESSAGE: contact with key " + syncItemKey.getKeyAsString() + " not found");
                syncItem = null;
            }
        } catch (EntityException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getSyncItemFromId[" +
                        syncItemKey.getKeyAsString() + "]: " + ex.getMessage());
            throw new SyncSourceException("Error converting contact into SyncItem", ex);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getSyncItemFromId[" + syncItemKey.getKeyAsString() + "]");

        return syncItem;
    }

    /**
     * @param principal     Principal
     * @param syncItemsKeys SyncItemKey[]
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] getSyncItemsFromIds(Principal principal,
                                          SyncItemKey[] syncItemsKeys) throws
            SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getSyncItemFromIds");

        ArrayList listItem = new ArrayList();
        SyncItem syncItem;

        int num = syncItemsKeys.length;

        for (int i = 0; i < num; i++) {
            syncItem = getSyncItemFromId(principal, syncItemsKeys[i]);
            listItem.add(syncItem);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getSyncItemFromIds");
        return (SyncItem[]) (listItem.toArray(new SyncItem[0]));

    }

    /**
     * The same asgetSyncItemFromId()
     *
     * @param principal   Principal
     * @param syncItemExt SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem getSyncItemFromTwin(Principal principal,
                                        SyncItem syncItemExt)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getSyncItemFromTwin [" +
                    syncItemExt.getKey().getKeyAsString() + "]");

        SyncItem syncItem = null;

        Contact contact;
        Contact contactTwin;

        // get content of the item
        try {
            contact = getFoundationContactFromSyncItem(syncItemExt);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getSyncItemFromTwin [" +
                        syncItemExt.getKey().getKeyAsString() + "]: " + e);
            throw new SyncSourceException("Error getting the item: " +
                    syncItemExt, e);
        }

        try {
            contactTwin = this.contactManager.getContactTwin(contact, this.getType());
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getSyncItemFromTwin [" +
                        syncItemExt.getKey().getKeyAsString() + "]: " + e);
            throw new SyncSourceException("Error reading items", e);
        }

        if (contactTwin != null) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Document not Null ");
            }

            try {
                syncItem = getSyncItemFromContact(contactTwin,
                        SyncItemState.NEW);
            } catch (EntityException ex) {
                if (log.isLoggable(Level.SEVERE))
                    log.severe("ERROR: ZimbraContactSyncSource.getSyncItemFromTwin [" +
                            syncItemExt.getKey().getKeyAsString() + "]: " + ex);
                throw new SyncSourceException("Error converting contact into SyncItem", ex);
            }

        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getSyncItemFromTwin [" +
                    syncItemExt.getKey().getKeyAsString() + "]");

        return syncItem;

    }


    /**
     * The same as getSyncItemFromIds()
     *
     * @param principal Principal
     * @param syncItems SyncItem[]
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] getSyncItemsFromTwins(Principal principal,
                                            SyncItem[] syncItems)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getSyncItemsFromTwins");

        ArrayList listItem = new ArrayList();
        SyncItem syncItem;

        int num = syncItems.length;

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: Number of items for which to search twins: " + num);

        for (int i = 0; i < num; i++) {
            syncItem = getSyncItemFromTwin(principal, syncItems[i]);
            listItem.add(syncItem);
        }

        SyncItem[] toReturn;
        toReturn = (SyncItem[]) (listItem.toArray(new SyncItem[0]));

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getSyncItemsFromTwins");

        return toReturn;
    }

    /**
     * @param principal Principal
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] getAllSyncItems(Principal principal)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getAllSyncItems");

        SyncItem[] syncItems;
        Contact[] contacts;
        char status = SyncItemState.UNKNOWN;

        try {
            contacts = this.contactManager.getAllContacts(this.getType());
            syncItems = getSyncItemsFromContacts(contacts, status);
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.finest("ERROR: ZimbraContactSyncSource.getAllSyncItems: " + e);
            throw new SyncSourceException("Error reading contacts [" +
                    e.getMessage() + "]", e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getAllSyncItems");

        return syncItems;
    }

    /**
     * @param principal Principal
     * @param syncItems SyncItem[]
     * @return SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem[] setSyncItems(Principal principal,
                                   SyncItem[] syncItems)
            throws SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.setSyncItems");

        ArrayList syncItemsInError = new ArrayList();
        ArrayList ret = new ArrayList();
        SyncItem syncItem;

        for (int i = 0, l = syncItems.length; i < l; ++i) {
            try {
                syncItem = setSyncItem(principal, syncItems[i]);
                ret.add(syncItem);
            } catch (SyncSourceException e) {
                syncItemsInError.add(syncItems[i]);
            }
        } // next i

        if (syncItemsInError.size() > 0) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.setSyncItems");
            throw new SyncSourceException("Error setting the following items: "
                    + syncItemsInError.toString());
        }
        if (log.isLoggable(Level.SEVERE)) {
            log.severe("Zimbra SyncSource " + sourceURI + " set syncItems");
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.setSyncItems");

        return (SyncItem[]) ret.toArray(new SyncItem[]{});

    }


    /**
     * @param principal   Principal
     * @param syncItemExt SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public SyncItem setSyncItem(Principal principal,
                                SyncItem syncItemExt)
            throws SyncSourceException {

        String id = syncItemExt.getKey().getKeyAsString();

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.setSyncItem [" + id + "]");

        SyncItem syncItem;


        Contact contactToSet;
        char status;

        if (log.isLoggable(Level.FINEST)) {
            Object value = syncItemExt.getPropertyValue(SyncItem.PROPERTY_TIMESTAMP);
            log.finest("TIMESTAMP: " + value.getClass().getName());
        }
        Timestamp time = (Timestamp) syncItemExt.getPropertyValue(SyncItem.PROPERTY_TIMESTAMP);

        try {
            contactToSet = getFoundationContactFromSyncItem(syncItemExt);
        } catch (EntityException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.setSyncItem [" + id + "]: " + ex);
            throw new SyncSourceException("Error converting item in contact", ex);
        }

        try {
            //
            // check if there is a mapped key. If this happend,
            // we must do an update
            //
            if (syncItemExt.getMappedKey() != null) {
                if (log.isLoggable(Level.FINEST))
                    log.finest("MESSAGE: Update item with id: " + id);
                status = SyncItemState.UPDATED;
                this.contactManager.updateContact(contactToSet, this.getType(), time);

            } else {

                if (log.isLoggable(Level.FINEST))
                    log.finest("MESSAGE: Insert item with id: " + id);
                status = SyncItemState.NEW;
                contactToSet = this.contactManager.insertContact(contactToSet,
                        this.getType(),
                        time);

            }

            // create the output syncitem
            syncItem = getSyncItemFromContact(contactToSet,
                    status);

        } catch (Exception e) {
            String errorMessage = "Error setting the item: " + syncItemExt;
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.setSyncItem [" + id + "]: " + e);
            throw new SyncSourceException(errorMessage, e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.setSyncItem [" + id + "]");

        return syncItem;

    }

    /**
     * @param principal Principal
     * @param syncItems SyncItem[]
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void removeSyncItems(Principal principal, SyncItem[] syncItems) throws
            SyncSourceException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.removeSyncItems");

        ArrayList syncItemsInError = new ArrayList();

        for (int i = 0, l = syncItems.length; i < l; ++i) {
            try {
                removeSyncItem(principal, syncItems[i]);
            } catch (SyncSourceException e) {
                syncItemsInError.add(syncItems[i]);
            }
        } // next i

        if (syncItemsInError.size() > 0) {
            String errMessage = "Error deleting the following items: " + syncItemsInError.toString();
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.removeSyncItems: " + errMessage);
            throw new SyncSourceException(errMessage);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.removeSyncItems");

    }


    /**
     * @param principal Principal
     * @param syncItem  SyncItem
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void removeSyncItem(Principal principal,
                               SyncItem syncItem)
            throws SyncSourceException {

        String id = syncItem.getKey().getKeyAsString();

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.removeSyncItem[" + id + "]");

        try {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("MESSAGE: remove from server item with UID: " + id);
            }
            this.contactManager.removeContact(id);
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.removeSyncItem[" + id + "]: " + e);
            throw new SyncSourceException("Error reading items", e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.removeSyncItem[" + id + "]");


    }


    // --------------------------------------------------------- Private methods


    /**
     * Get Data from VCard message
     * converting the v-card item into a Contact object
     * the contact object is a sync4j.foundation.pdi.contact.Contact
     *
     * @param content String
     * @return Contact
     * @throws EntityException
     */
    protected Contact getFoundationContactFromVCard(String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getFoundationContactFromVCard");

        sync4j.foundation.pdi.contact.Contact contact;

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: vcard to convert: \n" + content);

        content = ICalParser.remove13(content);
        content = UtilityContact.removeQuotedEncoding(content);

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: UPDATED vcard to convert: \n" + content);


        try {
            contact = new sync4j.foundation.pdi.contact.Contact();
            StringBuffer vCard = new StringBuffer();
            if ((content.getBytes()).length > 0) {
                BufferedReader reader = new BufferedReader(
                                            new StringReader(content));
                String line=reader.readLine();
                while (line!=null){
                    vCard.append(line+"\r\n");
                    line=reader.readLine();
                }
                vCard = new StringBuffer(SourceUtils.handleLineDelimiting(
                                                        vCard.toString()));

                log.finest("Device TimezoneDescr: " + deviceTimezoneDescr);
                log.finest("Device Charset: " + deviceCharset);

                VcardParser parser = new VcardParser(
                         new ByteArrayInputStream(vCard.toString().getBytes()),
                                          deviceTimezoneDescr, deviceCharset);
                contact = parser.vCard();
                if (log.isLoggable(Level.FINEST)) {
                    String message = "MESSAGE: obtained calendar:\n" +
                            contact.getName().getFirstName().getPropertyValue() + " " +
                            contact.getName().getLastName().getPropertyValue() + " ";
                    String[] emails = UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_1_ADDRESS);
                    if (emails != null && emails.length > 0)
                        message += emails[0];
                    log.finest(message);
                }
            }

        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getFoundationContactFromVCard: " + e);
            throw new EntityException(e.getMessage());
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getFoundationContactFromVCard");

        return contact;

    }

    /**
     * Get Data from sync4j.foundation.pdi.contact.Contact
     * converting the Contact object into an xml item
     *
     * @param contact Contact
     * @return String
     * @throws EntityException
     */
    protected String getXMLFromFoundationContact(Contact contact)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getXMLFromFoundationContact");
        String xml;
        try {
            if (log.isLoggable(Level.FINEST)) {
                String message = "MESSAGE: contact to convert:\n" +
                        contact.getName().getFirstName().getPropertyValue() + " " +
                        contact.getName().getLastName().getPropertyValue() + " ";
                String[] emails = UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_1_ADDRESS);
                if (emails != null && emails.length > 0)
                    message += emails[0];
                log.finest(message);
            }
            ContactToXML c2xml = new ContactToXML(deviceTimezone, deviceCharset);
            xml = c2xml.convert(contact);
            if (log.isLoggable(Level.FINEST))
                log.finest("MESSAGE: obtained XML: \n" + xml);
        } catch (ConverterException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getXMLFromFoundationContact: " + ex);
            throw new EntityException("Error converting contact in xml", ex);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getXMLFromFoundationContact");
        return xml;
    }

    /**
     * Get Data from XML message
     * converting the xml item into a Contact object
     * the contact object is a sync4j.foundation.pdi.contact.Contact
     *
     * @param content String
     * @return Contact
     * @throws EntityException
     */
    protected Contact getFoundationContactFromXML(String content)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getFoundationContactFromXML");

        log.finest("MESSAGE: XML to convert: " + content);

        ByteArrayInputStream buffer;
        XMLContactParser parser;
        Contact contact;
        try {
            if (log.isLoggable(Level.FINEST))
                log.finest("MESSAGE: XML to convert:\n" + content);
            contact = null;
            buffer = new ByteArrayInputStream(content.getBytes());
            if ((content.getBytes()).length > 0) {
                parser = new XMLContactParser(buffer);
                contact = parser.parse();
                if (log.isLoggable(Level.FINEST)) {
                    String message = "MESSAGE: obtained contact:\n" +
                            contact.getName().getFirstName().getPropertyValue() + " " +
                            contact.getName().getLastName().getPropertyValue() + " ";
                    String[] emails = UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_1_ADDRESS);
                    if (emails != null && emails.length > 0)
                        message += emails[0];
                    log.finest(message);
                }
            }
        } catch (Exception e) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ERROR: ZimbraContactSyncSource.getFoundationContactFromXML: " + e);
            throw new EntityException(e.toString());
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getFoundationContactFromXML");
        return contact;
    }

    protected sync4j.foundation.pdi.contact.Contact getFoundationContactFromSyncItem(SyncItem item)
            throws EntityException {
        sync4j.foundation.pdi.contact.Contact foundationContact;

        String id = item.getKey().getKeyAsString();

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getFoundationContactFromSyncItem[" + id + "]");

        String content = Utility.getContentFromSyncItem(item, this.isEncode());

        if (Def.MSG_TYPE_VCARD.equals(this.getType())) {
            foundationContact = getFoundationContactFromVCard(content);
        } else {
            foundationContact = getFoundationContactFromXML(content);
        }
        foundationContact.setUid(item.getKey().getKeyAsString());
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getFoundationContactFromSyncItem[" + id + "]");
        return foundationContact;
    }

    protected SyncItem getSyncItemFromContact(Contact contact,
                                              char status)
            throws EntityException {

        SyncItem syncItem;
        String content;
        String id = contact.getUid();
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getSyncItemFromContact[" + id + "]");
        if (Def.MSG_TYPE_VCARD.equals(this.getType())) {
            content = getVCardFromFoundationContact(contact);
        } else {
            content = getXMLFromFoundationContact(contact);
        }

        syncItem = new SyncItemImpl(this,
                contact.getUid(),
                status);

        if (this.isEncode()) {
            syncItem.setProperty(new SyncProperty(SyncItem.PROPERTY_BINARY_CONTENT,
                    Base64.encode(content.getBytes())));
        } else {
            syncItem.setProperty(new SyncProperty(SyncItem.PROPERTY_BINARY_CONTENT,
                    content.getBytes()));
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getSyncItemFromContact[" + id + "]");

        return syncItem;

    }


    protected SyncItem[] getSyncItemsFromContacts(Contact[] contacts, char status)
            throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getSyncItemsFromContacts");
        int size = contacts.length;
        SyncItem[] items = new SyncItem[size];
        for (int i = 0; i < size; i++) {
            items[i] = getSyncItemFromContact(contacts[i], status);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getSyncItemsFromContacts");
        return items;
    }

    /**
     * Get Data from sync4j.foundation.pdi.contact.Contact
     * converting the Contact object into a v-card item
     *
     * @param contact Contact
     * @return String
     * @throws EntityException
     */
    protected String getVCardFromFoundationContact(Contact contact)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getVCardFromFoundationContact");
        String vcard;

        try {
            if (log.isLoggable(Level.FINEST)) {
                String message = "MESSAGE: contact to convert:\n" +
                        contact.getName().getFirstName().getPropertyValue() + " " +
                        contact.getName().getLastName().getPropertyValue() + " ";
                String[] emails = UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_1_ADDRESS);
                if (emails != null && emails.length > 0)
                    message += emails[0];
                log.finest(message);
            }
            ContactToVcard c2vcard = new ContactToVcard(deviceTimezone, deviceCharset);
            vcard = c2vcard.convert(contact);
            log.finest("MESSAGE: obtained vCard:\n" + vcard);
        } catch (ConverterException ex) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ZimbraContactSyncSource.getVCardFromFoundationContact: " + ex);
            throw new EntityException("Error converting contact in vCard", ex);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getVCardFromFoundationContact");
        return vcard;
    }

    /**
     * Gets all the SyncItemInfo (Key, Lastmodified values) from all the Contacts for the given principal
     *
     * @param principal Principal
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public SyncItemInfo[] getAllSyncItemInfo(Principal principal)
            throws SyncSourceException {

        SyncItemInfo[] itemInfos;

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ZimbraContactSyncSource.getAllSyncItemInfo");
        try {
            itemInfos = contactManager.getAllSyncItemInfo(principal);
        } catch (EntityException e) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ERROR: ZimbraContactSyncSource.getAllSyncItemInfo: " + e);
            throw new SyncSourceException(e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ZimbraContactSyncSource.getAllSyncItemInfo");
        return itemInfos;
    }

}

