package sync4j.zimbra.engine.source;

import sync4j.foundation.engine.source.CachedSyncSource;
import sync4j.framework.engine.SyncItem;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceException;
import sync4j.framework.security.Sync4jPrincipal;
import sync4j.framework.server.Sync4jDevice;
import sync4j.framework.server.store.PersistentStore;
import sync4j.framework.server.store.PersistentStoreException;
import sync4j.framework.tools.beans.BeanInitializationException;
import sync4j.framework.tools.beans.LazyInitBean;
import sync4j.server.config.Configuration;
import sync4j.zimbra.util.Def;
import sync4j.zimbra.util.Utility;

import java.io.Serializable;
import java.security.Principal;
import java.util.TimeZone;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Represents a generic Zimbra syncsource.
 */
public abstract class ZimbraSyncSource extends CachedSyncSource implements
        SyncSource, Serializable, LazyInitBean {

    // --------------------------------------------------------------- Constants
    private static final String DEFAULT_ZIMBRA_APPLICATION_NAME = "ZimbraWap";
    private static final String DEFAULT_ZIMBRA_SERVER_VERSION = ".1";

    // ---------------------------------------------------------- Protected data

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    protected Sync4jDevice device = null;
    protected String deviceTimezoneDescr = null;
    protected TimeZone deviceTimezone = null;
    protected String deviceCharset = null;
    protected String phone = null;


    /**
     * User to synchronize
     */
    private String username = null;

    /**
     * Getter for property username.
     *
     * @return the username.
     */
    public String getUsername() {
        return username;
    }

    /**
     * User's password
     */
    private String password = null;

    /**
     * Getter for property password.
     *
     * @return the password.
     */
    public String getPassword() {
        return password;
    }

    /**
     * Zimbra Server Host
     */
    private String zimbraUrl = null;

    /**
     * Getter for property Zimbra Server Host.
     *
     * @return the Zimbra Server Host.
     */
    public String getZimbraUrl() {
        return zimbraUrl;
    }

    /**
     * Setter for property Zimbra Server Host.
     *
     * @param zimbraUrl new value of property Zimbra Server Host.
     */
    public void setZimbraUrl(String zimbraUrl) {
        this.zimbraUrl = zimbraUrl;
    }

    /**
     * Zimbra Server version
     */
    private String version = DEFAULT_ZIMBRA_SERVER_VERSION;

    /**
     * Getter for property Zimbra Server version.
     *
     * @return the Zimbra Server version.
     */
    public String getVersion() {
        return this.version;
    }

    /**
     * Setter for property Zimbra Server version.
     *
     * @param newVersion new value of property Zimbra Server version.
     */
    public void setVersion(String newVersion) {
        this.version = newVersion;
    }

    /**
     * Zimbra application name
     */
    private String applicationName = DEFAULT_ZIMBRA_APPLICATION_NAME;

    /**
     * Getter for property Zimbra application name.
     *
     * @return the Zimbra application name.
     */
    public String getApplicationName() {
        return this.applicationName;
    }

    /**
     * Setter for property Zimbra application name.
     *
     * @param newApplicationName new value of property Zimbra application name.
     */
    public void setApplicationName(String newApplicationName) {
        this.applicationName = newApplicationName;
    }

    /**
     * Should the content be encoded?
     */
    private boolean encode = true;

    /**
     * Getter for property encode.
     *
     * @return the encode property.
     */
    public boolean isEncode() {
        return encode;
    }

    /**
     * Setter for property encode.
     *
     * @param newEncode new value of property encode.
     */
    public void setEncode(boolean newEncode) {
        this.encode = newEncode;
    }

    protected SyncItem[] newSyncItems;
    protected SyncItem[] updatedSyncItems;
    protected SyncItem[] deletedSyncItems;

    // ---------------------------------------------------------- Public Methods

    /**
     * @param principal the principal to sync
     * @param syncMode  the sync mode
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void beginSync(Principal principal, int syncMode)
            throws SyncSourceException {

        String credentials;

        if (log.isLoggable(Level.FINEST))
            log.finest("ZimbraSyncSource.beginSync started");

        credentials = ((Sync4jPrincipal) principal).getEncodedCredentials();

        this.username = ((Sync4jPrincipal) principal).getUsername();
        this.password = Utility.getPassword(username, credentials);

        super.beginSync(principal, syncMode);

        String deviceId = null;
        try {
            deviceId = ((Sync4jPrincipal) principal).getDeviceId();
            device = getDevice(deviceId);
            String timezone = device.getTimeZone();
            if (device.getConvertDate()) {
                if (timezone != null && timezone.length() > 0) {
                    deviceTimezoneDescr = timezone;
                    deviceTimezone = TimeZone.getTimeZone(deviceTimezoneDescr);
                }
            }

            deviceCharset = device.getCharset();
        } catch (Exception ex) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ZimbraSyncSource.beginSync failed: " + ex);
            throw new SyncSourceException("Error reading device '" + deviceId + "'", ex);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("ZimbraSyncSource.beginSync ended");

    }

    /**
     * @param principal Principal
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void endSync(Principal principal) throws SyncSourceException {
        super.endSync(principal);
    }


    // ---------------------------------------------------------- Private Methods

    /**
     * Return the device with the given deviceId
     *
     * @param deviceId String
     * @return Sync4jDevice
     * @throws PersistentStoreException
     */
    private Sync4jDevice getDevice(String deviceId)
            throws PersistentStoreException {
        Sync4jDevice device = new Sync4jDevice(deviceId);
        PersistentStore store = Configuration.getConfiguration().getStore();

        store.read(device);
        return device;
    }

    //determines the phone type
    protected String getPhone(String deviceId) {

        if (deviceId.indexOf(Def.BLACKBERRY_DEVICE) != -1) return Def.BLACKBERRY;
        if (deviceId.indexOf(Def.PPC_DEVICE) != -1) return Def.POCKET_PC;

	// Funambol Agent on a Windows Mobile device
	// ..not sure if this actually behaves the same as a standard pocket pc, so
	// removing for now..
//	if (deviceId.trim().startsWith("fwm-")) return Def.POCKET_PC;

        boolean ok;

        String tac = "";
        for (int i = 0; i < deviceId.length(); i++) {
            ok = true;
            try {
                Byte.parseByte(deviceId.substring(i, i + 1));
            } catch (NumberFormatException e) {
                ok = false;
            }
            if (ok) {
                tac = deviceId.substring(i, i + 6);
                break;
            }
        }

        if (tac.length() > 0) {
            if (Def.NOKIA_6600_TACS.indexOf(tac) != -1)
                return Def.NOKIA_6600;
            else if (Def.SE_P900_TACS.indexOf(tac) != -1)
                return Def.SE_P900;
            else if (Def.MOTOROLA_RAZR_TACS.indexOf(tac) != -1)
                return Def.MOTOROLA_RAZR;
            else if (Def.MOTOROLA_V400_TACS.indexOf(tac) != -1) return Def.MOTOROLA_V400;
        }

        if (deviceId.trim().startsWith("IMEI")) return Def.NOKIA_6600;

        if (deviceId.indexOf("-") != -1)
            try {
                Long.parseLong(deviceId.trim().replaceAll("-", ""));
                return Def.SE_P900;
            } catch (NumberFormatException e) {
                log.warning("Not a SE phone ...");
            }

        try {
            Long.parseLong(deviceId.trim());
            return Def.MOTOROLA_RAZR;
        } catch (Exception e) {
            log.warning("Not motorola ...");
        }

        return Def.UNKNOWN_PHONE;
    }
}

