package sync4j.zimbra.items.dao;

import java.sql.Timestamp;
import java.util.logging.Level;
import java.util.HashMap;
import java.util.LinkedList;

import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.util.*;

import java.text.MessageFormat;
import java.security.Principal;

import sync4j.zimbra.items.model.MeetingProperties;
import sync4j.foundation.engine.source.SyncItemInfo;
import zimbra.soap.*;
import zimbra.soap.util.SyncDetails;
import zimbra.soap.util.CalendarDetails;
import zimbra.soap.exceptions.SOAPException;

public class CalendarDAO extends EntityDAO {

    //------------------------------------------------------------- Constructors
    /**
     * Creates a new DAO. The constractor is private because a CalendarDAO requires
     * always a ZimbraWrapper.
     */
    protected CalendarDAO() {
    }

    /**
     * Creates a new DAO using the given <code>ZimbraWrapper</code>.
     *
     * @param zimbraWrapperEx ZimbraWrapper
     */
    public CalendarDAO(ZimbraWrapper zimbraWrapperEx) {
        super(zimbraWrapperEx);

    }

    //----------------------------------------------------------- Public methods

    /**
     * Gets all meetings.
     *
     * @return MeetingProperties[]
     * @throws EntityException
     */

    // TODO: This method should cache results rather than hitting Zimbra server
    // every time.
    public MeetingProperties[] getAllMeetings() throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getAllMeetings started");

        MeetingProperties[] allMeetings;

        try {
            Object[] ids =
                    this.zimbraWrapper.getSOAPTransporter().sendRequest(SOAPTransporter.M_GET_ALL_CALENDARS_IDS,
                            new GetAllCalendarsIdsResponseHandler());
            allMeetings = new MeetingProperties[ids.length];
            for (int i = 0; i < allMeetings.length; i++)
                allMeetings[i] = this.getMeetingsByUID(ids[i].toString());

        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarDAO.getAllMeetings failed [" + e.getMessage() + "]");
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getAllMeetings ended");

        return allMeetings;
    }

    /**
     * Gets the meeting content with the given UID from Zimbra server.
     *
     * @param UID the uid to search
     * @return the meeting with the given uid. <code>null</code> if the meeting
     *         doesn't exit.
     * @throws EntityException
     */
    public MeetingProperties getMeetingsByUID(String UID) throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getMeetingsByUID started");

        MeetingProperties calendarProperties;

        try {
            Object[] meetings =
                    this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_GET_CALENDAR_BY_ID,
                            new Object[]{UID}),
                            new GetAppointementResponseHandler());

            if (log.isLoggable(Level.FINEST)) {
                log.finest("Meeting with ID '" + UID + "' found");
            }

            HashMap map = (HashMap) meetings[0];
            calendarProperties = new MeetingProperties(map);

            String inviteId = (String) map.get(CalendarDetails.K_DEFUALT_INV_ID);

            log.finest("ID: " + UID);
            log.finest("Invite ID: " + inviteId);

            meetings =
                    this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_GET_MSG_REQUEST,
                            new Object[]{UID, inviteId}),
                            new MessageResponseHandler());

            map = (HashMap) meetings[0];

            log.finest("Start time:" + map.get(CalendarDetails.K_START_DATETIME));
            log.finest("End time:" + map.get(CalendarDetails.K_END_DATETIME));
            log.finest("Timezone:" + map.get(CalendarDetails.K_TIMEZONE));

            calendarProperties.put(CalendarDetails.K_START_DATETIME, map.get(CalendarDetails.K_START_DATETIME));
            calendarProperties.put(CalendarDetails.K_END_DATETIME, map.get(CalendarDetails.K_END_DATETIME));

        } catch (Exception e) {
            if (log.isLoggable(Level.WARNING)) {
                log.warning("MESSAGE: Meeting with ID '" + UID + "' NOT found");
            }
            calendarProperties = null;
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getMeetingsByUID ended");
        return calendarProperties;

    }


    /**
     * Removes the meeting with the given uid from Zimbra server
     *
     * @param UID String
     * @throws EntityException
     */
    public void removeMeeting(String UID) throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.removeMeeting started for id " + UID);

        try {
            this.zimbraWrapper.getSOAPTransporter().deleteItem(UID);
        } catch (SOAPException e) {
            if (log.isLoggable(Level.WARNING))
                log.warning("ERROR: CalendarDAO.removeMeeting failed for id " + UID);
            throw new EntityException("Failed to delete calendar with id: " + UID);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.removeMeeting ended for id " + UID);

    }


    /**
     * Updates the given (existing) meeting on the Zimbra server.
     *
     * @param meeting the meeting to update
     * @throws EntityException
     */
    public void updateMeeting(MeetingProperties meeting)
            throws EntityException {
        String id = meeting.get(MeetingProperties.K_ID).toString();

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.updateMeeting started for id " + id);

        try {
            CalendarDetails calendarDetails = new CalendarDetails(meeting.getHashMap());
            if (calendarDetails.get(CalendarDetails.K_ORGANIZER) == null) {
                HashMap organizer = new HashMap();
                organizer.put(CalendarDetails.K_ADDRESS,
                        this.zimbraWrapper.getZimbraMailAccount());
                organizer.put(CalendarDetails.K_FRIENDLY_NAME,
                        this.zimbraWrapper.getZimbraMailAccount());
                calendarDetails.put(CalendarDetails.K_ORGANIZER, organizer);
            }
            String newId = this.zimbraWrapper.getSOAPTransporter().updateCalendar(calendarDetails, zimbraWrapper.getUserID());
            if (meeting.getHashMap().containsKey(MeetingProperties.K_ID))
                meeting.getHashMap().remove(MeetingProperties.K_ID);
            meeting.getHashMap().put(MeetingProperties.K_ID, newId);
        } catch (Exception e) {
            if (log.isLoggable(Level.WARNING))
                if (log.isLoggable(Level.SEVERE))
                    log.severe("CalendarDAO.insertMeeting failed for " + meeting.getHashMap() + " error: " + e);
            throw new EntityException("Failed to update calendar with id: " + id);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.updateMeeting ended for id " + id);
    }


    /**
     * Insert the given meeting. Returns the same meeting with the id returned by
     * Zimbra for the new meeting.
     *
     * @param meeting the meeting to insert
     * @return the given meeting with the id returned by Zimbra for the new meeting.
     * @throws EntityException
     */
    public MeetingProperties insertMeeting(MeetingProperties meeting)
            throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.insertMeeting started");
        try {
            CalendarDetails calendarDetails = new CalendarDetails(meeting.getHashMap());
            if (calendarDetails.get(CalendarDetails.K_ORGANIZER) == null) {
                HashMap organizer = new HashMap();
                organizer.put(CalendarDetails.K_ADDRESS,
                        this.zimbraWrapper.getZimbraMailAccount());
                organizer.put(CalendarDetails.K_FRIENDLY_NAME,
                        this.zimbraWrapper.getZimbraMailAccount());
                calendarDetails.put(CalendarDetails.K_ORGANIZER, organizer);
            }
            String newId = this.zimbraWrapper.getSOAPTransporter().createCalendar(calendarDetails);
            if (meeting.getHashMap().containsKey(MeetingProperties.K_ID))
                meeting.getHashMap().remove(MeetingProperties.K_ID);
            meeting.getHashMap().put(MeetingProperties.K_ID, newId);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarDAO.insertMeeting failed for " + meeting.getHashMap() + " error: " + e);
            throw new EntityException(e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.insertMeeting ended");
        return meeting;
    }


    /**
     * Gets the meetings with the given values for the given fields (searches for meetings).
     *
     * @param fields the fields to check
     * @param values the values for the fields
     * @return MeetingProperties[]
     * @throws EntityException
     */
    public MeetingProperties[] getCalendars(String[] fields,
                                            String[] values,
                                            int maxResults)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getCalendars started");

        LinkedList meetingsList = new LinkedList();
	// Fetch all meetings first
        MeetingProperties[] allMeetings = this.getAllMeetings();

	// Iterate through each meeting
        for (int i = 0; i < allMeetings.length; i++) {
            int j;
	    // And for each meeting, iterate through each field to compare
            for (j = 0; j < fields.length; j++) {
                if (fields[j].equals(MeetingProperties.K_DATETIME_START) ||
                        fields[j].equals(MeetingProperties.K_DATETIME_END)) {
                    Object allDay = allMeetings[i].get(MeetingProperties.K_ALL_DAY);

                    String dateInGMT = (String) allMeetings[i].get(fields[j]);
                    if (dateInGMT == null) break;
                    if (allDay != null && "1".equals(allDay))
                        dateInGMT = dateInGMT.substring(0, 8);
                    else if (!dateInGMT.endsWith("Z"))
                        try {
                            dateInGMT =
                                    UtilityCalendar.covertLocalDateToGMT(dateInGMT,
                                            (String) allMeetings[i].get(MeetingProperties.K_TIMEZONE));
                        } catch (Exception e) {
                            if (log.isLoggable(Level.WARNING))
                                log.warning("ERROR: CalendarDAO.getCalendars: " + e);
                            throw new EntityException(e);
                        }

                    if (!dateInGMT.equals(values[j])) break;
                    continue;
                }
                if (values[j] != null &&
                        !allMeetings[i].get(fields[j]).equals(values[j]))
                    break;
            }
            if (j == fields.length) meetingsList.add(allMeetings[i]);
        }

        MeetingProperties[] meetings = (MeetingProperties[])
                meetingsList.toArray(new MeetingProperties[0]);

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getCalendars found " + meetings.length + " matches");

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getCalendars ended");

        return meetings;
    }

    /**
     * Returns all the SyncItemInfo (Keys, LastModified values) for the given principal
     * Do this by sending Zimbra a SOAP 'SYNC' request, which fetches all objects and their last-change time
     *
     * @param principal
     * @return all the SyncItemInfo (Keys, LastModified values)
     */
    public SyncItemInfo[] getAllSyncItemInfo(Principal principal)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getAllSyncItemInfo started");

        SyncItemInfo[] toReturn;

        try {
            HashMap[] outcome = (HashMap[])
                    this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_SYNC,
                            new Object[]{"1"}),
                            new SyncResponseHandler());

            Object[] idsAndMds = (Object[]) outcome[0].get(SyncDetails.K_CALENDARS);
            toReturn = new SyncItemInfo[idsAndMds.length];
            for (int i = 0; i < toReturn.length; i++) {
                HashMap idAndMd = (HashMap) idsAndMds[i];
                toReturn[i] = new SyncItemInfo(idAndMd.get(CalendarDetails.K_ID).toString(),
                        new Timestamp(1000 * Long.parseLong(idAndMd.get(CalendarDetails.K_MODIFICATION_DATE).
                        toString())));
            }
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarDAO.getAllSyncItemInfo failed");
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getAllSyncItemInfo found " + toReturn.length + " calendars");

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarDAO.getAllSyncItemInfo ended");

        return toReturn;
    }

}

