package sync4j.zimbra.items.dao;


import java.sql.Timestamp;
import java.util.*;
import java.util.logging.Level;


import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.items.model.ContactProperties;
import sync4j.zimbra.util.*;
import sync4j.foundation.engine.source.SyncItemInfo;

import java.text.MessageFormat;
import java.security.Principal;

import zimbra.soap.*;
import zimbra.soap.exceptions.SOAPException;
import zimbra.soap.util.AccountInfoDetails;
import zimbra.soap.util.ContactDetails;
import zimbra.soap.util.SyncDetails;

/**
 * This class implements methods to access contacts data
 * in Zimbra server datastore
 */
public class ContactDAO extends EntityDAO {
    //------------------------------------------------------------- Constructors
    /**
     * Creates a new DAO. The constractor is private because a ContactDAO requires
     * always a ZimbraWrapper.
     */
    protected ContactDAO() {
    }

    /**
     * Creates a new DAO using the given <code>ZimbraWrapper</code>.
     *
     * @param zimbraWrapperEx ZimbraWrapper
     */
    public ContactDAO(ZimbraWrapper zimbraWrapperEx) {
        super(zimbraWrapperEx);

    }

    //----------------------------------------------------------- Public methods


    /**
     * Gets all contacts.
     *
     * @return ContactProperties[]
     * @throws EntityException
     */
    public ContactProperties[] getAllContacts() throws EntityException {
        log.finest("START: ContactDAO.getAllContacts");
        ContactProperties[] allContacts;
        List allWithoutTrash = new LinkedList();
        try {
            allContacts = getContactByClause(SOAPTransporter.M_GET_ALL_CONTACTS);
            log.finest("ContactDAO.getAllContacts found " + allContacts.length + " contacts");
            String trashId;
            try {
                trashId = zimbraWrapper.getSOAPTransporter().getTrashFolderId();
            } catch (SOAPException e) {
                trashId = null;
            }
            if (trashId != null)
                for (int i = 0; i < allContacts.length; i++)
                    if (!trashId.equals(allContacts[i].getHashMap().
                            get(ContactDetails.K_FOLDER)))
                        allWithoutTrash.add(allContacts[i]);
        } catch (Exception e) {
            log.severe("ERROR: ContactDAO.getAllContacts: " + e);
            throw new EntityException(e);
        }

        log.finest("MESSAGE: ContactManager.getAllContacts found " + allWithoutTrash.size());

        log.finest("END: ContactManager.getAllContacts");

        return (ContactProperties[]) allWithoutTrash.toArray(new ContactProperties[0]);
    }

    /**
     * Gets the contacts with the given UID.
     *
     * @param UID the uid to search
     * @return the contact with the given uid. <code>null</code> if the contact
     *         doesn't exit.
     * @throws EntityException
     */
    public ContactProperties getContactByUID(String UID) throws EntityException {

        if (log.isLoggable(Level.ALL)) {
            log.finest("Calling ContactDAO.getContactByUID");
        }

        String clause = MessageFormat.format(SOAPTransporter.M_GET_CONTACT_BY_ID,
                new Object[]{UID});

        ContactProperties[] contacts = getContactByClause(clause);

        int numResults = contacts.length;

        if (numResults > 0) {
            if (log.isLoggable(Level.ALL)) {
                log.finest("found Contact with ID: " + UID);
            }
        } else {
            if (log.isLoggable(Level.ALL)) {
                log.finest("Contact with ID '" + UID + "' NOT found");
            }
            return null;
        }
        return contacts[0];
    }


    /**
     * Removes the contact with the given uid
     *
     * @param UID String
     * @throws EntityException
     */
    public void removeContact(String UID)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactDAO.removeContact started");
        try {
            String trashId;
            try {
                trashId = zimbraWrapper.getSOAPTransporter().getTrashFolderId();
            } catch (SOAPException e) {
                trashId = null;
            }
            if (trashId != null) {
                this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_ITEM_MOVE,
                        new Object[]{UID, trashId}), new ItemActionResponseHandler());
            } else {
                this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_CONTACT_ACTION,
                        new Object[]{"delete", UID}), new ContactActionResponseHandler());
            }
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.finest("ContactDAO.removeContact failed to remove" +
                        "contact with UID " + UID + ": " + e);
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: ContactDAO.removeContact removed contact with UID " + UID);

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactDAO.removeContact ended");

    }


    /**
     * Updates the given contact.
     *
     * @param contact the contact to update
     * @throws EntityException
     */
    public void updateContact(ContactProperties contact) throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactDAO.updateContact started with phone: " + this.zimbraWrapper.getPhone());
        try {
            ContactDetails contactDetails = new ContactDetails(contact.getHashMap());
            this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_MODIFY_CONTACT,
                    new Object[]{
                        contact.getUniversalID(),
                        contactDetails.getCreateClause(this.zimbraWrapper.getPhone())
                    }),
                    new ModifyContactResponseHandler());

        } catch (Exception e) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ContactDAO.updateContact failed");
            throw new EntityException(e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("ContactDAO.updateContact ended");


    }

    /**
     * Insert the given contact. Returns the same contact with the id returned by
     * Zimbra for the new contact.
     *
     * @param contact the contact to insert
     * @return the given contact with the id returned by Zimbra for the new contact.
     * @throws EntityException
     */
    public ContactProperties insertContact(ContactProperties contact)
            throws EntityException {

        if (log.isLoggable(Level.ALL)) {
            log.finest("Calling ContactDAO.insertContact");
        }

        ContactDetails contactDetails = new ContactDetails(contact.getHashMap());
        try {
            Object[] outcome =
                    this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_CREATE_CONTACT,
                            new Object[]{contactDetails.getCreateClause(this.zimbraWrapper.getPhone())}),
                            new CreateContactResponseHandler());
            String newId = (String) ((HashMap) outcome[0]).get(ContactDetails.K_ID);
            contact.remove(ContactProperties.K_UID);
            contact.put(ContactProperties.K_UID, newId);
            return contact;
        } catch (Exception e) {
            e.printStackTrace();
            throw new EntityException(e);
        }

    }


    /**
     * Gets the contacts with the given values for the given fields.
     *
     * @param fields the fields to check
     * @param values the values for the fields
     * @return ContactProperties[]
     * @throws EntityException
     */
    public ContactProperties[] getContacts(String[] fields,
                                           String[] values)
            throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ContactDAO.getContacts");
        ContactProperties[] allContacts = this.getAllContacts();
        LinkedList contactsList = new LinkedList();
        for (int i = 0; i < allContacts.length; i++) {
            int j;
            for (j = 0; j < fields.length; j++) {
                Object obj = allContacts[i].get(fields[j]);
                if (obj == null && values[j] != null) break;
                if (obj != null && !obj.equals(values[j])) break;

            }
            if (j == fields.length) contactsList.add(allContacts[i]);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ContactDAO.getContacts");

        return (ContactProperties[]) contactsList.toArray(new ContactProperties[0]);
    }


    /**
     * Returns a String array  where the first element is the account id
     * and the second is the real account name.
     *
     * @param accountName String
     * @return String[]
     */
    public String[] getAccountIDByName(String accountName) throws
            EntityException {

        if (log.isLoggable(Level.ALL)) {
            log.finest("Calling ContactDAO.getAccountIDByName");
            log.finest("Account Name: " + accountName);
        }

        SOAPTransporter soapTrans = this.zimbraWrapper.getSOAPTransporter();
        try {
            Object[] accountInfo =
                    soapTrans.sendRequest(MessageFormat.format(SOAPTransporter.M_GET_ACCOUNT_INFO,
                            new Object[]{accountName}),
                            new AccountInfoResponseHandler());
            String[] toReturn = new String[2];
            toReturn[0] = (String) ((HashMap) accountInfo[0]).get(AccountInfoDetails.K_ZIMBRA_ID);
            toReturn[1] = (String) ((HashMap) accountInfo[0]).get(AccountInfoDetails.K_ZIMBRA_MAIL_HOST);
            return toReturn;
        } catch (Exception e) {
            throw new EntityException(e);
        }
    }

    /**
     * Returns the contacts using the given clause.
     *
     * @param clause String
     * @return ContactProperties[]
     * @throws EntityException
     */
    private ContactProperties[] getContactByClause(String clause)
            throws EntityException {

        ResponseHandler rh = new GetContactsResponseHandler();

        try {
            this.zimbraWrapper.getSOAPTransporter().sendRequest(clause,
                    rh);
        } catch (Exception e) {
            throw new EntityException(e);
        }

        Object[] entries = rh.getOutcome();

        ContactProperties[] cps = new ContactProperties[entries.length];

        for (int i = 0; i < entries.length; i++) {
            cps[i] = new ContactProperties((HashMap) entries[i]);
        }

        return cps;
    }


    /**
     * Returns all the SyncItemInfo (Keys, LastModified values) for the given principal
     *
     * @param principal
     * @return all the SyncItemInfo (Keys, LastModified values)
     */
    public SyncItemInfo[] getAllSyncItemInfo(Principal principal)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactDAO.getAllSyncItemInfo started");

        List toReturn = new LinkedList();

        try {
            HashMap[] outcome = (HashMap[])
                    this.zimbraWrapper.getSOAPTransporter().sendRequest(MessageFormat.format(SOAPTransporter.M_SYNC,
                            new Object[]{"1"}),
                            new SyncResponseHandler());

            Object[] itemIdsMdsFlds = (Object[]) outcome[0].get(SyncDetails.K_CONTACTS);

            String trashId;
            try {
                trashId = zimbraWrapper.getSOAPTransporter().getTrashFolderId();
            } catch (SOAPException e) {
                trashId = null;
            }

            for (int i = 0; i < itemIdsMdsFlds.length; i++) {
                HashMap idMdAndFld = (HashMap) itemIdsMdsFlds[i];
                if (trashId != null && !trashId.equals(idMdAndFld.get(ContactDetails.K_FOLDER)))
                    toReturn.add(new SyncItemInfo(idMdAndFld.get(ContactDetails.K_ID).toString(),
                            new Timestamp(1000 * Long.parseLong(idMdAndFld.get(ContactDetails.K_MODIFICATION_DATE).
                            toString()))));

            }
        } catch (Exception e) {
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactDAO.getAllSyncItemInfo found " + toReturn.size() + " items");

        return (SyncItemInfo[]) toReturn.toArray(new SyncItemInfo[0]);
    }

}