package sync4j.zimbra.items.manager;

import java.sql.Timestamp;
import java.util.*;
import java.util.logging.Level;
import java.security.Principal;

import sync4j.foundation.pdi.utils.TimeUtils;

import sync4j.foundation.pdi.event.Calendar;
import sync4j.foundation.pdi.event.Event;
import sync4j.foundation.pdi.event.RecurrencePattern;
import sync4j.foundation.pdi.common.XTag;
import sync4j.foundation.engine.source.SyncItemInfo;

import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.items.model.MeetingProperties;
import sync4j.zimbra.items.dao.CalendarDAO;
import sync4j.zimbra.util.UtilityCalendar;
import sync4j.zimbra.util.Utility;
import sync4j.zimbra.util.RecurrencePatternExt;
import sync4j.zimbra.util.Def;
import zimbra.soap.util.RecurrenceDetails;
import zimbra.soap.util.RRuleParser;
import zimbra.soap.util.CalendarDetails;
import zimbra.soap.exceptions.MalformedRRuleException;
import org.apache.commons.lang.StringUtils;


/**
 * Represents the class used by the <code>ZimbraCalendarSyncSource</code> to handle
 * the calendars. It uses <code>CalendarDAO</code> to access to Zimbra data (It works as
 * wrapper between the <code>ZimbraCalendarSyncSource</code> and the <code>CalendarDAO</code>).
 * <br>All methods invoked by the syncsource return/work with Calendar (foundationCalendar)
 * <br>All methods invoked on the DAO return/work with MeetingProperties.
 */
public class CalendarManager extends EntityManager {

    // ------------------------------------------------------------ Private data

    /**
     * The DAO to access to Zimbra meetings
     */
    private CalendarDAO calendarDao = null;

    //-------------------------------------------------------------- Costructors

    /**
     * Creates a new CalendarManager using the given properties
     *
     * @throws EntityException
     */
    public CalendarManager(HashMap properties) throws EntityException {
        super(properties);
        calendarDao = new CalendarDAO(zimbraWrapper);
    }

    private String deviceTimezoneDescr = null;

    public void setDeviceTimezoneDescr( String value )
    {
        deviceTimezoneDescr = value;
    }


    //----------------------------------------------------------- Public methods

    /**
     * Returns all meetings.
     *
     * @return Calendar[]
     * @throws EntityException
     */
    public Calendar[] getAllCalendars(String itemFormat) throws EntityException {

        MeetingProperties[] allCalendarsProperties;
        Calendar[] allCalendars;
        try {
            allCalendarsProperties = calendarDao.getAllMeetings();
            allCalendars = getCalendarsFromMeetingsProperties(allCalendarsProperties, itemFormat);
        } catch (Exception e) {
            throw new EntityException(e);
        }
        return allCalendars;
    }

    /**
     * Returns the Calendar with the given id. <code>null</code> if the calendar isn't found.
     *
     * @param UID String
     * @return Calendar
     * @throws EntityException
     */
    public Calendar getCalendarByUID(String UID, String itemFormat) throws EntityException {

        MeetingProperties doc;
        Calendar foundationCalendar;

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.getCalendarByUID started for id " + UID);

        try {
            doc = calendarDao.getMeetingsByUID(UID);
            if (doc == null) {
                return null;
            }
            foundationCalendar = getCalendarFromMeetingProperties(doc);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarManager.getCalendarByUID failed for id " + UID +
                        " with exception: " + e);
            throw new EntityException(e.toString(), e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.getCalendarByUID ended for id " + UID);

        return foundationCalendar;
    }


    /**
     * Updates the calendar defined by the given foundationCalendar.
     *
     * @param foundationCalendar Calendar
     * @param itemFormat         String
     * @throws EntityException
     */
    public void updateCalendar(Calendar foundationCalendar,
                               String itemFormat,
                               Timestamp time ) throws EntityException {

        MeetingProperties meetingOnServer;

        MeetingProperties doc = getMeetingPropertiesFromCalendar(foundationCalendar, itemFormat );

        String id = doc.get(MeetingProperties.K_ID).toString();

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.updateCalendar started for id " + id);

        try {
            meetingOnServer = calendarDao.getMeetingsByUID(foundationCalendar.getEvent().getUid().getPropertyValueAsString());
            HashMap[] attendees = (HashMap[]) meetingOnServer.getHashMap().get(CalendarDetails.K_ATTENDEES);

            doc.getHashMap().put(CalendarDetails.K_ATTENDEES, meetingOnServer.getHashMap().get(CalendarDetails.K_ATTENDEES));
            doc.getHashMap().put(CalendarDetails.K_FREE_BUSSY, meetingOnServer.getHashMap().get(CalendarDetails.K_FREE_BUSSY));
            doc.getHashMap().put(CalendarDetails.K_ACTUAL_FREE_BUSSY, meetingOnServer.getHashMap().get(CalendarDetails.K_ACTUAL_FREE_BUSSY));

            //this is because RAZR and V400 does not suport location
            if (zimbraWrapper.getPhone().equalsIgnoreCase(Def.MOTOROLA_RAZR)
                    || zimbraWrapper.getPhone().equalsIgnoreCase(Def.MOTOROLA_V400))
                doc.getHashMap().put(CalendarDetails.K_LOCATION, meetingOnServer.getHashMap().get(CalendarDetails.K_LOCATION));

            calendarDao.updateMeeting(doc);
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarManager.updateCalendar failed for id " + id);
            throw new EntityException(e.toString());
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.updateCalendar ended for id " + id);
    }


    /**
     * Inserts the given foundationCalendar and returns a new calendar with the
     * rigth UID.
     *
     * @param foundationCalendar
     * @param itemFormat         String
     * @return Calendar
     * @throws EntityException
     */
    public Calendar insertCalendar(Calendar foundationCalendar,
                                   String itemFormat,
                                   Timestamp time ) throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.insertCalendar started");

        MeetingProperties newMeeting;
        try {
            MeetingProperties meetingProperties
                = getMeetingPropertiesFromCalendar(foundationCalendar, itemFormat );
            newMeeting = calendarDao.insertMeeting(meetingProperties);
        } catch (EntityException e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarManager.insertCalendar failed: " + e);
            throw new EntityException(e.toString());
        }

        Calendar calendar = null;
        try {
            calendar = getCalendarFromMeetingProperties(newMeeting);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("CalendarManager.insertCalendar failed: " + e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.insertCalendar ended");

        return calendar;
    }


    /**
     * Returns the twin of the given calendar. A twin is searched looking for the firstName,
     * the lastName and the location.
     *
     * @param calendar   Calendar
     * @param itemFormat String
     * @return Calendar
     * @throws EntityException
     */
    public Calendar getCalendarTwin(Calendar calendar,
                                    String itemFormat)
            throws EntityException {

        Calendar foundationTwin = null;

        Event event = calendar.getEvent();

        String sDateTimeStart = (String) event.getDtStart().getPropertyValue();
        String sDateTimeEnd = (String) event.getDtEnd().getPropertyValue();

        log.finest("Start time: " + sDateTimeStart);

        if (sDateTimeStart.endsWith("Z") && !zimbraWrapper.getPhone().equals(Def.POCKET_PC)) {
            try {
                sDateTimeStart = sDateTimeStart.substring(0, sDateTimeStart.length() - 1);
                log.finest("DateTimeStart before conversion: " + sDateTimeStart);
                log.finest("TimeZone: " + zimbraWrapper.getTimeZone());
                sDateTimeStart = UtilityCalendar.covertLocalDateToGMT(sDateTimeStart, zimbraWrapper.getTimeZone());
            } catch (Exception e) {
                log.warning("Conversion error! " + e.getMessage());
            }
        }
        if (sDateTimeEnd.endsWith("Z") && !zimbraWrapper.getPhone().equals(Def.POCKET_PC)) {
            try {
                sDateTimeEnd = sDateTimeEnd.substring(0, sDateTimeEnd.length() - 1);
                log.finest("DateTimeEnd before conversion: " + sDateTimeEnd);
                log.finest("TimeZone: " + zimbraWrapper.getTimeZone());
                sDateTimeEnd = UtilityCalendar.covertLocalDateToGMT(sDateTimeEnd, zimbraWrapper.getTimeZone());
            } catch (Exception e) {
                log.warning("Conversion error! " + e.getMessage());
            }
        }

        if (event.getAllDay().booleanValue()) {
            try {
                sDateTimeStart =
                        UtilityCalendar.covertSifEDateToZimbraDate(sDateTimeStart);
                sDateTimeEnd =
                        UtilityCalendar.covertSifEDateToZimbraDate(sDateTimeEnd);
            } catch (Exception e) {
                throw new EntityException(e);
            }
        }

        String location = (String) calendar.getEvent().getLocation().getPropertyValue();


        String[] values = {sDateTimeStart, sDateTimeEnd, location};
        try {
            String[] fields = {
                MeetingProperties.K_DATETIME_START,
                MeetingProperties.K_DATETIME_END,
                MeetingProperties.K_LOCATION
            };

            log.finest("Searching twin with the following params (S/E/L):");
            log.finest(values[0]);
            log.finest(values[1]);
            log.finest(values[2]);

            MeetingProperties[] twins = calendarDao.getCalendars(fields, values, 1);
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Found " + twins.length + " twins");
            }
            if (twins.length > 0) {
                //
                // Returns first calendar
                //
                foundationTwin = getCalendarFromMeetingProperties(twins[0]);
            }

        } catch (Exception e) {
            throw new EntityException("Error getting twins", e);
        }

        return foundationTwin;
    }


    /**
     * Deletes the calendar with the given UID
     *
     * @param UID
     * @throws EntityException
     */
    public void removeCalendar(String UID) throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.removeCalendar started for id " + UID);
        try {
            calendarDao.removeMeeting(UID);
        } catch (EntityException e) {
            if (log.isLoggable(Level.WARNING))
                log.warning("CalendarManager.removeCalendar failed for id " + UID);
            throw new EntityException("Error deleting meeting", e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.removeCalendar ended for id " + UID);
    }


    // ---------------------------------------------------------- Private Methods

    /**
     * Converts the given MeetingProperties into a Calendar.
     *
     * @param meeting
     * @return Calendar
     */
    private Calendar getCalendarFromMeetingProperties(MeetingProperties meeting)
            throws EntityException {
        String id = Utility.nullToVoid(meeting.get(MeetingProperties.K_ID));

        if (log.isLoggable(Level.FINEST))
            log.finest("CalendarManager.getCalendarFromMeetingProperties started for id " + id);

        String organizer;
        try {
            organizer = (String) ((HashMap) meeting.get(MeetingProperties.K_ORGANIZER)).
                    get(MeetingProperties.K_ADDRESS);
        } catch (Exception e) {
            organizer = null;
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("Organizer: " + organizer);

        boolean allDay = "1".equals(meeting.get(MeetingProperties.K_ALL_DAY));

        String sDateTimeStart = (String) meeting.get(MeetingProperties.K_DATETIME_START);
        String sDateTimeEnd = (String) meeting.get(MeetingProperties.K_DATETIME_END);


        try {
            if (allDay) {
                sDateTimeStart = UtilityCalendar.covertZimbraDateToSifEDate(sDateTimeStart);
                sDateTimeEnd = UtilityCalendar.covertZimbraDateToSifEDate(sDateTimeEnd);
            }
            else 
            {

                // - kal -
                if ( deviceTimezoneDescr != null )
                {
                    // convert zimbra meeting times to UTC 
                    String sTz = (String) meeting.get(MeetingProperties.K_TIMEZONE);
                    // convert &amp; to &
                    sTz = StringUtils.replace(sTz, "&amp;", "&");
                    TimeZone eventTimeZone = TimeZone.getTimeZone( this.zimbraWrapper.zimbraToJavaTimeZone( sTz ) );
                
                    sDateTimeStart = TimeUtils.convertLocalDateToUTC( sDateTimeStart, eventTimeZone );
                    sDateTimeEnd = TimeUtils.convertLocalDateToUTC( sDateTimeEnd, eventTimeZone );
                }
                // - kal -


                if (this.zimbraWrapper.getPhone() == null || this.zimbraWrapper.getPhone().trim().length() == 0) 
                {
                    log.finest("PHONE in wrapper: " + this.zimbraWrapper.getPhone());
                    if (!sDateTimeStart.endsWith("Z"))
                        sDateTimeStart =
                            UtilityCalendar.covertLocalDateToGMT(sDateTimeStart, zimbraWrapper.getTimeZone());

                    if (!sDateTimeEnd.endsWith("Z"))
                        sDateTimeEnd =
                            UtilityCalendar.covertLocalDateToGMT(sDateTimeEnd, zimbraWrapper.getTimeZone());

                    log.finest("Converted start date: " + sDateTimeStart);
                    log.finest("Converted end date: " + sDateTimeEnd);

                }
            }
            log.finest("Start date: " + sDateTimeStart);
            log.finest("End date: " + sDateTimeEnd);

        } catch (Exception e) {
            throw new EntityException("Conversion to SIF-E failed: " + e);
        }

        String status = (String) meeting.get(MeetingProperties.K_STATUS);
        if ("CANC".equals(status))
            status = "Cancelled";
        else if ("TENT".equals(status))
            status = "Tentative";
        else
            status = "Confirmed";

        String location = Utility.nullToVoid(meeting.get(MeetingProperties.K_LOCATION));
        String subject = Utility.nullToVoid(meeting.get(MeetingProperties.K_NAME));

        short meetingStatus = 2;
        String sMeetingStatus = (String) meeting.get(MeetingProperties.K_FREE_BUSSY);
        if ("F".equals(sMeetingStatus))
            meetingStatus = (short) 0;
        else if ("T".equals(sMeetingStatus))
            meetingStatus = (short) 1;
        else if ("U".equals(sMeetingStatus)) meetingStatus = (short) 3;

        String transp = (String) meeting.get(MeetingProperties.K_TRANSPARENCY);

        log.finest("Status: " + status);

        Object[] objs;
        try {
            objs = ((Object[]) meeting.get(MeetingProperties.K_RECURENCE));
        } catch (Exception e) {
            objs = null;
        }
        RecurrenceDetails recurrenceDetails = null;
        if (objs != null && objs.length > 0 && objs[0] instanceof RecurrenceDetails)
            recurrenceDetails = (RecurrenceDetails) objs[0];

        RecurrencePattern recurrencePattern =
                RecurrencePatternExt.getRecurrencePattern(recurrenceDetails);

        String rRule = null;
        if (recurrenceDetails != null)
            try {
		log.finest("Recurrence is non-null: " + recurrenceDetails.toString());
                rRule = recurrenceDetails.getRRule();
                if (allDay) rRule = rRule.substring(0, rRule.indexOf("T") + 1) + "235959Z";
                log.finest("RRULE: " + rRule);
            } catch (Exception e) {
                throw new EntityException("Error setting rrule for ID " + id + ": " + e.getMessage());
            }


        Calendar calendar = new Calendar();
        calendar.getEvent().getUid().setPropertyValue(id);
        calendar.getEvent().getDtStart().setPropertyValue(sDateTimeStart);
        calendar.getEvent().getDtEnd().setPropertyValue(sDateTimeEnd);
        calendar.getEvent().setAllDay(Boolean.valueOf(allDay));
        calendar.getEvent().getStatus().setPropertyValue(status);
        calendar.getEvent().setMeetingStatus(new Short(meetingStatus));
        calendar.getEvent().getLocation().setPropertyValue(location);
        calendar.getEvent().getSummary().setPropertyValue(subject);
        calendar.getEvent().getTransp().setPropertyValue(transp);
        calendar.getEvent().setRecurrencePattern(recurrencePattern);
        calendar.getEvent().getOrganizer().setPropertyValue(organizer);
        calendar.getEvent().getRrule().setPropertyValue(rRule);

        log.finest("CalendarManager.getCalendarFromMeetingProperties ended for id " + id);

        return calendar;
    }


    /**
     * Converts the given Calendar into a MeetingProperties.
     *
     * @param calendar   Calendar
     * @param itemFormat String
     * @return MeetingProperties
     */
    protected MeetingProperties getMeetingPropertiesFromCalendar(Calendar calendar,
                                                                 String itemFormat)
            throws EntityException {

        // create a MeetingProperties
        MeetingProperties meetingProperties = new MeetingProperties();

        Event event = calendar.getEvent();
        meetingProperties.put(MeetingProperties.K_ID,
                event.getUid().getPropertyValue().toString());

        String summary = (String) event.getSummary().getPropertyValue();
        String description = (String) event.getDescription().getPropertyValue();

        // - kal - 
        // Default summary to 'Unnamed' when summary and description are null or empty strings
        if (summary == null)
            summary = "Unnamed";
        // - kal - 

        if (summary == null || summary.equals("")) summary = description;
        meetingProperties.put(MeetingProperties.K_NAME, summary);

        try {
            meetingProperties.put(MeetingProperties.K_LOCATION,
                    event.getLocation().getPropertyValue().toString());
        } catch (Exception e) {
            meetingProperties.put(MeetingProperties.K_LOCATION, "");
        }


        String sDateTimeStart = (String) event.getDtStart().getPropertyValue();
        String sDateTimeEnd = (String) event.getDtEnd().getPropertyValue();


        // - kal -
        String zimbraDeviceTimezone = zimbraWrapper.getTimeZone();

        // deviceTimezoneDescr != null  when convert dates to this timezone is checked in the admin panel
        if ( deviceTimezoneDescr != null )
        {
            TimeZone deviceTimezone = TimeZone.getTimeZone( deviceTimezoneDescr );

            // set zimbra timezone to device timezone
            zimbraDeviceTimezone = this.zimbraWrapper.javaToZimbraTimeZone( deviceTimezoneDescr );
            // XML encode  &  to  &amp;  for MeetingProperties.K_TIMEZONE
            zimbraDeviceTimezone = StringUtils.replace(zimbraDeviceTimezone, "&", "&amp;");
        
            // convert iCal times to device local times
            try
            {
                sDateTimeStart = TimeUtils.convertUTCDateToLocal( sDateTimeStart, deviceTimezone );
                sDateTimeEnd = TimeUtils.convertUTCDateToLocal( sDateTimeEnd, deviceTimezone );
            } catch ( Exception e ) {
                throw new EntityException("Error converting appointment start and end times");
            }
        }

        // get event type
        List xtags = calendar.getEvent().getXTags();
        String event_type = "";
        if ( xtags != null )
        {
            for ( int i = 0; i < xtags.size(); i++ )
            {
                XTag tag = (XTag)xtags.get(i);
                if ( tag.getXTagValue().equals( "X-EPOCAGENDAENTRYTYPE" ) )
                    event_type = tag.getXTag().getPropertyValueAsString();
            }
        }

        // special check to make memo's all day events  // event_type.equals( "EVENT" )
        if ( sDateTimeStart.endsWith( "T000000Z" ) &&
             sDateTimeEnd.endsWith( "T000000Z" ) )
        {
            try 
            {
                String start_day = sDateTimeStart.substring( 0, sDateTimeStart.indexOf( "T" ) );
                String end_day = sDateTimeEnd.substring( 0, sDateTimeEnd.indexOf( "T" ) );

                if ( !start_day.equals( end_day ) )
                {
                    java.text.DateFormat utcFormatter = new java.text.SimpleDateFormat( "yyyyMMdd'T'HHmmss'Z'" );
                    Date end = utcFormatter.parse( sDateTimeEnd );
                    long time = end.getTime();
                    // subtract 1 min
                    end.setTime( time - ( 60 * 1000 ) );
                    sDateTimeEnd = utcFormatter.format( end );
                }
            } catch ( Exception e ) {
                throw new EntityException("Error converting memo end date time." );
            }


            sDateTimeStart = sync4j.foundation.pdi.utils.TimeUtils.convertDateFromTo( sDateTimeStart,
                                                                                      sync4j.foundation.pdi.utils.TimeUtils.PATTERN_YYYY_MM_DD);
            sDateTimeEnd = sync4j.foundation.pdi.utils.TimeUtils.convertDateFromTo( sDateTimeEnd,
                                                                                    sync4j.foundation.pdi.utils.TimeUtils.PATTERN_YYYY_MM_DD);
            event.setAllDay( Boolean.valueOf( true ) );

        }

        meetingProperties.put(MeetingProperties.K_TIMEZONE, zimbraDeviceTimezone );
        // - kal -


        if (event.getAllDay().booleanValue()) {
            meetingProperties.put(MeetingProperties.K_ALL_DAY, "1");
            try {
                sDateTimeStart =
                        UtilityCalendar.covertSifEDateToZimbraDate(sDateTimeStart);
                sDateTimeEnd =
                        UtilityCalendar.covertSifEDateToZimbraDate(sDateTimeEnd);
            } catch (Exception e) {
                throw new EntityException("Conversion failed!");
            }
        } else
            meetingProperties.put(MeetingProperties.K_ALL_DAY, "0");

        if (sDateTimeStart.endsWith("Z")) {
            if (!itemFormat.equals(Def.MSG_TYPE_ICAL)) {
                try {
                    String newSDate = UtilityCalendar.covertUTCDateToLocal(sDateTimeStart, zimbraWrapper.getTimeZone());
                    meetingProperties.put(MeetingProperties.K_DATETIME_START, newSDate.substring(0, newSDate.length() - 1));
                } catch (Exception e) {
                    log.warning("Error converting appointment start time!");
                }
            } else
                meetingProperties.put(MeetingProperties.K_DATETIME_START, sDateTimeStart.substring(0, sDateTimeStart.length() - 1));
        } else
            meetingProperties.put(MeetingProperties.K_DATETIME_START, sDateTimeStart);

        if (sDateTimeEnd.endsWith("Z")) {
            if (!itemFormat.equals(Def.MSG_TYPE_ICAL)) {
                try {
                    String newEDate = UtilityCalendar.covertUTCDateToLocal(sDateTimeEnd, zimbraWrapper.getTimeZone());
                    meetingProperties.put(MeetingProperties.K_DATETIME_END, newEDate.substring(0, newEDate.length() - 1));
                } catch (Exception e) {
                    log.warning("Error converting appointment end time!");
                }
            } else
                meetingProperties.put(MeetingProperties.K_DATETIME_END, sDateTimeEnd.substring(0, sDateTimeEnd.length() - 1));
        } else
            meetingProperties.put(MeetingProperties.K_DATETIME_END, sDateTimeEnd);



        String sOrganizer = (String) event.getOrganizer().getPropertyValue();


        if (sOrganizer != null) {
            HashMap organizer = new HashMap();
            organizer.put(MeetingProperties.K_ADDRESS, sOrganizer);
            organizer.put(MeetingProperties.K_FRIENDLY_NAME, sOrganizer);
            meetingProperties.put(MeetingProperties.K_ORGANIZER, organizer);

        }

        String status = (String) event.getStatus().getPropertyValue();
        if (status == null)
            status = "CONF";
        else if ("Cancelled".equals(status))
            status = "CANC";
        else if ("Tentative".equals(status))
            status = "TENT";
        else
            status = "CONF";
        meetingProperties.put(MeetingProperties.K_STATUS, status);


        String sMeetingStatus = "B";
        if (event.getMeetingStatus() != null) {
            short meetingStatus = event.getMeetingStatus().shortValue();
            if (meetingStatus == (short) 0)
                sMeetingStatus = "F";
            else if (meetingStatus == (short) 1)
                sMeetingStatus = "T";
            else if (meetingStatus == (short) 3) sMeetingStatus = "U";
        }

        meetingProperties.put(MeetingProperties.K_ACTUAL_FREE_BUSSY, sMeetingStatus);
        meetingProperties.put(MeetingProperties.K_FREE_BUSSY, sMeetingStatus);

        String transp = (String) event.getStatus().getPropertyValue();
        if (transp == null ||
                (!"O".equalsIgnoreCase(transp) &&
                !"T".equalsIgnoreCase(transp)))
            meetingProperties.put(MeetingProperties.K_TRANSPARENCY, "O");
        else
            meetingProperties.put(MeetingProperties.K_TRANSPARENCY, transp);

        RecurrenceDetails recurrenceDetails = null;
        if (event.isRecurrent()) {
            recurrenceDetails = new RecurrenceDetails();

            recurrenceDetails.put(RecurrenceDetails.K_ADD_EXCLUDE, "add");
            RecurrencePattern recurrencePattern = event.getRecurrencePattern();
            String patternStartDate = recurrencePattern.getStartDatePattern();
            String patternEndDate = recurrencePattern.getEndDatePattern();
            boolean noEndDate = recurrencePattern.isNoEndDate();
            short recurrenceType = recurrencePattern.getType();
            int interval = recurrencePattern.getInterval() > 0 ?
                    recurrencePattern.getInterval() : 1;
            short daysOfWeekMask = recurrencePattern.getDayOfWeekMask();
            short monthOfYear = recurrencePattern.getMonthOfYear();
            short dayOfMonth = recurrencePattern.getDayOfMonth();
            short instance = recurrencePattern.getInstance();

            recurrenceDetails.put(RecurrenceDetails.K_TIMEZONE, zimbraWrapper.getTimeZone());
            recurrenceDetails.put(RecurrenceDetails.K_DATETIME_START,
                    patternStartDate);

            if (!noEndDate) {
                /*if(patternEndDate.endsWith("Z"))
                    recurrenceDetails.put(RecurrenceDetails.K_TIMEZONE,
                                     "(GMT) Greenwich Mean Time - Dublin /" +
                                     " Edinburgh / Lisbon / London"); */
                recurrenceDetails.put(RecurrenceDetails.K_RULE_DATETIME_END,
                        patternEndDate);
            }
            if (recurrenceType == (short) 0)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY, "DAI");
            if (recurrenceType == (short) 1)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY, "WEE");
            if (recurrenceType == (short) 2 || recurrenceType == (short) 3)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY, "MON");
            if (recurrenceType == (short) 5 || recurrenceType == (short) 6)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_FREQUENCY, "YEA");

            recurrenceDetails.put(RecurrenceDetails.K_RULE_INTERVAL, "" + interval);

            List weekDaysList = new LinkedList();
            if (daysOfWeekMask >= 64) {
                weekDaysList.add("SA");
                daysOfWeekMask -= 64;
            }
            if (daysOfWeekMask >= 32) {
                weekDaysList.add("FR");
                daysOfWeekMask -= 32;
            }
            if (daysOfWeekMask >= 16) {
                weekDaysList.add("TH");
                daysOfWeekMask -= 16;
            }
            if (daysOfWeekMask >= 8) {
                weekDaysList.add("WE");
                daysOfWeekMask -= 8;
            }
            if (daysOfWeekMask >= 4) {
                weekDaysList.add("TU");
                daysOfWeekMask -= 4;
            }
            if (daysOfWeekMask >= 2) {
                weekDaysList.add("MO");
                daysOfWeekMask -= 2;
            }
            if (daysOfWeekMask >= 1) {
                weekDaysList.add("SU");
                daysOfWeekMask -= 1;
            }
            Iterator iter = weekDaysList.iterator();
            HashMap[] weekDays = new HashMap[weekDaysList.size()];
            int i = 0;
            while (iter.hasNext()) {
                weekDays[i] = new HashMap();
                weekDays[i].put(RecurrenceDetails.K_RULE_BYDAY_DAY, iter.next());
                if (instance > 0)
                    weekDays[i].put(RecurrenceDetails.K_RULE_BYDAY_ORDWK, "" + instance);
                i++;
            }

            recurrenceDetails.put(RecurrenceDetails.K_RULE_BYDAY, weekDays);


            if (dayOfMonth > 0)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMONTHDAY,
                        new String[]{"" + dayOfMonth});
            if (monthOfYear > 0)
                recurrenceDetails.put(RecurrenceDetails.K_RULE_BYMONTH,
                        new String[]{"" + monthOfYear});
        }
        if (calendar.getEvent().getRrule() != null &&
                calendar.getEvent().getRrule().getPropertyValue() != null) {
            String rrule =
                    (String) calendar.getEvent().getRrule().getPropertyValue();
            try {
                recurrenceDetails = RRuleParser.parse(rrule);
            } catch (MalformedRRuleException e) {
                throw new EntityException(e);
            }
            if (!recurrenceDetails.containsKey(RecurrenceDetails.K_TIMEZONE))
                recurrenceDetails.put(RecurrenceDetails.K_TIMEZONE, zimbraWrapper.getTimeZone());
            /*"(GMT) Greenwich Mean Time - Dublin /" +
            " Edinburgh / Lisbon / London");*/
        }
        if (recurrenceDetails != null)
            meetingProperties.put(MeetingProperties.K_RECURENCE,
                    new RecurrenceDetails[]{recurrenceDetails});

        return meetingProperties;
    }

    /**
     * Converts a MeetingProperties[] in a Calendar[].
     *
     * @param meetingProperties
     * @param itemFormat        String
     * @return Calendar[]
     * @throws EntityException
     */
    protected Calendar[] getCalendarsFromMeetingsProperties(MeetingProperties[] meetingProperties,
                                                            String itemFormat) throws EntityException {

        Calendar[] meetings = new Calendar[meetingProperties.length];
        for (int i = 0; i < meetings.length; i++) {
            meetings[i] = getCalendarFromMeetingProperties(meetingProperties[i]);
        }
        return meetings;
    }

    /**
     * Get all the Contacts info (key, lastmodified) for a specified pricipal
     *
     * @param principal
     * @return SyncItemInfo[]
     */
    public SyncItemInfo[] getAllSyncItemInfo(Principal principal) throws EntityException {
        return calendarDao.getAllSyncItemInfo(principal);
    }


}
