package sync4j.zimbra.items.manager;

import java.sql.Timestamp;
import java.util.*;
import java.util.logging.Level;
import java.security.Principal;

import sync4j.foundation.pdi.contact.*;
import sync4j.foundation.engine.source.SyncItemInfo;

import sync4j.zimbra.exception.EntityException;
import sync4j.zimbra.items.dao.ContactDAO;
import sync4j.zimbra.items.model.ContactProperties;
import sync4j.zimbra.util.*;


/**
 * Represents the class used by the <code>ZimbraContactSyncSource</code> to handle
 * the contacts. It uses <code>ContactDAO</code> to access to Zimbra data (It works as
 * wrapper between the <code>ZimbraContactSyncSource</code> and the <code>ContactDAO</code>).
 * <br>All methods invoked by the syncsource return/work with Contact (foundationContact)
 * <br>All methods invoked on the DAO return/work with ContactProperties.
 */
public class ContactManager extends EntityManager {


    // ------------------------------------------------------------ Private data

    /**
     * The DAO to access to Zimbra contacts
     */
    private ContactDAO contactDao = null;

    /**
     *
     */

    private String phone;
    //-------------------------------------------------------------- Costructors

    /**
     * Creates a new ContactManager using the given properties
     *
     * @throws EntityException
     */
    public ContactManager(HashMap properties) throws EntityException {
        super(properties);
        contactDao = new ContactDAO(zimbraWrapper);
        phone = (String) properties.get(PHONE);
    }

    //----------------------------------------------------------- Public methods

    /**
     * Returns all contacts.
     *
     * @return Contact[]
     * @throws EntityException
     */
    public Contact[] getAllContacts(String itemFormat) throws EntityException {

        log.finest("ContactManager.getAllContacts start");

        ContactProperties[] allContactsProperties;
        Contact[] allContacts;
        try {
            allContactsProperties = contactDao.getAllContacts();
            allContacts = getContactsFromContactsProperties(allContactsProperties, itemFormat);
        } catch (Exception e) {
            log.finest("ContactManager.getAllContacts failed: " + e);
            throw new EntityException(e);
        }

        log.finest("ContactManager.getAllContacts end");

        return allContacts;
    }

    /**
     * Returns the Contact with the given id. <code>null</code> if the contact isn't found.
     *
     * @param UID String
     * @return Contact
     * @throws EntityException
     */
    public Contact getContactByUID(String UID, String itemFormat) throws EntityException {

        ContactProperties doc;
        Contact foundationContact;
        try {
            doc = contactDao.getContactByUID(UID);
            if (doc == null) {
                return null;
            }
            foundationContact = getContactFromContactProperties(doc, itemFormat);
        } catch (Exception e) {
            throw new EntityException(e.toString());
        }
        return foundationContact;
    }


    /**
     * Updates the contact defined by the given foundationContact.
     *
     * @param foundationContact Contact
     * @param itemFormat        String
     * @throws EntityException
     */
    public void updateContact(Contact foundationContact,
                              String itemFormat,
                              Timestamp time)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ContactManager.updateContact");

        ContactProperties doc = getContactPropertiesFromContact(foundationContact, itemFormat);

        try {
            contactDao.updateContact(doc);
        } catch (EntityException e) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ERROR: ContactManager.updateContact: " + e);
            throw new EntityException(e);
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("END: ContactManager.updateContact");

    }


    /**
     * Inserts the given foundationContact and returns a new contact with the
     * rigth UID.
     *
     * @param foundationContact
     * @param itemFormat        String
     * @return Contact
     * @throws EntityException
     */
    public Contact insertContact(Contact foundationContact,
                                 String itemFormat,
                                 Timestamp time)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactManager.insertContact started");

        ContactProperties contactProperties
                = getContactPropertiesFromContact(foundationContact, itemFormat);

        ContactProperties docNew;
        try {
            docNew = contactDao.insertContact(contactProperties);
        } catch (EntityException e) {
            throw new EntityException(e.toString());
        }

        if (log.isLoggable(Level.FINEST))
            log.finest("ContactManager.insertContact ended");

        return getContactFromContactProperties(docNew, itemFormat);
    }


    /**
     * Returns the twin of the given contact. A twin is searched looking the firstName,
     * the lastName and the email.
     *
     * @param contact    Contact
     * @param itemFormat String
     * @return Contact
     * @throws EntityException
     */
    public Contact getContactTwin(Contact contact,
                                  String itemFormat)
            throws EntityException {

        Contact foundationTwin = null;

        String firstName;
        String lastName;
        String email1;

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ContactManager.getContactTwin");

        firstName = null;
        lastName = null;
        if (contact.getName() != null)
            if (contact.getName().getFirstName() != null)
                firstName = (String) contact.getName().getFirstName().getPropertyValue();
        if (contact.getName().getLastName() != null)
            lastName = (String) contact.getName().getLastName().getPropertyValue();
        if ("".equals(firstName) || " ".equals(firstName)) firstName = null;
        if ("".equals(lastName) || " ".equals(lastName)) lastName = null;
        String[] emails = UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_1_ADDRESS);
        if (emails != null && emails.length > 0)
            email1 = emails[0];
        else
            email1 = null;

        if (log.isLoggable(Level.FINEST))
            log.finest("MESSAGE: Search twin for: " + firstName + " " + lastName + " " + email1);

        try {
            String[] fields =
                    {
                        ContactProperties.K_FIRST_NAME,
                        ContactProperties.K_LAST_NAME,
                        ContactProperties.K_PD_EMAIL1
                    };
            String[] values =
                    {
                        firstName,
                        lastName,
                        email1
                    };

            ContactProperties[] twins = contactDao.getContacts(fields, values);
            if (log.isLoggable(Level.FINEST))
                log.finest("MESSAGE: Found " + twins.length + " twins");
            if (twins.length > 0)
                foundationTwin = getContactFromContactProperties(twins[0], itemFormat);
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ContactManager.getContactTwin: " + e);
            throw new EntityException("Error getting twins: " + e.getMessage(), e);
        }
        return foundationTwin;
    }


    /**
     * Deletes the contact with the given UID
     *
     * @param UID
     * @throws EntityException
     */
    public void removeContact(String UID) throws EntityException {
        if (log.isLoggable(Level.FINEST))
            log.finest("ContactManager.removeContact started");
        try {
            contactDao.removeContact(UID);
        } catch (EntityException e) {
            throw new EntityException("Error deleting contact: " + e.getMessage(), e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("ContactManager.removeContact ended");
    }


    // ---------------------------------------------------------- Private Methods


    /**
     * Converts the given ContactProperties in a Contact.
     *
     * @param contact
     * @param itemFormat
     * @return Contact
     */
    private Contact getContactFromContactProperties(ContactProperties contact,
                                                    String itemFormat)
            throws EntityException {

        if (log.isLoggable(Level.FINEST))
            log.finest("START: ContactManager.getContactFromContactsProperties");


        Contact foundationContact =
                new sync4j.foundation.pdi.contact.Contact();
        try {
            foundationContact.setUid((String) contact.get(ContactProperties.K_UID));

            Address padd = foundationContact.getPersonalDetail().getAddress();
            Address oadd = foundationContact.getPersonalDetail().getOtherAddress();
            BusinessDetail bd = foundationContact.getBusinessDetail();
            Address badd = foundationContact.getBusinessDetail().getAddress();

            UtilityContact.setNameInContact(foundationContact,
                    contact.get(ContactProperties.K_FIRST_NAME),
                    contact.get(ContactProperties.K_MIDDLE_NAME),
                    contact.get(ContactProperties.K_LAST_NAME));

            UtilityContact.setBusinessTitleInFoundation(foundationContact,
                    contact.get(ContactProperties.K_BD_TITLE));

            UtilityContact.setNote1InFoundation(foundationContact, contact.get(ContactProperties.K_NOTE));

            bd.setAssistant((String) contact.get(ContactProperties.K_BD_ASSISTANT));
            bd.getCompany().setPropertyValue(contact.get(ContactProperties.K_BD_COMPANY));

            UtilityContact.addWebPageInContact(foundationContact,
                    Def.BUSINESS_WEBPAGE,
                    contact.get(ContactProperties.K_BD_WEBPAGE));
            UtilityContact.addWebPageInContact(foundationContact,
                    Def.PERSONAL_WEBPAGE,
                    contact.get(ContactProperties.K_PD_WEBPAGE));
            UtilityContact.addWebPageInContact(foundationContact,
                    Def.OTHER_WEBPAGE,
                    contact.get(ContactProperties.K_PD_OTHERWEBPAGE));

            badd.getStreet().setPropertyValue(contact.get(ContactProperties.K_BD_ADDRESS_STREET));
            badd.getCity().setPropertyValue(contact.get(ContactProperties.K_BD_ADDRESS_CITY));
            badd.getState().setPropertyValue(contact.get(ContactProperties.K_BD_ADDRESS_STATE));
            badd.getPostalCode().setPropertyValue(contact.get(ContactProperties.K_BD_ADDRESS_CODE));
            badd.getCountry().setPropertyValue(contact.get(ContactProperties.K_BD_ADDRESS_COUNTRY));

            padd.getStreet().setPropertyValue(contact.get(ContactProperties.K_PD_ADDRESS_STREET));
            padd.getCity().setPropertyValue(contact.get(ContactProperties.K_PD_ADDRESS_CITY));
            padd.getState().setPropertyValue(contact.get(ContactProperties.K_PD_ADDRESS_STATE));
            padd.getPostalCode().setPropertyValue(contact.get(ContactProperties.K_PD_ADDRESS_CODE));
            padd.getCountry().setPropertyValue(contact.get(ContactProperties.K_PD_ADDRESS_COUNTRY));

            oadd.getStreet().setPropertyValue(contact.get(ContactProperties.K_PD_OADDRESS_STREET));
            oadd.getCity().setPropertyValue(contact.get(ContactProperties.K_PD_OADDRESS_CITY));
            oadd.getState().setPropertyValue(contact.get(ContactProperties.K_PD_OADDRESS_STATE));
            oadd.getPostalCode().setPropertyValue(contact.get(ContactProperties.K_PD_OADDRESS_CODE));
            oadd.getCountry().setPropertyValue(contact.get(ContactProperties.K_PD_OADDRESS_COUNTRY));

            // phone home
            // different in VCARD and XML
            if (Def.MSG_TYPE_VCARD.equals(itemFormat))
                UtilityContact.addPhoneInContact(foundationContact,
                        Def.PERSONAL_OTHER_PHONE_NUM,
                        contact.get(ContactProperties.K_PD_PHONE1));
            else
                UtilityContact.addPhoneInContact(foundationContact,
                        Def.PERSONAL_HOME_PHONE_NUM,
                        contact.get(ContactProperties.K_PD_PHONE1));
            UtilityContact.addPhoneInContact(foundationContact,
                    Def.PERSONAL_MOBILE_PHONE_NUM,
                    contact.get(ContactProperties.K_PD_PHONE2));
            UtilityContact.addPhoneInContact(foundationContact,
                    Def.PERSONAL_HOME_PHONE_NUM,
                    contact.get(ContactProperties.K_PD_PHONE3));

            if (zimbraWrapper.getPhone() != null &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.NOKIA_6600) &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.SE_P900) &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.MOTOROLA_RAZR) &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.MOTOROLA_V400))
                UtilityContact.addPhoneInContact(foundationContact,
                        Def.PERSONAL_HOME_2_PHONE_NUM,
                        contact.get(ContactProperties.K_PD_PHONE4));

            UtilityContact.addPhoneInContact(foundationContact,
                    Def.PERSONAL_HOME_FAX_NUMBER,
                    contact.get(ContactProperties.K_PD_PHONE5));
            UtilityContact.addPhoneInContact(foundationContact,
                    Def.PERSONAL_OTHER_FAX_NUMBER,
                    contact.get(ContactProperties.K_PD_OTHER_FAX));

            // phone work
            UtilityContact.setBusinessPhoneInFoundation(foundationContact,
                    (String) contact.get(ContactProperties.K_BD_PHONE1),
                    1);

            if (zimbraWrapper.getPhone() != null &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.NOKIA_6600) &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.SE_P900) &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.MOTOROLA_RAZR) &&
                    !zimbraWrapper.getPhone().equalsIgnoreCase(Def.MOTOROLA_V400))
                UtilityContact.setBusinessPhoneInFoundation(foundationContact,
                        (String) contact.get(ContactProperties.K_BD_PHONE2),
                        2);

            UtilityContact.setBusinessPhoneInFoundation(foundationContact,
                    (String) contact.get(ContactProperties.K_BD_PHONE3),
                    3);

            UtilityContact.addEmailInContact(foundationContact,
                    Def.PERSONAL_EMAIL_1_ADDRESS,
                    Utility.nullToVoid(contact.getHashMap().get(ContactProperties.K_PD_EMAIL1)));
            UtilityContact.addEmailInContact(foundationContact,
                    Def.PERSONAL_EMAIL_2_ADDRESS,
                    contact.getHashMap().get(ContactProperties.K_PD_EMAIL2));
            UtilityContact.addEmailInContact(foundationContact,
                    Def.BUSINESS_EMAIL_3_ADDRESS,
                    contact.getHashMap().get(ContactProperties.K_BD_EMAIL));
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ContactManager.getContactFromContactsProperties: " + e);
            throw new EntityException("Error converting ContactProperties from Contact: " +
                    e.getMessage(), e);
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ContactManager.getContactFromContactsProperties");
        return foundationContact;
    }


    /**
     * Converts the given Contact in a ContactPropeties.
     *
     * @param contact    Contact
     * @param itemFormat String
     * @return ContactProperties
     */
    protected ContactProperties getContactPropertiesFromContact(Contact contact,
                                                                String itemFormat)
            throws EntityException {

        // create a ContactProperties
        ContactProperties contactProperties = new ContactProperties();
        if (log.isLoggable(Level.FINEST))
            log.finest("START: ContactManager.getContactPropertiesFromContact");
        try {
            // retrive sync4j foundation Contact detail
            Name name = contact.getName();
            BusinessDetail bd = contact.getBusinessDetail();
            Address badd = bd.getAddress();
            PersonalDetail pd = contact.getPersonalDetail();
            Address padd = pd.getAddress();
            Address oadd = pd.getOtherAddress();

            contactProperties.put(ContactProperties.K_UID, contact.getUid());

            contactProperties.put(ContactProperties.K_FIRST_NAME,
                    name.getFirstName().getPropertyValue());
            contactProperties.put(ContactProperties.K_MIDDLE_NAME,
                    name.getMiddleName().getPropertyValue());
            contactProperties.put(ContactProperties.K_LAST_NAME,
                    name.getLastName().getPropertyValue());

            contactProperties.put(ContactProperties.K_NOTE, UtilityContact.getNote1FromFoundation(contact));
            contactProperties.put(ContactProperties.K_BD_TITLE, UtilityContact.getBusinessTitleFromFoundation(contact, 1));

            if (bd.getCompany() != null &&
                    bd.getCompany().getPropertyValue() != null &&
                    bd.getCompany().getPropertyValue() instanceof String)
                contactProperties.put(ContactProperties.K_BD_COMPANY,
                        bd.getCompany().getPropertyValue());
            else
                contactProperties.put(ContactProperties.K_BD_COMPANY, "");

            contactProperties.put(ContactProperties.K_BD_ASSISTANT,
                    bd.getAssistant());

            String[] businessWebPages =
                    UtilityContact.getWebPagesOfTypeFromContact(contact, Def.BUSINESS_WEBPAGE);
            String[] personalWebPages =
                    UtilityContact.getWebPagesOfTypeFromContact(contact, Def.PERSONAL_WEBPAGE);
            String[] otherWebPages =
                    UtilityContact.getWebPagesOfTypeFromContact(contact, Def.OTHER_WEBPAGE);
            if (businessWebPages.length > 0)
                contactProperties.put(ContactProperties.K_BD_WEBPAGE,
                        businessWebPages[0]);
            if (personalWebPages.length > 0)
                contactProperties.put(ContactProperties.K_PD_WEBPAGE,
                        personalWebPages[0]);
            if (otherWebPages.length > 0)
                contactProperties.put(ContactProperties.K_PD_OTHERWEBPAGE,
                        otherWebPages[0]);

            //business address
            contactProperties.put(ContactProperties.K_BD_ADDRESS_STREET,
                    badd.getStreet().getPropertyValue());
            contactProperties.put(ContactProperties.K_BD_ADDRESS_CITY,
                    badd.getCity().getPropertyValue());
            contactProperties.put(ContactProperties.K_BD_ADDRESS_STATE,
                    badd.getState().getPropertyValue());
            contactProperties.put(ContactProperties.K_BD_ADDRESS_CODE,
                    badd.getPostalCode().getPropertyValue());
            contactProperties.put(ContactProperties.K_BD_ADDRESS_COUNTRY,
                    badd.getCountry().getPropertyValue());

            //other address
            contactProperties.put(ContactProperties.K_PD_OADDRESS_STREET,
                    oadd.getStreet().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_OADDRESS_CITY,
                    oadd.getCity().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_OADDRESS_STATE,
                    oadd.getState().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_OADDRESS_CODE,
                    oadd.getPostalCode().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_OADDRESS_COUNTRY,
                    oadd.getCountry().
                    getPropertyValue());

            contactProperties.put(ContactProperties.K_PD_ADDRESS_STREET,
                    padd.getStreet().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_ADDRESS_CITY,
                    padd.getCity().getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_ADDRESS_STATE,
                    padd.getState().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_ADDRESS_CODE,
                    padd.getPostalCode().
                    getPropertyValue());
            contactProperties.put(ContactProperties.K_PD_ADDRESS_COUNTRY,
                    padd.getCountry().
                    getPropertyValue());

            //personal other phone
            if (Def.MSG_TYPE_VCARD.equals(itemFormat))
                contactProperties.put(ContactProperties.K_PD_PHONE1,
                        UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_OTHER_PHONE_NUM, 0));
            else
                contactProperties.put(ContactProperties.K_PD_PHONE1,
                        UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_HOME_PHONE_NUM, 0));
            //personal mobile phone

            contactProperties.put(ContactProperties.K_PD_PHONE2,
                    UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_MOBILE_PHONE_NUM, 0));
            //personal home phone 1
            contactProperties.put(ContactProperties.K_PD_PHONE3,
                    UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_HOME_PHONE_NUM, 0));
            //personal home phone 2
            contactProperties.put(ContactProperties.K_PD_PHONE4,
                    UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_HOME_2_PHONE_NUM, 0));
            //personal home fax

            String[] workPhones = UtilityContact.getBusinessPhonesFromContact(contact, null);
            String[] workPhones1 = UtilityContact.getBusinessPhonesFromContact(contact,
                    Def.BUSINESS_PHONE_NUM);
            String[] workPhones2 = UtilityContact.getBusinessPhonesFromContact(contact,
                    Def.BUSINESS_2_PHONE_NUM);

            String[] workFaxes;
            if (phone.equals(Def.MOTOROLA_RAZR) || phone.equalsIgnoreCase(Def.MOTOROLA_V400)) {
                workFaxes = new String[1];
                workFaxes[0] = UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_OTHER3_FAX_NUMBER, 0);
                contactProperties.put(ContactProperties.K_PD_PHONE5, UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_OTHER_FAX_NUMBER, 0));
                contactProperties.put(ContactProperties.K_PD_OTHER_FAX, UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_OTHER2_FAX_NUMBER, 0));
            } else {
                workFaxes = UtilityContact.getBusinessPhonesFromContact(contact, Def.BUSINESS_FAX_NUM);
                contactProperties.put(ContactProperties.K_PD_PHONE5, UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_HOME_FAX_NUMBER, 0));
                contactProperties.put(ContactProperties.K_PD_OTHER_FAX, UtilityContact.getPersonalPhoneFromContact(contact, Def.PERSONAL_OTHER_FAX_NUMBER, 0));
            }

            // phone 1
            if (workPhones2.length <= 0 && workPhones.length > 1 && workPhones.length > workPhones1.length + workFaxes.length) {
                contactProperties.put(ContactProperties.K_BD_PHONE1,
                        workPhones[0]);
                contactProperties.put(ContactProperties.K_BD_PHONE2,
                        workPhones[1]);
            } else {
                if (workPhones1.length > 0)
                    contactProperties.put(ContactProperties.K_BD_PHONE1,
                            workPhones1[0]);
                if (workPhones2.length > 0)
                    contactProperties.put(ContactProperties.K_BD_PHONE2,
                            workPhones2[0]);
            }
            if (workFaxes.length > 0) {
                contactProperties.put(ContactProperties.K_BD_PHONE3, workFaxes[0]);
            }

            String emailsp[] =
                    UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL);
            String emails1[] =
                    UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_1_ADDRESS);
            String emails2[] =
                    UtilityContact.getEmailsFromContact(contact, Def.PERSONAL_EMAIL_2_ADDRESS);
            String emails3[] =
                    UtilityContact.getEmailsFromContact(contact, Def.BUSINESS_EMAIL_3_ADDRESS);

            if (emails2.length <= 0 && emailsp.length > 1) {
                contactProperties.put(ContactProperties.K_PD_EMAIL1,
                        emailsp[0]);
                contactProperties.put(ContactProperties.K_PD_EMAIL2,
                        emailsp[1]);
                if (emails3.length <= 0 && emailsp.length > 2)
                    contactProperties.put(ContactProperties.K_BD_EMAIL,
                            emailsp[2]);
            } else {
                if (emails1.length > 0)
                    contactProperties.put(ContactProperties.K_PD_EMAIL1,
                            emails1[0]);

                if (emails2.length > 0)
                    contactProperties.put(ContactProperties.K_PD_EMAIL2,
                            emails2[0]);
                if (emails3.length > 0)
                    contactProperties.put(ContactProperties.K_BD_EMAIL,
                            emails3[0]);
            }
        } catch (Exception e) {
            if (log.isLoggable(Level.SEVERE))
                log.severe("ERROR: ContactManager.getContactPropertiesFromContact: " + e);
            throw new EntityException("Error converting from Contact to ContactProperties: " +
                    e.getMessage());
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("END: ContactManager.getContactPropertiesFromContact");
        return contactProperties;
    }

    /**
     * Converts a ContactProperties[] in a Contact[].
     *
     * @param contactsProperties
     * @param itemFormat         String
     * @return Contact[]
     * @throws EntityException
     */
    protected Contact[] getContactsFromContactsProperties
            (ContactProperties[] contactsProperties,
             String itemFormat)
            throws EntityException {

        Contact[] contacts = new Contact[contactsProperties.length];
        for (int i = 0; i < contacts.length; i++) {
            contacts[i] = getContactFromContactProperties(contactsProperties[i],
                    itemFormat);
        }

        return contacts;
    }

    /**
     * Get all the Contacts info (key, lastmodified) for a specified pricipal
     *
     * @param principal
     * @return SyncItemInfo[]
     */
    public SyncItemInfo[] getAllSyncItemInfo(Principal principal)
            throws EntityException {
        SyncItemInfo[] itemInfos;
        if (log.isLoggable(Level.FINEST))
            log.finest("ContactManager.getAllSyncItemInfo started");
        try {
            itemInfos = contactDao.getAllSyncItemInfo(principal);
        } catch (EntityException e) {
            if (log.isLoggable(Level.FINEST))
                log.finest("ContactManager.getAllSyncItemInfo failed");
            throw e;
        }
        if (log.isLoggable(Level.FINEST))
            log.finest("ContactManager.getAllSyncItemInfo started");
        return itemInfos;
    }

}
