package sync4j.zimbra.security;

import java.security.Principal;
import java.util.logging.Level;
import java.util.logging.Logger;

import sync4j.framework.core.Authentication;
import sync4j.framework.core.Cred;
import sync4j.framework.security.Officer;
import sync4j.framework.security.Sync4jPrincipal;
import sync4j.framework.server.store.*;
import sync4j.framework.server.Sync4jDevice;
import sync4j.framework.tools.Base64;

import sync4j.server.config.Configuration;

import sync4j.zimbra.exception.ZimbraAccessException;
import sync4j.zimbra.util.Def;
import sync4j.zimbra.util.ZimbraWrapper;

public class ZimbraOfficer implements Officer, java.io.Serializable {


    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    protected PersistentStore ps = null;

    /**
     * Has the last login failed for incorrect login/password?
     */
    private boolean loginFailed = false;

    public boolean isLoginFailed() {
        return loginFailed;
    }

    /**
     * Has the last login failed for expired temporary login?
     */
    private boolean loginExpired = false;

    /**
     * Which type of authetication does impose the server?
     */
    private String clientAuth = Cred.AUTH_TYPE_BASIC;

    public String getClientAuth() {
        return this.clientAuth;
    }

    public void setClientAuth(String clientAuth) {
        this.clientAuth = clientAuth;
    }

    /**
     * Which type of authetication use the server to send his credential?
     */
    private String serverAuth = Cred.AUTH_NONE;

    public String getServerAuth() {
        return this.serverAuth;
    }

    public void setServerAuth(String serverAuth) {
        this.serverAuth = serverAuth;
    }

    /**
     * Zimbra host
     */
    private String ZimbraUrl = null;

    public String getZimbraUrl() {
        return this.ZimbraUrl;
    }

    public void setZimbraUrl(String ZimbraUrl) {
        this.ZimbraUrl = ZimbraUrl;
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Authenticates a credential.
     *
     * @param credential the credential to be authenticated
     * @return true if the credential is autenticated, false otherwise
     */
    public boolean authenticate(Cred credential) {

        if (log.isLoggable(Level.FINEST)) {
            log.finest(" ZimbraOfficer authenticate() Start");
        }

        Configuration config = Configuration.getConfiguration();
        ps = config.getStore();
        String type = credential.getType();

        if ((Cred.AUTH_TYPE_BASIC).equals(type)) {
            return authenticateBasicCredential(credential);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest(" ZimbraOfficer authenticate() End");
        }
        return false;
    }


    /**
     * Authorizes a resource.
     *
     * @param principal the requesting entity
     * @param resource  the name (or the identifier) of the resource to be authorized
     * @return true if the credential is authorized to access the resource, false
     *         otherwise
     */
    public boolean authorize(Principal principal, String resource) {
        return true;
    }

    /**
     * Un-authenticates a credential.
     * Do nothing. In the current implementation, the authentication is
     * discarde as soon as the LoginContext is garbage collected.
     *
     * @param credential the credential to be unauthenticated
     */
    public void unAuthenticate(Cred credential) {
    }

    /**
     * @return boolean
     * @see sync4j.framework.security.Officer
     */
    public boolean isAccountExpired() {
        return loginExpired;
    }

    // --------------------------------------------------------- Private Methods

    /**
     *
     */
    private boolean authenticateBasicCredential(Cred credential) {

        String username, password;
        Sync4jPrincipal principal;

        Authentication auth = credential.getAuthentication();
        String deviceId = auth.getDeviceId(); //"syncml-phone"
        String userpwd = new String(Base64.decode(auth.getData()));

        int p = userpwd.indexOf(':');

        if (p == -1) {
            username = userpwd;
            password = "";
        } else {
            username = (p > 0) ? userpwd.substring(0, p) : "";
            password = (p == (userpwd.length() - 1)) ? "" :
                    userpwd.substring(p + 1);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Username: " + username);
        }

        //
        // Verify that exist the principal for the
        // specify deviceId and username
        //
        try
        {
            principal = new Sync4jPrincipal(username, deviceId);
            ps.read(principal);
            credential.getAuthentication().setPrincipalId(principal.getId());
            if (!checkZimbraCredentials(this.getZimbraUrl(), username, password)) return false;
        }
        catch (NotFoundException nfe)
        {
            if (log.isLoggable(Level.FINEST))
                log.finest("Principal for " + username + ":" + deviceId +
                        " not found");
            if (!checkZimbraCredentials(this.getZimbraUrl(), username, password)) return false;
            principal = new Sync4jPrincipal(username, deviceId);
            try {
                storeDevice(ps, deviceId);
                ps.store(principal);
            } catch(Exception ex) {
                log.severe("Error creating new principal: " + ex);
                log.throwing(getClass().getName(),
                        "authenticateBasicCredential", ex);
                return false;
            }

            credential.getAuthentication().setPrincipalId(principal.getId());
        }
        catch (PersistentStoreException e)
        {
            log.severe("Error reading principal: " + e);
            log.throwing(getClass().getName(), "authenticateBasicCredential", e);
            return false;
        }
        catch(Exception e)
        {
            log.severe("Error: " + e);
        }
        return true;
    }

    /**
     * Checks if the given login and password are valid
     * credentials on zimbra server
     *
     * @param serverUrl String
     * @param login     String
     * @return boolean
     */
    private boolean checkZimbraCredentials(String serverUrl,
                                           String login,
                                           String password) {

        if (serverUrl == null || serverUrl.equals("")) {

            String message = "'Zimbra Url' is null or empty. You have to set " +
                    "the Zimbra server url in the file: " +
                    "Sync4j/config/sync4j/server/security/ZimbraOfficer.xml";

            if (log.isLoggable(Level.SEVERE)) {
                log.severe(message);
            }

            throw new IllegalStateException(message);
        }

        ZimbraWrapper sw = new ZimbraWrapper();
        boolean check = false;
        try {
            check = sw.isZimbraUser(serverUrl,
                    login,
                    password);
        } catch (ZimbraAccessException dae) {
            log.severe("Error getting connection: " + dae);
            log.throwing(getClass().getName(), "check Zimbra Credentials",
                    dae);
        }

        return check;
    }

    public static Sync4jDevice storeDevice(PersistentStore ps, String deviceId)
    throws PersistentStoreException
    {
        Sync4jDevice device = new Sync4jDevice(deviceId);
        try{ps.read(device);}
        catch(PersistentStoreException e)
        {
            device.setType("Phone");
            device.setCharset("UTF-8");
            device.setDescription("Sync4j Phone");
            device.setClientNonce("".getBytes());
            device.setServerNonce("".getBytes());
            ps.store(device);
        }
        return device;
    }
}
