package sync4j.zimbra.util;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.TimeZone;
import java.util.Calendar;
import java.text.DateFormat;

import sync4j.zimbra.exception.EntityException;

/**
 * <p>Utility class for Contact Manager</p>
 */

public class UtilityCalendar {

    // --------------------------------------------------------------- Constants
    private static final String DATE_UTC_PATTERN = "yyyyMMdd'T'HHmmss'Z'";

    private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    private static final DateFormat dateFormat = new SimpleDateFormat(DATE_FORMAT);


    // ------------------------------------------------------------ Private data

    private static SimpleDateFormat dateFormatUTC = new SimpleDateFormat(DATE_UTC_PATTERN);

    static {
        dateFormatUTC.setTimeZone(TimeZone.getTimeZone("UTC"));
    }

    // ---------------------------------------------------------- Public Methods

    /**
     * Returns the given date in utc format.
     *
     * @param date Date
     * @return String
     */
    public static String getUTCFormat(Date date) {
        String utc = null;
        if (date != null) {
            utc = dateFormatUTC.format(date);
        }
        return utc;
    }

    /**
     * Returns a java.util.Date from the given sDate in utc format.
     *
     * @param sDate String
     * @return Date
     * @throws Exception
     */
    public static Date getDateFromUTCString(String sDate) throws Exception {
        Date date = null;
        if (sDate != null) {
            date = dateFormatUTC.parse(sDate);
        }
        return date;

    }

    /**
     * Convert the given date following this roles:
     * <ui>
     * <li>if the given date is in UTC no conversion is required</li>
     * <li>if the given date is not in UTC and the given timezone isn't null,
     * the date is localizated with the timezone and then it's converted in UTC.</li>
     * <li>the returned string is always in UTC</li>
     * <li>if the given timezone is null and the date isn't in UTC, the date is localizated
     * with the default timezone</li>
     * <li>the given date can be in one beetwen the following formats:
     * <code>yyyyMMdd'T'HHmmss'Z',  yyyyMMdd'T'HHmmss</code></li>
     * </ui>
     *
     * @param sDate    the given string date to convert
     * @param timezone TimeZone
     * @return String
     * @throws Exception
     */
    public static String convertLocalDateToUTC(String sDate,
                                               TimeZone timezone)
            throws Exception {

        if (sDate == null || sDate.equals("")) {
            return sDate;
        }

        String utcPattern = "yyyyMMdd'T'HHmmss'Z'";
        DateFormat utcFormatter = new SimpleDateFormat(utcPattern);
        utcFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));

        String noUtcPattern = "yyyyMMdd'T'HHmmss";
        DateFormat noUtcFormatter = new SimpleDateFormat(noUtcPattern);

        if (sDate.indexOf('Z') != -1) {
            //
            // No conversion is required
            //
            return sDate;
        }

        Date date;
        if (timezone != null) {
            noUtcFormatter.setTimeZone(timezone);
        }
        date = noUtcFormatter.parse(sDate);

        return utcFormatter.format(date);
    }

    /**
     * Convert the given date following this roles:
     * <ui>
     * <li>if the given timezone isn't null,
     * the date is localizated with the timezone and then reformatted in
     * <code>yyyyMMdd'T'HHmmss'Z'</code>.</li>
     * <li>the returned string is always in this format <code>yyyyMMdd'T'HHmmss'Z'</code></li>
     * <li>if the given timezone is null the date is not changed</li>
     * </ui>
     *
     * @param sDate    the given string date to convert
     * @param timezone TimeZone
     * @return String
     * @throws Exception
     */
    public static String convertUTCDateToLocal(String sDate,
                                               TimeZone timezone)
            throws Exception {

        if (sDate == null || sDate.equals("")) {
            return sDate;
        }

        if (timezone == null) {
            return sDate;
        }

        String utcPattern = "yyyyMMdd'T'HHmmss'Z'";
        DateFormat utcFormatter = new SimpleDateFormat(utcPattern);
        utcFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));

        Date date;
        date = utcFormatter.parse(sDate);

        utcFormatter.setTimeZone(timezone);

        return utcFormatter.format(date);
    }

    /**
     *
     * @param sDate string representation of a date in yyyyMMdd'T'HHmmss format
     * @param timeZone string representation of the TimeZone in format "(UTC+(or -)xx.xx) Decsciption"
     * @return the UTC representation
     */
    public static String covertLocalDateToGMT(String sDate, String timeZone)
    throws Exception
    {
        if(timeZone  == null)
            timeZone = "(GMT) London";
        try
        {
            if(sDate.length()>8)
                return UtilityCalendar.convertLocalDateToUTC(
                       sDate,
                       getTimezoneFromString(timeZone));
            else
                return UtilityCalendar.convertLocalDateToUTC(
                       sDate+"T000000",
                       getTimezoneFromString(timeZone));
        }
        catch(Exception e)
        {
            throw new Exception("Error while try to convert " + sDate +" "+
                                timeZone+ " to GMT: " + e.getMessage());
        }
    }

    public static String getTimeZoneSpecImportantPart(String timeZoneSpec)
    throws Exception
    {
        StringBuffer importantPart = new StringBuffer();

        for(int i=0; i<timeZoneSpec.length()-3; i++)
            if("GMT".equalsIgnoreCase(timeZoneSpec.substring(i, i+3)))
            {
                importantPart.append("GMT");
                int j = i+3;
                if(j+1<timeZoneSpec.length() &&
                   (timeZoneSpec.charAt(j)=='+' || timeZoneSpec.charAt(j)=='-'))
                {
                    if(timeZoneSpec.charAt(j+1)>='1' && timeZoneSpec.charAt(j+1)<='9')
                        if(j+2<timeZoneSpec.length() &&
                           timeZoneSpec.charAt(j+2)>='0' &&
                           timeZoneSpec.charAt(j+2)<='9')
                            importantPart.append(timeZoneSpec.substring(j, j+3));
                        else importantPart.append(timeZoneSpec.substring(j, j+2));
                    if(timeZoneSpec.charAt(j+1)=='0' &&
                       j+2<timeZoneSpec.length() &&
                       timeZoneSpec.charAt(j+2)>='1' &&
                       timeZoneSpec.charAt(j+2)<='9')
                        importantPart.append(timeZoneSpec.substring(j, j+1)).append(timeZoneSpec.substring(j+2, j+3));
                }
                return importantPart.toString();
            }
        throw new Exception("Invalid timezone specification "+timeZoneSpec);
    }

     /**
     *
     * @param sDate string representation of a date in yyyyMMdd'T'HHmmssZ format
     * @param timeZone string representation of the TimeZone in format "(UTC+(or -)xx.xx) Decsciption"
     * @return the hour described by sDate in timeZone
     */
    public static String covertUTCDateToLocal(String sDate, String timeZone)
    throws Exception
    {
        try
        {
        String importantPart=getTimeZoneSpecImportantPart(timeZone);
        return UtilityCalendar.convertUTCDateToLocal(
                                    sDate,
                                    TimeZone.getTimeZone(importantPart));
        }
        catch(Exception e)
        {
            throw new Exception("Invalid timezone: " + timeZone);
        }

    }

    /**
     * Returns the mains date and time properties to create/update a meetings into
     * Sugar from the given dtStart and dtEnd (given in utf format).
     *
     * @param dtStart String
     * @param dtEnd   String
     * @return an array string with:
     *         <ui>
     *         <li>date start in yyyy-MM-dd format</li>
     *         <li>time start in HH:mm:ss format</li>
     *         <li>duration hours</li>
     *         <li>duration minutes (rounded to 0, 15, 30, 45)</li>
     *         <li>date end in yyyy-MM-dd format</li>
     *         </ui>
     * @throws EntityException
     */
    public static String[] getDateTimeProperties(String dtStart, String dtEnd)
            throws EntityException {

        String[] dateTimeProperties = new String[5];

        Date dateStart;
        Date dateEnd;

        try {
            dateStart = UtilityCalendar.getDateFromUTCString(dtStart);
        } catch (Exception ex) {
            throw new EntityException("Error parsing dtStart '" + dtStart + "'", ex);
        }

        String dateStartFormatted = dateFormat.format(dateStart); // yyyy-MM-dd HH:mm:ss

        int indexSpace = dateStartFormatted.indexOf(" ");
        String sDateStart = dateStartFormatted.substring(0, indexSpace);
        String sTimeStart = dateStartFormatted.substring(indexSpace + 1);


        try {
            dateEnd = UtilityCalendar.getDateFromUTCString(dtEnd);
        } catch (Exception ex) {
            throw new EntityException("Error parsing dtEnd '" + dtEnd + "'", ex);
        }

        String dateEndFormatted = dateFormat.format(dateEnd); // yyyy-MM-dd HH:mm:ss

        indexSpace = dateEndFormatted.indexOf(" ");
        String sDateEnd = dateEndFormatted.substring(0, indexSpace);

        long duration = dateEnd.getTime() - dateStart.getTime();

        int hours = (int) (duration / 3600000);
        long tmpMinutes = (int) ((duration - (hours * 3600000)) / 60000);
        int minutes;

        //
        // We have to round the minutes to 0, 15, 30, 45
        //
        if (tmpMinutes <= 7.5) {
            minutes = 0;
        } else if (tmpMinutes <= 22.5) {
            minutes = 15;
        } else if (tmpMinutes <= 37.5) {
            minutes = 30;
        } else if (tmpMinutes <= 52.5) {
            minutes = 45;
        } else {
            minutes = 0;
            hours++;
        }

        dateTimeProperties[0] = sDateStart;
        dateTimeProperties[1] = sTimeStart;
        dateTimeProperties[2] = String.valueOf(hours);
        dateTimeProperties[3] = String.valueOf(minutes);
        dateTimeProperties[4] = sDateEnd;

        return dateTimeProperties;
    }

    public static String covertSifEDateToZimbraDate(String sifeDate)
    throws Exception
    {
        String sifeFormat = "yyyy-MM-dd";
        String zimbraFormat = "yyyyMMdd";

        DateFormat  sifeDF   = new SimpleDateFormat(sifeFormat);
        DateFormat  zimbraDF = new SimpleDateFormat(zimbraFormat);
        Date date = sifeDF.parse(sifeDate);
        return zimbraDF.format(date);
    }

    public static String  covertZimbraDateToSifEDate(String zimbraDate)
    throws Exception
    {
        String sifeFormat = "yyyy-MM-dd";
        String zimbraFormat = "yyyyMMdd";

        DateFormat  sifeDF   = new SimpleDateFormat(sifeFormat);
        DateFormat  zimbraDF = new SimpleDateFormat(zimbraFormat);
        Date date = zimbraDF.parse(zimbraDate);
        return sifeDF.format(date);
    }

    public static short getDayOfWeek(String dateSpec, String timeZoneSpec)
    throws Exception
    {
        if(dateSpec.length()<8)
            throw new Exception("Invalid date specification!");
        TimeZone timeZone;
        if("".equals(timeZoneSpec) || timeZoneSpec == null)
            if(dateSpec.endsWith("Z"))
                timeZone = TimeZone.getTimeZone("GMT");
            else throw new Exception("Timezone required!");
        else timeZone = getTimezoneFromString(timeZoneSpec);

        Calendar calendar = Calendar.getInstance(timeZone);
        try
        {
            int year  = Integer.parseInt(dateSpec.substring(0, 4));
            int month = Integer.parseInt(dateSpec.substring(4, 6));
            int date  = Integer.parseInt(dateSpec.substring(6, 8));
            calendar.set(year, month-1, date);
        }
        catch(Exception e)
        {
            throw new Exception("Invalid date specification!", e);
        }
        switch(calendar.get(Calendar.DAY_OF_WEEK))
        {
            case Calendar.SUNDAY:
                return (short) 1;
            case Calendar.MONDAY:
                return (short) 2;
            case Calendar.TUESDAY:
                return (short) 4;
            case Calendar.WEDNESDAY:
                return (short) 8;
            case Calendar.THURSDAY:
                return (short) 16;
            case Calendar.FRIDAY:
                return (short) 32;
            case Calendar.SATURDAY:
                return (short) 64;
        }
        return (short) 0;
    }

    public static short getDayOfMonth(String dateSpec, String timeZoneSpec)
    throws Exception
    {
        if(dateSpec.length()<8)
            throw new Exception("Invalid date specification!");
        TimeZone timeZone;
        if("".equals(timeZoneSpec) || timeZoneSpec == null)
            /*if(dateSpec.endsWith("Z"))*/
                timeZone = TimeZone.getTimeZone("GMT");
            //else throw new Exception("Timezone required!");
        else timeZone = getTimezoneFromString(timeZoneSpec);

        Calendar calendar = Calendar.getInstance(timeZone);
        try
        {
            int year  = Integer.parseInt(dateSpec.substring(0, 4));
            int month = Integer.parseInt(dateSpec.substring(4, 6));
            int date  = Integer.parseInt(dateSpec.substring(6, 8));
            calendar.set(year, month-1, date);
        }
        catch(Exception e)
        {
            throw new Exception("Invalid date specification!", e);
        }
        return (short) calendar.get(Calendar.DAY_OF_MONTH);
    }
        
    private static TimeZone getTimezoneFromString(String timeZoneSpec)
    throws Exception
    {
        try
        {
            return TimeZone.getTimeZone(getTimeZoneSpecImportantPart(timeZoneSpec));
        }
        catch(Exception e)
        {
            throw new Exception("Invalid timezone specification!", e);
        }
    }
}




