package sync4j.zimbra.util;

/**
 * User: adeac
 * Date: Dec 1, 2005
 * Time: 10:46:42 AM
 */

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.HashMap;
import java.text.MessageFormat;

import zimbra.soap.SOAPTransporter;
import zimbra.soap.AuthResponseHandler;
import zimbra.soap.util.AuthDetails;
import zimbra.soap.util.AccountInfoDetails;
import zimbra.soap.exceptions.*;

import sync4j.zimbra.exception.ZimbraAccessException;

public class ZimbraWrapper {

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    private SOAPTransporter soapTrans;

    public SOAPTransporter getSOAPTransporter() {
        return this.soapTrans;
    }

    /**
     * client's phone
     */
    private String phone;

    /**
     * client's preffered timezone
     */
    private String timeZone;

    /**
     * Zimbra session id
     */
    private String sessionID = null;

    /**
     * Returns the Zimbra session id
     *
     * @return the Zimbra session id.
     */
    public String getSessionID() {
        return this.sessionID;
    }

    /**
     * Zimbra user id
     */
    private String userID = null;

    /**
     * Returns the Zimbra user id
     *
     * @return the Zimbra user id
     */
    public String getUserID() {
        return this.userID;
    }

    /**
     * setter for phone
     *
     * @param phone
     */
    public void setPhone(String phone) {
        this.phone = phone;
    }

    /**
     * getter for phone
     *
     * @return
     */
    public String getPhone() {
        return this.phone;
    }

    /**
     * getter for timezone
     *
     * @return
     */
    public String getTimeZone() {
        return this.timeZone;
    }

    public String zimbraMailAccount = null;

    public String getZimbraMailAccount() {
        return this.zimbraMailAccount;
    }

    /**
     * /**
     * Creates a new session with Zimbra
     *
     * @param serverUrl String
     * @param username  String
     * @param password  String
     * @throws ZimbraAccessException
     */
    public void createSession(String serverUrl,
                              String username,
                              String password)
            throws ZimbraAccessException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Open Zimbra connection with:" +
                    "\n\tserverUrl: " + serverUrl +
                    "\n\tusername: " + username +
                    "\n\tpassword: *** ");
        }
        try {
            this.soapTrans = new SOAPTransporter(serverUrl);
            HashMap loginOutcome = this.soapTrans.login(username, password);
            this.sessionID = (String) loginOutcome.get(AuthDetails.K_SESSION_ID);
            this.timeZone = (String) loginOutcome.get(AuthDetails.K_TIME_ZONE);
            this.userID = (String) loginOutcome.get(AccountInfoDetails.K_ZIMBRA_ID);
            this.zimbraMailAccount = username + "@" +
                    loginOutcome.get(AccountInfoDetails.K_ZIMBRA_MAIL_HOST);

            //this for encoding purposes
            timeZone = timeZone.replaceAll("&", "&amp;");

            log.finest("TIMEZONE: " + timeZone);

        } catch (SOAPException e) {
            throw new ZimbraAccessException(e);
        }
    }


    /**
     * Closes connection.
     *
     * @throws ZimbraAccessException
     */
    public void releaseSession() throws ZimbraAccessException {
        this.soapTrans.releaseSession();
    }

    /**
     * Checks the given credential on zimbra server
     */
    public boolean isZimbraUser(String serverUrl,
                                String username,
                                String password)
            throws ZimbraAccessException {

        SOAPTransporter trans;
        String sessionID;
        try {
            trans = new SOAPTransporter(serverUrl);
        } catch (SOAPException e) {
            throw new ZimbraAccessException(e);
        }

        try {
            Object[] authOutcome =
                    trans.sendRequest(MessageFormat.format(SOAPTransporter.M_AUTHENTICATE,
                            new Object[]{username, password}),
                            new AuthResponseHandler());
            sessionID = (String) ((HashMap) authOutcome[0]).get(AuthDetails.K_SESSION_ID);
            if (sessionID == null || "".equals(sessionID)) return false;
        } catch (Exception e) {
            return false;
        }
        return true;
    }


    // - kal -
    //
    // Zimbra timezone to java timezone mapping
    //

    public String javaToZimbraTimeZone( String tz )
    {
        String res = (String)java2zimbra.get( tz );
        if ( res != null )
            return res;
        // default zimbra timezone if a timezone is not found
        return defaultZimbraTimezone;
    }

    public String zimbraToJavaTimeZone( String tz )
    {   
        String res = (String)zimbra2java.get( tz );
        if ( res != null )
            return res;
        // default java timezone if a timezone is not found
        return defaultJavaTimezone;
    }


    static String defaultZimbraTimezone = "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London";
    static String defaultJavaTimezone = "UTC";

	static HashMap zimbra2java = new HashMap();
	static HashMap java2zimbra = new HashMap();

    static {
        // populate the java2zimbra mapping
	java2zimbra.put( "ACT", "(GMT+09.30) Darwin" );
	java2zimbra.put( "AET", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "AGT", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "ART", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "AST", "(GMT-09.00) Alaska" );
	java2zimbra.put( "Africa/Abidjan", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Accra", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Addis_Ababa", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Algiers", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Asmera", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Bamako", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Bangui", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Banjul", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Bissau", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Blantyre", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Brazzaville", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Bujumbura", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Cairo", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Africa/Casablanca", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Ceuta", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Africa/Conakry", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Dakar", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Dar_es_Salaam", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Djibouti", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Douala", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/El_Aaiun", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Freetown", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Gaborone", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Harare", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Johannesburg", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Kampala", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Khartoum", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Kigali", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Kinshasa", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Lagos", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Libreville", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Lome", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Luanda", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Lubumbashi", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Lusaka", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Malabo", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Maputo", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Maseru", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Mbabane", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Mogadishu", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Monrovia", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Nairobi", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Africa/Ndjamena", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Niamey", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Nouakchott", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Ouagadougou", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Porto-Novo", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Sao_Tome", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Timbuktu", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Africa/Tripoli", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Africa/Tunis", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Africa/Windhoek", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "America/Anchorage", "(GMT-09.00) Alaska" );
	java2zimbra.put( "America/Anguilla", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Antigua", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Araguaina", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Aruba", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Asuncion", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Bahia", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Barbados", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Belem", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Belize", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Boa_Vista", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Bogota", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Boise", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Buenos_Aires", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Cambridge_Bay", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Campo_Grande", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Cancun", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Caracas", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Catamarca", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Cayenne", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Cayman", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Chicago", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Chihuahua", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Cordoba", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Costa_Rica", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Cuiaba", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Curacao", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Danmarkshavn", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "America/Dawson", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "America/Dawson_Creek", "(GMT-07.00) Arizona" );
	java2zimbra.put( "America/Denver", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Detroit", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Dominica", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Edmonton", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Eirunepe", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/El_Salvador", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Ensenada", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "America/Fort_Wayne", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Fortaleza", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Glace_Bay", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Godthab", "(GMT-03.00) Brasilia" );
	java2zimbra.put( "America/Goose_Bay", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Grand_Turk", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Grenada", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Guadeloupe", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Guatemala", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Guayaquil", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Guyana", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Halifax", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Havana", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Hermosillo", "(GMT-07.00) Arizona" );
	java2zimbra.put( "America/Indiana/Indianapolis", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Indiana/Knox", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Indiana/Marengo", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Indiana/Vevay", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Indianapolis", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Inuvik", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Iqaluit", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Jamaica", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Jujuy", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Juneau", "(GMT-09.00) Alaska" );
	java2zimbra.put( "America/Kentucky/Louisville", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Kentucky/Monticello", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Knox_IN", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/La_Paz", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Lima", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Los_Angeles", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "America/Louisville", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Maceio", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Managua", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Manaus", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Martinique", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Mazatlan", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/Mendoza", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Menominee", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Merida", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Mexico_City", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Miquelon", "(GMT-03.00) Brasilia" );
	java2zimbra.put( "America/Monterrey", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Montevideo", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Montreal", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Montserrat", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Nassau", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/New_York", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Nipigon", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Nome", "(GMT-09.00) Alaska" );
	java2zimbra.put( "America/North_Dakota/Center", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Panama", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Pangnirtung", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Paramaribo", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Phoenix", "(GMT-07.00) Arizona" );
	java2zimbra.put( "America/Port-au-Prince", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Port_of_Spain", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Porto_Acre", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Porto_Velho", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Puerto_Rico", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Rainy_River", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Rankin_Inlet", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Recife", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Regina", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Rio_Branco", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "America/Rosario", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "America/Santiago", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Santo_Domingo", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Sao_Paulo", "(GMT-03.00) Brasilia" );
	java2zimbra.put( "America/Scoresbysund", "(GMT-01.00) Azores" );
	java2zimbra.put( "America/Shiprock", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "America/St_Johns", "(GMT-03.30) Newfoundland" );
	java2zimbra.put( "America/St_Kitts", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/St_Lucia", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/St_Thomas", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/St_Vincent", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Swift_Current", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Tegucigalpa", "(GMT-06.00) Central America" );
	java2zimbra.put( "America/Thule", "(GMT-04.00) Santiago" );
	java2zimbra.put( "America/Thunder_Bay", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Tijuana", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "America/Toronto", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "America/Tortola", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Vancouver", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "America/Virgin", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "America/Whitehorse", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "America/Winnipeg", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "America/Yakutat", "(GMT-09.00) Alaska" );
	java2zimbra.put( "America/Yellowknife", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "Antarctica/Casey", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Antarctica/Davis", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Antarctica/DumontDUrville", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Antarctica/Mawson", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Antarctica/McMurdo", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "Antarctica/Palmer", "(GMT-04.00) Santiago" );
	java2zimbra.put( "Antarctica/Rothera", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "Antarctica/South_Pole", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "Antarctica/Syowa", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Antarctica/Vostok", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Arctic/Longyearbyen", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Asia/Aden", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Asia/Almaty", "(GMT+06.00) Almaty / Novosibirsk" );
	java2zimbra.put( "Asia/Amman", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Anadyr", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "Asia/Aqtau", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "Asia/Aqtobe", "(GMT+05.00) Ekaterinburg" );
	java2zimbra.put( "Asia/Ashgabat", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Asia/Ashkhabad", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Asia/Baghdad", "(GMT+03.00) Baghdad" );
	java2zimbra.put( "Asia/Bahrain", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Asia/Baku", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "Asia/Bangkok", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Beirut", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Bishkek", "(GMT+05.00) Ekaterinburg" );
	java2zimbra.put( "Asia/Brunei", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Calcutta", "(GMT+05.30) Chennai / Kolkata / Mumbai / New Delhi" );
	java2zimbra.put( "Asia/Choibalsan", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Asia/Chongqing", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Chungking", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Colombo", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Asia/Dacca", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Asia/Damascus", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Dhaka", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Asia/Dili", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Asia/Dubai", "(GMT+04.00) Abu Dhabi / Muscat" );
	java2zimbra.put( "Asia/Dushanbe", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Asia/Gaza", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Harbin", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Hong_Kong", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Hovd", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Irkutsk", "(GMT+08.00) Irkutsk / Ulaan Bataar" );
	java2zimbra.put( "Asia/Istanbul", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Jakarta", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Jayapura", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Asia/Jerusalem", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Kabul", "(GMT+04.30) Kabul" );
	java2zimbra.put( "Asia/Kamchatka", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "Asia/Karachi", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Asia/Kashgar", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Katmandu", "(GMT+05.45) Kathmandu" );
	java2zimbra.put( "Asia/Krasnoyarsk", "(GMT+07.00) Krasnoyarsk" );
	java2zimbra.put( "Asia/Kuala_Lumpur", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Kuching", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Kuwait", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Asia/Macao", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Macau", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Makassar", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Manila", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Muscat", "(GMT+04.00) Abu Dhabi / Muscat" );
	java2zimbra.put( "Asia/Nicosia", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Novosibirsk", "(GMT+06.00) Almaty / Novosibirsk" );
	java2zimbra.put( "Asia/Omsk", "(GMT+06.00) Almaty / Novosibirsk" );
	java2zimbra.put( "Asia/Oral", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "Asia/Phnom_Penh", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Pontianak", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Pyongyang", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Asia/Qatar", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Asia/Qyzylorda", "(GMT+06.00) Almaty / Novosibirsk" );
	java2zimbra.put( "Asia/Rangoon", "(GMT+06.30) Rangoon" );
	java2zimbra.put( "Asia/Riyadh", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Asia/Saigon", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Sakhalin", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Asia/Samarkand", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Asia/Seoul", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Asia/Shanghai", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Singapore", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Taipei", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Tashkent", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Asia/Tbilisi", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "Asia/Tehran", "(GMT+03.30) Tehran" );
	java2zimbra.put( "Asia/Tel_Aviv", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Asia/Thimbu", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Asia/Thimphu", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Asia/Tokyo", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Asia/Ujung_Pandang", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Ulaanbaatar", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Ulan_Bator", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Urumqi", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Asia/Vientiane", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Asia/Vladivostok", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Asia/Yakutsk", "(GMT+09.00) Yakutsk" );
	java2zimbra.put( "Asia/Yekaterinburg", "(GMT+05.00) Ekaterinburg" );
	java2zimbra.put( "Asia/Yerevan", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "Atlantic/Azores", "(GMT-01.00) Azores" );
	java2zimbra.put( "Atlantic/Bermuda", "(GMT-04.00) Santiago" );
	java2zimbra.put( "Atlantic/Canary", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Atlantic/Cape_Verde", "(GMT-01.00) Cape Verde Is." );
	java2zimbra.put( "Atlantic/Faeroe", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Atlantic/Jan_Mayen", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Atlantic/Madeira", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Atlantic/Reykjavik", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Atlantic/St_Helena", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Atlantic/Stanley", "(GMT-04.00) Santiago" );
	java2zimbra.put( "Australia/ACT", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/Adelaide", "(GMT+09.30) Adelaide" );
	java2zimbra.put( "Australia/Brisbane", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Australia/Broken_Hill", "(GMT+09.30) Adelaide" );
	java2zimbra.put( "Australia/Canberra", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/Darwin", "(GMT+09.30) Darwin" );
	java2zimbra.put( "Australia/Hobart", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/Lindeman", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Australia/Melbourne", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/NSW", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/North", "(GMT+09.30) Darwin" );
	java2zimbra.put( "Australia/Perth", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Australia/Queensland", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Australia/South", "(GMT+09.30) Adelaide" );
	java2zimbra.put( "Australia/Sydney", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/Tasmania", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/Victoria", "(GMT+10.00) Canberra / Melbourne / Sydney" );
	java2zimbra.put( "Australia/West", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Australia/Yancowinna", "(GMT+09.30) Adelaide" );
	java2zimbra.put( "BET", "(GMT-03.00) Brasilia" );
	java2zimbra.put( "BST", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Brazil/Acre", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "Brazil/East", "(GMT-03.00) Brasilia" );
	java2zimbra.put( "Brazil/West", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "CAT", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "CET", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "CNT", "(GMT-03.30) Newfoundland" );
	java2zimbra.put( "CST", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "CST6CDT", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "CTT", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Canada/Atlantic", "(GMT-04.00) Santiago" );
	java2zimbra.put( "Canada/Central", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "Canada/East-Saskatchewan", "(GMT-06.00) Central America" );
	java2zimbra.put( "Canada/Eastern", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "Canada/Mountain", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "Canada/Newfoundland", "(GMT-03.30) Newfoundland" );
	java2zimbra.put( "Canada/Pacific", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "Canada/Saskatchewan", "(GMT-06.00) Central America" );
	java2zimbra.put( "Canada/Yukon", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "Chile/Continental", "(GMT-04.00) Santiago" );
	java2zimbra.put( "Chile/EasterIsland", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "Cuba", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "EAT", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "ECT", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "EET", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "EST", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "EST5EDT", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "Egypt", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Eire", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Etc/GMT", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/GMT+0", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/GMT+1", "(GMT-01.00) Cape Verde Is." );
	java2zimbra.put( "Etc/GMT+10", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Etc/GMT+11", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "Etc/GMT+12", "(GMT-12.00) International Date Line West" );
	java2zimbra.put( "Etc/GMT+3", "(GMT-03.00) Buenos Aires / Georgetown" );
	java2zimbra.put( "Etc/GMT+4", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "Etc/GMT+5", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "Etc/GMT+6", "(GMT-06.00) Central America" );
	java2zimbra.put( "Etc/GMT+7", "(GMT-07.00) Arizona" );
	java2zimbra.put( "Etc/GMT-0", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/GMT-1", "(GMT+01.00) West Central Africa" );
	java2zimbra.put( "Etc/GMT-10", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Etc/GMT-11", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Etc/GMT-12", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Etc/GMT-2", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "Etc/GMT-3", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Etc/GMT-4", "(GMT+04.00) Abu Dhabi / Muscat" );
	java2zimbra.put( "Etc/GMT-5", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Etc/GMT-6", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Etc/GMT-7", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Etc/GMT-8", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "Etc/GMT-9", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Etc/GMT0", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/Greenwich", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/UCT", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/UTC", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/Universal", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Etc/Zulu", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Europe/Amsterdam", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Andorra", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Athens", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Belfast", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Europe/Belgrade", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Berlin", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Bratislava", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Brussels", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Bucharest", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Budapest", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Chisinau", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Copenhagen", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Dublin", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Europe/Gibraltar", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Helsinki", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Istanbul", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Kaliningrad", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Kiev", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Lisbon", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Europe/Ljubljana", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/London", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Europe/Luxembourg", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Madrid", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Malta", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Minsk", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Monaco", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Moscow", "(GMT+03.00) Baghdad" );
	java2zimbra.put( "Europe/Nicosia", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Oslo", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Paris", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Prague", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Riga", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Rome", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Samara", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "Europe/San_Marino", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Sarajevo", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Simferopol", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Skopje", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Sofia", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Stockholm", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Tallinn", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Tirane", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Tiraspol", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Uzhgorod", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Vaduz", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Vatican", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Vienna", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Vilnius", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Warsaw", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Zagreb", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Europe/Zaporozhye", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "Europe/Zurich", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "GB", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "GB-Eire", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "GMT", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "GMT0", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Greenwich", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "HST", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Hongkong", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "IET", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "IST", "(GMT+05.30) Chennai / Kolkata / Mumbai / New Delhi" );
	java2zimbra.put( "Iceland", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Indian/Antananarivo", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Indian/Chagos", "(GMT+06.00) Astana / Dhaka" );
	java2zimbra.put( "Indian/Christmas", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "Indian/Cocos", "(GMT+06.30) Rangoon" );
	java2zimbra.put( "Indian/Comoro", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Indian/Kerguelen", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Indian/Mahe", "(GMT+04.00) Abu Dhabi / Muscat" );
	java2zimbra.put( "Indian/Maldives", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "Indian/Mauritius", "(GMT+04.00) Abu Dhabi / Muscat" );
	java2zimbra.put( "Indian/Mayotte", "(GMT+03.00) Kuwait / Riyadh" );
	java2zimbra.put( "Indian/Reunion", "(GMT+04.00) Abu Dhabi / Muscat" );
	java2zimbra.put( "Iran", "(GMT+03.30) Tehran" );
	java2zimbra.put( "Israel", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "JST", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Jamaica", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "Japan", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Kwajalein", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Libya", "(GMT+02.00) Harare / Pretoria" );
	java2zimbra.put( "MET", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "MIT", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "MST", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "MST7MDT", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "Mexico/BajaNorte", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "Mexico/BajaSur", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "Mexico/General", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "NET", "(GMT+04.00) Baku / Tbilisi / Yerevan" );
	java2zimbra.put( "NST", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "NZ", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "Navajo", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "PLT", "(GMT+05.00) Islamabad / Karachi / Tashkent" );
	java2zimbra.put( "PNT", "(GMT-07.00) Arizona" );
	java2zimbra.put( "PRC", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "PRT", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "PST", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "PST8PDT", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "Pacific/Apia", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "Pacific/Auckland", "(GMT+12.00) Auckland / Wellington" );
	java2zimbra.put( "Pacific/Easter", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "Pacific/Efate", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Pacific/Fakaofo", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Pacific/Fiji", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Funafuti", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Galapagos", "(GMT-06.00) Central America" );
	java2zimbra.put( "Pacific/Guadalcanal", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Pacific/Guam", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Pacific/Honolulu", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Pacific/Johnston", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Pacific/Kosrae", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Pacific/Kwajalein", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Majuro", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Midway", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "Pacific/Nauru", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Niue", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "Pacific/Noumea", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Pacific/Pago_Pago", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "Pacific/Palau", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "Pacific/Ponape", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Pacific/Port_Moresby", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Pacific/Rarotonga", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Pacific/Saipan", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Pacific/Samoa", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "Pacific/Tahiti", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "Pacific/Tarawa", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Truk", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Pacific/Wake", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Wallis", "(GMT+12.00) Fiji / Kamchatka / Marshall Is." );
	java2zimbra.put( "Pacific/Yap", "(GMT+10.00) Brisbane" );
	java2zimbra.put( "Poland", "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna" );
	java2zimbra.put( "Portugal", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "ROK", "(GMT+09.00) Osaka / Sapporo / Tokyo" );
	java2zimbra.put( "SST", "(GMT+11.00) Magadan / Solomon Is. / New Calenodia" );
	java2zimbra.put( "Singapore", "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi" );
	java2zimbra.put( "SystemV/AST4", "(GMT-04.00) Caracas / La Paz" );
	java2zimbra.put( "SystemV/AST4ADT", "(GMT-04.00) Santiago" );
	java2zimbra.put( "SystemV/CST6", "(GMT-06.00) Central America" );
	java2zimbra.put( "SystemV/CST6CDT", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "SystemV/EST5", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "SystemV/EST5EDT", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "SystemV/HST10", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "SystemV/MST7", "(GMT-07.00) Arizona" );
	java2zimbra.put( "SystemV/MST7MDT", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "SystemV/PST8PDT", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "SystemV/YST9YDT", "(GMT-09.00) Alaska" );
	java2zimbra.put( "Turkey", "(GMT+02.00) Athens / Beirut / Istanbul / Minsk" );
	java2zimbra.put( "UCT", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "US/Alaska", "(GMT-09.00) Alaska" );
	java2zimbra.put( "US/Arizona", "(GMT-07.00) Arizona" );
	java2zimbra.put( "US/Central", "(GMT-06.00) Central Time (US & Canada)" );
	java2zimbra.put( "US/East-Indiana", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "US/Eastern", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "US/Hawaii", "(GMT-10.00) Hawaii" );
	java2zimbra.put( "US/Indiana-Starke", "(GMT-05.00) Bogota / Lima / Quito" );
	java2zimbra.put( "US/Michigan", "(GMT-05.00) Eastern Time (US & Canada)" );
	java2zimbra.put( "US/Mountain", "(GMT-07.00) Chihuahua / La Paz / Mazatlan" );
	java2zimbra.put( "US/Pacific", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "US/Pacific-New", "(GMT-08.00) Pacific Time (US & Canada) / Tijuana" );
	java2zimbra.put( "US/Samoa", "(GMT-11.00) Midway Island / Samoa" );
	java2zimbra.put( "UTC", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "Universal", "(GMT) Casablanca / Monrovia" );
	java2zimbra.put( "VST", "(GMT+07.00) Bangkok / Hanoi / Jakarta" );
	java2zimbra.put( "W-SU", "(GMT+03.00) Baghdad" );
	java2zimbra.put( "WET", "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London" );
	java2zimbra.put( "Zulu", "(GMT) Casablanca / Monrovia" );
        // Java timezones that do not have a corresponding Zimbra timezone.
        //
        // -10:00	+01:00	America/Adak
        // -10:00	+01:00	America/Atka
        // -02:00	      	America/Noronha
        // +11:00	+01:00	Asia/Magadan
        // +03:07	      	Asia/Riyadh87
        // +03:07	      	Asia/Riyadh88
        // +03:07	      	Asia/Riyadh89
        // -02:00	      	Atlantic/South_Georgia
        // +10:30	+00:30	Australia/LHI
        // +10:30	+00:30	Australia/Lord_Howe
        // -02:00	      	Brazil/DeNoronha
        // -02:00	      	Etc/GMT+2
        // -08:00	      	Etc/GMT+8
        // -09:00	      	Etc/GMT+9
        // +13:00	      	Etc/GMT-13
        // +14:00	      	Etc/GMT-14
        // +03:07	      	Mideast/Riyadh87
        // +03:07	      	Mideast/Riyadh88
        // +03:07	      	Mideast/Riyadh89
        // +12:45	+01:00	NZ-CHAT
        // +12:45	+01:00	Pacific/Chatham
        // +13:00	      	Pacific/Enderbury
        // -09:00	      	Pacific/Gambier
        // +14:00	      	Pacific/Kiritimati
        // -09:30	      	Pacific/Marquesas
        // +11:30	      	Pacific/Norfolk
        // -08:00	      	Pacific/Pitcairn
        // +13:00	      	Pacific/Tongatapu
        // -08:00	      	SystemV/PST8
        // -09:00	      	SystemV/YST9
        // -10:00	+01:00	US/Aleutian
        //

        // populate the zimbra2java mapping
	zimbra2java.put( "(GMT-12.00) International Date Line West", "Etc/GMT+12" );
	zimbra2java.put( "(GMT-11.00) Midway Island / Samoa", "Etc/GMT+11" );
	zimbra2java.put( "(GMT-10.00) Hawaii", "Etc/GMT+10" );
	zimbra2java.put( "(GMT-09.00) Alaska", "AST" );
	zimbra2java.put( "(GMT-08.00) Pacific Time (US & Canada) / Tijuana", "America/Dawson" );
	zimbra2java.put( "(GMT-07.00) Arizona", "America/Dawson_Creek" );
	zimbra2java.put( "(GMT-07.00) Chihuahua / La Paz / Mazatlan", "America/Boise" );
	zimbra2java.put( "(GMT-07.00) Mountain Time (US & Canada)", "America/Boise" );
	zimbra2java.put( "(GMT-06.00) Central America", "America/Belize" );
	zimbra2java.put( "(GMT-06.00) Central Time (US & Canada)", "America/Cancun" );
	zimbra2java.put( "(GMT-06.00) Guadalajara / Mexico City / Monterrey", "America/Cancun" );
	zimbra2java.put( "(GMT-06.00) Saskatchewan", "America/Belize" );
	zimbra2java.put( "(GMT-05.00) Bogota / Lima / Quito", "America/Bogota" );
	zimbra2java.put( "(GMT-05.00) Eastern Time (US & Canada)", "America/Detroit" );
	zimbra2java.put( "(GMT-05.00) Indiana (East)", "America/Bogota" );
	zimbra2java.put( "(GMT-04.00) Atlantic Time (Canada)", "America/Bogota" );
	zimbra2java.put( "(GMT-04.00) Caracas / La Paz", "America/Anguilla" );
	zimbra2java.put( "(GMT-04.00) Santiago", "America/Asuncion" );
	zimbra2java.put( "(GMT-03.30) Newfoundland", "America/St_Johns" );
	zimbra2java.put( "(GMT-03.00) Brasilia", "America/Godthab" );
	zimbra2java.put( "(GMT-03.00) Buenos Aires / Georgetown", "AGT" );
	zimbra2java.put( "(GMT-03.00) Greenland", "America/Godthab" );
	zimbra2java.put( "(GMT-01.00) Azores", "America/Scoresbysund" );
	zimbra2java.put( "(GMT-01.00) Cape Verde Is.", "Atlantic/Cape_Verde" );
	zimbra2java.put( "(GMT) Casablanca / Monrovia", "Africa/Abidjan" );
	zimbra2java.put( "(GMT) Greenwich Mean Time - Dublin / Edinburgh / Lisbon / London", "Atlantic/Canary" );
	zimbra2java.put( "(GMT+01.00) Amsterdam / Berlin / Bern / Rome / Stockholm / Vienna", "Africa/Ceuta" );
	zimbra2java.put( "(GMT+01.00) Belgrade / Bratislava / Budapest / Ljubljana / Prague", "Africa/Ceuta" );
	zimbra2java.put( "(GMT+01.00) Brussels / Copenhagen / Madrid / Paris", "Africa/Ceuta" );
	zimbra2java.put( "(GMT+01.00) Sarajevo / Skopje / Warsaw / Zagreb", "Africa/Ceuta" );
	zimbra2java.put( "(GMT+01.00) West Central Africa", "Africa/Algiers" );
	zimbra2java.put( "(GMT+02.00) Athens / Beirut / Istanbul / Minsk", "ART" );
	zimbra2java.put( "(GMT+02.00) Bucharest", "ART" );
	zimbra2java.put( "(GMT+02.00) Cairo", "ART" );
	zimbra2java.put( "(GMT+02.00) Harare / Pretoria", "Africa/Blantyre" );
	zimbra2java.put( "(GMT+02.00) Helsinki / Kyiv / Riga / Sofia / Tallinn / Vilnius", "ART" );
	zimbra2java.put( "(GMT+02.00) Jerusalem", "Africa/Blantyre" );
	zimbra2java.put( "(GMT+03.00) Baghdad", "Asia/Baghdad" );
	zimbra2java.put( "(GMT+03.00) Kuwait / Riyadh", "Africa/Addis_Ababa" );
	zimbra2java.put( "(GMT+03.00) Moscow / St. Petersburg / Volgograd", "Asia/Baghdad" );
	zimbra2java.put( "(GMT+03.00) Nairobi", "Africa/Addis_Ababa" );
	zimbra2java.put( "(GMT+03.30) Tehran", "Asia/Tehran" );
	zimbra2java.put( "(GMT+04.00) Abu Dhabi / Muscat", "Asia/Dubai" );
	zimbra2java.put( "(GMT+04.00) Baku / Tbilisi / Yerevan", "Asia/Aqtau" );
	zimbra2java.put( "(GMT+04.30) Kabul", "Asia/Kabul" );
	zimbra2java.put( "(GMT+05.00) Ekaterinburg", "Asia/Aqtobe" );
	zimbra2java.put( "(GMT+05.00) Islamabad / Karachi / Tashkent", "Asia/Ashgabat" );
	zimbra2java.put( "(GMT+05.30) Chennai / Kolkata / Mumbai / New Delhi", "Asia/Calcutta" );
	zimbra2java.put( "(GMT+05.45) Kathmandu", "Asia/Katmandu" );
	zimbra2java.put( "(GMT+06.00) Almaty / Novosibirsk", "Asia/Almaty" );
	zimbra2java.put( "(GMT+06.00) Astana / Dhaka", "Antarctica/Mawson" );
	zimbra2java.put( "(GMT+06.00) Sri Jayawardenepura", "Antarctica/Mawson" );
	zimbra2java.put( "(GMT+06.30) Rangoon", "Asia/Rangoon" );
	zimbra2java.put( "(GMT+07.00) Bangkok / Hanoi / Jakarta", "Antarctica/Davis" );
	zimbra2java.put( "(GMT+07.00) Krasnoyarsk", "Asia/Krasnoyarsk" );
	zimbra2java.put( "(GMT+08.00) Beijing / Chongqing / Hong Kong / Urumqi", "Antarctica/Casey" );
	zimbra2java.put( "(GMT+08.00) Irkutsk / Ulaan Bataar", "Asia/Irkutsk" );
	zimbra2java.put( "(GMT+08.00) Kuala Lumpur / Singapore", "Antarctica/Casey" );
	zimbra2java.put( "(GMT+08.00) Perth", "Antarctica/Casey" );
	zimbra2java.put( "(GMT+08.00) Taipei", "Antarctica/Casey" );
	zimbra2java.put( "(GMT+09.00) Osaka / Sapporo / Tokyo", "Asia/Choibalsan" );
	zimbra2java.put( "(GMT+09.00) Seoul", "Asia/Choibalsan" );
	zimbra2java.put( "(GMT+09.00) Yakutsk", "Asia/Yakutsk" );
	zimbra2java.put( "(GMT+09.30) Adelaide", "Australia/Adelaide" );
	zimbra2java.put( "(GMT+09.30) Darwin", "ACT" );
	zimbra2java.put( "(GMT+10.00) Brisbane", "Antarctica/DumontDUrville" );
	zimbra2java.put( "(GMT+10.00) Canberra / Melbourne / Sydney", "AET" );
	zimbra2java.put( "(GMT+10.00) Guam / Port Moresby", "Antarctica/DumontDUrville" );
	zimbra2java.put( "(GMT+10.00) Hobart", "AET" );
	zimbra2java.put( "(GMT+10.00) Vladivostok", "AET" );
	zimbra2java.put( "(GMT+11.00) Magadan / Solomon Is. / New Calenodia", "Etc/GMT-11" );
	zimbra2java.put( "(GMT+12.00) Auckland / Wellington", "Antarctica/McMurdo" );
	zimbra2java.put( "(GMT+12.00) Fiji / Kamchatka / Marshall Is.", "Etc/GMT-12" );
        // Zimbra timezones that do not have a corresponding Java timezone.
        //
        // -02:00	+01:00	(GMT-02.00) Mid-Atlantic
        //
    };

    // - kal -
}

