package zimbra.soap;

import zimbra.soap.util.AuthDetails;
import zimbra.soap.response.AuthResponse;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.util.HashMap;
import java.util.logging.Logger;

import sync4j.zimbra.util.Def;

/**
 * parse handler for a AuthResponse response on SOAP protocol
 */
public class AuthResponseHandler extends ResponseHandler {

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    /**
     * private member that a given time in the XML parsing
     * will contain the name of the last element found
     */
    private String crtElement;
    /**
     * private member that a given time in the XML parsing
     * will contain the valueof the last element found
     */
    private String crtValue;

    private String timeZone = new String();

    /**
     * HashMap used to construct an AuthResponseImpl
     * when the getAuthResponse is called
     */
    private AuthDetails authResponseFields = new AuthDetails();

    /**
     * variable showing if the start authRespose element
     * has been found
     */
    private boolean foundAuthResponseStart;
    /**
     * variable showing if the end authRespose element
     * has been reached
     */
    private boolean foundAuthResponseEnd;

    private boolean isTimeZone;

    /**
     * default constructor
     */
    public AuthResponseHandler() {

    }

    /**
     * method called by the xml parser when the begining
     * of the XML document is found
     *
     * @throws SAXException
     */

    public void startDocument()
            throws SAXException {
        this.foundAuthResponseStart = false;
        this.foundAuthResponseEnd = false;
        this.authResponseFields.clear();
    }

    /**
     * method called by the xml parser when the end
     * of the XML document is found
     *
     * @throws SAXException
     */
    public void endDocument()
            throws SAXException {

        //if all important elements of the authentication response have been found,
        //is ok
        if (this.authResponseFields.containsKey(AuthDetails.K_AUTH_TOKEN) &&
                this.authResponseFields.containsKey(AuthDetails.K_LIFETIME) 
	//	&& this.authResponseFields.containsKey(AuthDetails.K_SESSION_ID)
		) {
            return;
        }

        throw new SAXException("Authentication Response broken!");
    }

    /**
     * handler of the event that occurs when the xml parser
     * found a new element in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @param attrs
     * @throws SAXException
     */
    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
            throws SAXException {

        //element's name
        String eName = localName;
        if ("".equals(eName)) eName = qName;

        isTimeZone = false;
        if (attrs.getLength() > 0 && attrs.getValue(0).equalsIgnoreCase("zimbraPrefTimeZoneId")) {
            this.crtElement = eName;
            isTimeZone = true;
        }


        if ("AuthResponse".equals(eName)) {
            this.foundAuthResponseStart = true;
            return;
        }
        //if the start of AuthRespose hasn't been found
        //element is not important
        if (!this.foundAuthResponseStart) return;
        if ("authToken".equals(eName) ||
                "lifetime".equals(eName) ||
                "sessionId".equals(eName)) {
            this.crtElement = eName;
        }
        if ("sessionId".equals(eName)) {
            this.crtValue = attrs.getValue("id");
        }
    }

    /**
     * handler of the event that occurs when the xml parser
     * founds the end of element's declaration in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @throws SAXException
     */
    public void endElement(String uri,
                           String localName,
                           String qName)
            throws SAXException {


        //if the end of the body has previously been found,
        // nothing mathers anymore
        if (this.foundAuthResponseEnd) return;
        //element's name
        String eName = localName;


        if ("".equals(eName)) eName = qName;

        if ("AuthResponse".equals(eName)) {
            this.foundAuthResponseEnd = true;
            return;
        }
        if ("authToken".equals(eName)) {
            this.authResponseFields.put(AuthDetails.K_AUTH_TOKEN, crtValue);
            return;
        }
        if ("lifetime".equals(eName)) {
            this.authResponseFields.put(AuthDetails.K_LIFETIME, crtValue);
            return;
        }
        if ("sessionId".equals(eName)) {
            this.authResponseFields.put(AuthDetails.K_SESSION_ID, crtValue);
            return;
        }
        if ("pref".equals(eName) && isTimeZone) {
            log.finest("Setting timezone to " + timeZone);
            this.authResponseFields.put(AuthDetails.K_TIME_ZONE, timeZone);
            timeZone = "";
        }

    }

    /**
     * handler of the event that occurs when character are found
     * inx the xml document
     *
     * @param charBuff
     * @param start
     * @param length
     * @throws SAXException
     */
    public void characters(char[] charBuff, int start, int length)
            throws SAXException {
        if ("authToken".equals(crtElement) ||
                "lifetime".equals(crtElement) ||
                "sessionID".equals(crtElement) ||
                ("pref".equalsIgnoreCase(crtElement) && isTimeZone)) {

            this.crtValue = new String(charBuff, start, length);
            if (isTimeZone) {
                timeZone += crtValue;
            }

        }
    }

    /**
     * @return a new AuthResonse object with the propreties obtained
     *         by parsing the xml document
     */
    public Object[] getOutcome() {
        return new HashMap[]{this.authResponseFields};
    }

}
