package zimbra.soap;

import java.net.URL;
import java.net.HttpURLConnection;

import java.io.*;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.logging.Logger;

import javax.xml.parsers.*;

import zimbra.soap.exceptions.*;
import zimbra.soap.util.*;
import org.xml.sax.SAXException;
import sync4j.zimbra.util.Def;

public class SOAPTransporter {

    //general requests

    public static final String M_AUTHENTICATE =
            "<ns0:AuthRequest xmlns:ns0=\"urn:zimbraAccount\">" +
            "<account by=\"name\">{0}</account>" +
            "<password>{1}</password>" +
            "<prefs><pref name=\"zimbraPrefTimeZoneId\"/></prefs>" +
            "</ns0:AuthRequest>";

    public static final String M_GET_ACCOUNT_INFO =
            "<GetAccountInfoRequest xmlns=\"urn:zimbraAccount\">" +
            "<account by=\"name\">{0}</account>" +
            "</GetAccountInfoRequest>";

    public static final String M_ITEM_DELETE =
            "<ItemActionRequest xmlns=\"urn:zimbraMail\">" +
            "<action op=\"delete\" id=\"{0}\" />" +
            "</ItemActionRequest>";
    public static final String M_ITEM_MOVE =
            "<ItemActionRequest xmlns=\"urn:zimbraMail\">" +
            "<action op=\"move\" id=\"{0}\" l=\"{1}\" />" +
            "</ItemActionRequest>";

    //Contacts related requests

    public static final String M_GET_ALL_CONTACTS =
            "<GetContactsRequest xmlns=\"urn:zimbraMail\"/>";

    public static final String M_GET_CONTACT_BY_ID =
            "<GetContactsRequest xmlns=\"urn:zimbraMail\">" +
            "<cn id=\"{0}\"/>" +
            "</GetContactsRequest>";

    public static final String M_CREATE_CONTACT =
            "<CreateContactRequest xmlns=\"urn:zimbraMail\">" +
            "<cn>{0}</cn>" +
            "</CreateContactRequest>";

    public static final String M_MODIFY_CONTACT =
            "<ModifyContactRequest replace=\"0\" xmlns=\"urn:zimbraMail\">" +
            "<cn id=\"{0}\">{1}</cn>" +
            "</ModifyContactRequest>";

    public static final String M_CONTACT_ACTION =
            "<ContactActionRequest xmlns=\"urn:zimbraMail\">" +
            "<action op=\"{0}\" id=\"{1}\"/>" +
            "</ContactActionRequest>";
    public static final String M_SYNC =
            "<SyncRequest token=\"{0}\" xmlns=\"urn:zimbraMail\"/>";

    //request patterns for Calendars

    public static final String M_GET_CALENDAR_BY_ID =
            "<GetAppointmentRequest id=\"{0}\" xmlns=\"urn:zimbraMail\"/>";

    public static final String M_GET_ALL_CALENDARS_IDS =
            "<SyncRequest xmlns=\"urn:zimbraMail\"/>";

    public static final String M_CREATE_CALENDAR =
            "<CreateAppointmentRequest xmlns=\"urn:zimbraMail\">" +
            "<m>" +
            "<inv type=\"{0}\" status=\"{1}\" fb=\"{2}\" fba=\"{3}\" transp=\"{4}\" allDay=\"{5}\" name=\"{6}\" loc=\"{7}\">" +
            "<or d=\"{8}\" a=\"{9}\"/>" +
            "{10}" +
            "<s d=\"{11}\" tz=\"{12}\"/>" +
            "<e d=\"{13}\" tz=\"{14}\"/>" +
            "{15}" +
            "</inv>" +
            "</m>" +
            "</CreateAppointmentRequest>";

    public static final String M_MODIFY_CALENDAR =
            "<ModifyAppointmentRequest id=\"{0}\" compNum=\"{1}\" xmlns=\"urn:zimbraMail\">" +
            "<m>" +
            "<inv uid=\"{2}\" type=\"{3}\" status=\"{4}\" fb=\"{5}\" fba=\"{6}\" transp=\"{7}\" allDay=\"{8}\" name=\"{9}\" loc=\"{10}\">" +
            "<or d=\"{11}\" a=\"{12}\"/>" +
            "{13}" +
            "<s d=\"{14}\" tz=\"{15}\"/>" +
            "<e d=\"{16}\" tz=\"{17}\"/>" +
            "{18}" +
            "</inv>" +
            "</m>" +
            "</ModifyAppointmentRequest>";

    public static final String M_GET_MSG_REQUEST =
            "<GetMsgRequest xmlns=\"urn:zimbraMail\"> " +
            "<m id=\"{0}-{1}\" read=\"1\" html=\"1\"/> " +
            "</GetMsgRequest> ";


    private URL serverURL;
    private SAXParser xmlParser;

    private String authToken = "";
    private String sessionId = "";
    private String userId = "";
    private String timeZone = "";

    private boolean isAuthenticated = false;

    protected Logger log = Logger.getLogger(Def.LOGGER_NAME);

    protected SOAPTransporter() {
    }

    public SOAPTransporter(String serverURLSpec)
            throws SOAPException {
        try {
            this.serverURL = new URL(serverURLSpec);
            SAXParserFactory factory = SAXParserFactory.newInstance();
            this.xmlParser = factory.newSAXParser();
        } catch (Throwable e) {
            throw new SOAPException(e.getMessage(), e);
        }

    }

    public Object[] sendRequest(String request, ResponseHandler responseHandler)
            throws SOAPException, IOException {
        if (!(responseHandler instanceof AuthResponseHandler) && !this.isAuthenticated)
            throw new SOAPException("Not autheticated!");
        try {
            HttpURLConnection serverConn = this.getConnection();
            Writer serverWriter = new OutputStreamWriter(serverConn.getOutputStream());

            log.finest("SENDING REQUEST: " + request);

            serverWriter.write(this.requestInContext(request));
            serverWriter.flush();
            serverWriter.close();

            InputStream serverIn = serverConn.getInputStream();

            this.xmlParser.parse(serverIn, responseHandler);

            serverIn.close();
            Object[] outcome = responseHandler.getOutcome();
            if (responseHandler instanceof AuthResponseHandler) {
                this.sessionId = (String) ((HashMap) outcome[0]).get(AuthDetails.K_SESSION_ID);
                this.authToken = (String) ((HashMap) outcome[0]).get(AuthDetails.K_AUTH_TOKEN);
                this.timeZone = (String) ((HashMap) outcome[0]).get(AuthDetails.K_TIME_ZONE);
                this.isAuthenticated = true;
            }
            if (responseHandler instanceof AccountInfoResponseHandler) {
                this.userId = (String) ((HashMap) outcome[0]).get(AccountInfoDetails.K_ZIMBRA_ID);

            }
            serverConn.disconnect();

            return outcome;
        } catch (SOAPException e) {
            throw new SOAPException(e);
        } catch (SAXException e) {
            throw new SOAPException(e);
        }
    }

    public HashMap login(String username, String password)
            throws SOAPException {

        Object[] outcome;

        try {
            this.sendRequest(MessageFormat.format(SOAPTransporter.M_AUTHENTICATE,
                    new Object[]{username, password}),
                    new AuthResponseHandler());
            outcome =
                    this.sendRequest(MessageFormat.format(SOAPTransporter.M_GET_ACCOUNT_INFO,
                            new Object[]{username}),
                            new AccountInfoResponseHandler());
        } catch (IOException e) {
            throw new SOAPException("Error loging in: " + e.getMessage(), e);
        }
        HashMap loginOutcome = new HashMap();
        loginOutcome.put(AuthDetails.K_SESSION_ID, this.sessionId);
        loginOutcome.put(AuthDetails.K_TIME_ZONE, this.timeZone);
        loginOutcome.put(AccountInfoDetails.K_ZIMBRA_ID, this.userId);
        loginOutcome.put(AccountInfoDetails.K_ZIMBRA_MAIL_HOST,
                ((HashMap) outcome[0]).get(AccountInfoDetails.K_ZIMBRA_MAIL_HOST));
        return loginOutcome;
    }

    public void releaseSession() {
        this.sessionId = "";
        this.authToken = "";
        this.isAuthenticated = false;
    }

    /**
     * Removes an server item given by its id
     *
     * @param itemId
     */
    public void deleteItem(String itemId)
            throws SOAPException {
        try {
            String request = MessageFormat.format(SOAPTransporter.M_ITEM_DELETE,
                    new Object[]{itemId});
            this.sendRequest(request, new ItemActionResponseHandler());
        } catch (SOAPException e) {
            throw new SOAPException("Failed to delete item: " + itemId, e);
        } catch (IOException e) {
            throw new SOAPException("Failed to delete item: " + itemId, e);
        }
    }

    /**
     * Removes an server item given by its id
     *
     * @param itemId
     */
    public void moveItem(String itemId, String folderId)
            throws SOAPException {
        FolderDetails[] folders = this.getFolders();
        String folderName = null;
        int i;
        for (i = 0; i < folders.length; i++)
            if (folderId.equals(folders[i].get(FolderDetails.K_ID))) {
                folderName = (String) folders[i].get(FolderDetails.K_NAME);
                break;
            }
        if (i >= folders.length)
            throw new SOAPException("Invalid folder id " + folderId);
        try {
            String request = MessageFormat.format(SOAPTransporter.M_ITEM_MOVE,
                    new Object[]{itemId, folderId});
            this.sendRequest(request, new ItemActionResponseHandler());
        } catch (SOAPException e) {
            throw new SOAPException("Failed to move item " + itemId + "to " +
                    folderName + " folder", e);
        } catch (IOException e) {
            throw new SOAPException("Failed to move item " + itemId + "to " +
                    folderName + " folder", e);
        }
    }

    /**
     * Updates the content of a calendar on the Server unsing a CalendarDetails object
     *
     * @param calendarDetails
     * @return the id of the updated created Calendar
     */
    public String updateCalendar(CalendarDetails calendarDetails, String userID)
            throws SOAPException {
        String[] newID;
        StringBuffer recur = new StringBuffer();
        RecurrenceDetails[] recurenceDetails =
                (RecurrenceDetails[]) calendarDetails.get(CalendarDetails.K_RECURRENCE);
        if (recurenceDetails != null && recurenceDetails.length > 0) {
            recur.append("<recur>");
            for (int i = 0; i < recurenceDetails.length; i++)
                recur.append(recurenceDetails[i]);
            recur.append("</recur>");
        }

        try {
            Object obj = calendarDetails.get(CalendarDetails.K_ORGANIZER);
            HashMap organizer = null;
            if (obj != null && obj instanceof HashMap) organizer = (HashMap) obj;
            String organizerFriendlyName = null;
            String organizerAddress = null;
            if (organizer != null) {
                organizerFriendlyName = (String) organizer.get(CalendarDetails.K_FRIENDLY_NAME);
                organizerAddress = (String) organizer.get(CalendarDetails.K_ADDRESS);
                if (organizerFriendlyName == null)
                    organizerFriendlyName = organizerAddress;
            }
            HashMap[] attendees = (HashMap[]) calendarDetails.get(CalendarDetails.K_ATTENDEES);

            StringBuffer sAttendees = new StringBuffer();
            if (attendees != null)
                for (int i = 0; i < attendees.length; i++) {
                    String friendlyName = (String) attendees[i].get(CalendarDetails.K_FRIENDLY_NAME);
                    String address = (String) attendees[i].get(CalendarDetails.K_ADDRESS);
                    String role = (String) attendees[i].get(CalendarDetails.K_ATTENDEE_ROLE);
                    String ptst = (String) attendees[i].get(CalendarDetails.K_ATTENDEE_PTST);
                    if (friendlyName == null || friendlyName.equalsIgnoreCase("NULL")) friendlyName = address;
                    sAttendees.append("<at d=\"").
                            append(friendlyName).
                            append("\" a=\"").
                            append(address).
                            append("\" role=\"").
                            append(role).
                            append("\" ptst=\"").
                            append(ptst).append("\"/>");
                }

            CalendarDetails[] oldCalDets =
                    (CalendarDetails[])
                    this.sendRequest(MessageFormat.format(M_GET_CALENDAR_BY_ID,
                            new Object[]{calendarDetails.get(CalendarDetails.K_ID)}),
                            new GetAppointementResponseHandler());
            String request =
                    MessageFormat.format(SOAPTransporter.M_MODIFY_CALENDAR,
                            new Object[]{
                                calendarDetails.get(CalendarDetails.K_ID) + "-" +
                    oldCalDets[0].get(CalendarDetails.K_DEFUALT_INV_ID),
                                oldCalDets[0].get(CalendarDetails.K_COMP_NUM),
                                userID,
                                calendarDetails.get(CalendarDetails.K_TYPE),
                                calendarDetails.get(CalendarDetails.K_STATUS),
                                calendarDetails.get(CalendarDetails.K_FREE_BUSSY),
                                calendarDetails.get(CalendarDetails.K_ACTUAL_FREE_BUSSY),
                                calendarDetails.get(CalendarDetails.K_TRANSPARENCY),
                                calendarDetails.get(CalendarDetails.K_ALL_DAY),
                                calendarDetails.get(CalendarDetails.K_NAME),
                                calendarDetails.get(CalendarDetails.K_LOCATION),
                                nullToVoid(organizerFriendlyName),
                                nullToVoid(organizerAddress),
                                sAttendees,
                                calendarDetails.get(CalendarDetails.K_START_DATETIME),
                                calendarDetails.get(CalendarDetails.K_TIMEZONE),
                                calendarDetails.get(CalendarDetails.K_END_DATETIME),
                                calendarDetails.get(CalendarDetails.K_TIMEZONE),
                                recur
                            });

            newID = (String[])
                    this.sendRequest(request,
                            new ModifyAppointmentResponseHandler());

        } catch (Exception e) {
            throw new SOAPException("Error modifying calendar: " + e);
        }
        calendarDetails.put(CalendarDetails.K_DEFUALT_INV_ID, newID[0]);
        return newID[0];
    }

    /**
     * Creates a calendar on the Server unsing a CalendarDetails object
     *
     * @param calendarDetails
     * @return the id of the newly created Calendar
     */
    public String createCalendar(CalendarDetails calendarDetails)
            throws SOAPException {
        String[] newID;
        StringBuffer recur = new StringBuffer();
        RecurrenceDetails[] recurenceDetails =
                (RecurrenceDetails[]) calendarDetails.get(CalendarDetails.K_RECURRENCE);
        if (recurenceDetails != null && recurenceDetails.length > 0) {
            recur.append("<recur>");
            for (int i = 0; i < recurenceDetails.length; i++)
                recur.append(recurenceDetails[i]);
            recur.append("</recur>");
        }
        String request = null;
        try {
            Object obj = calendarDetails.get(CalendarDetails.K_ORGANIZER);
            HashMap organizer = null;
            if (obj != null && obj instanceof HashMap) organizer = (HashMap) obj;
            String organizerFriendlyName = null;
            String organizerAddress = null;
            if (organizer != null) {
                organizerFriendlyName = (String) organizer.get(CalendarDetails.K_FRIENDLY_NAME);
                organizerAddress = (String) organizer.get(CalendarDetails.K_ADDRESS);
                if (organizerFriendlyName == null)
                    organizerFriendlyName = organizerAddress;
            }
            HashMap[] attendees = (HashMap[]) calendarDetails.get(CalendarDetails.K_ATTENDEES);
            StringBuffer sAttendees = new StringBuffer();
            if (attendees != null)
                for (int i = 0; i < attendees.length; i++) {
                    String friendlyName = (String) attendees[i].get(CalendarDetails.K_FRIENDLY_NAME);
                    String address = (String) attendees[i].get(CalendarDetails.K_ADDRESS);
                    String role = (String) attendees[i].get(CalendarDetails.K_ATTENDEE_ROLE);
                    String ptst = (String) attendees[i].get(CalendarDetails.K_ATTENDEE_PTST);
                    sAttendees.append("<at d=\"").
                            append(friendlyName).
                            append("\" a=\"").
                            append(address).
                            append("\" role=\"").
                            append(role).
                            append("\" ptst=\"").
                            append(ptst).append("\"/>");
                }

            request =
                    MessageFormat.format(SOAPTransporter.M_CREATE_CALENDAR,
                            new Object[]{
                                calendarDetails.get(CalendarDetails.K_TYPE),
                                calendarDetails.get(CalendarDetails.K_STATUS),
                                calendarDetails.get(CalendarDetails.K_FREE_BUSSY),
                                calendarDetails.get(CalendarDetails.K_ACTUAL_FREE_BUSSY),
                                calendarDetails.get(CalendarDetails.K_TRANSPARENCY),
                                calendarDetails.get(CalendarDetails.K_ALL_DAY),
                                calendarDetails.get(CalendarDetails.K_NAME),
                                calendarDetails.get(CalendarDetails.K_LOCATION),
                                nullToVoid(organizerFriendlyName),
                                nullToVoid(organizerAddress),
                                sAttendees,
                                calendarDetails.get(CalendarDetails.K_START_DATETIME),
                                calendarDetails.get(CalendarDetails.K_TIMEZONE),
                                calendarDetails.get(CalendarDetails.K_END_DATETIME),
                                calendarDetails.get(CalendarDetails.K_TIMEZONE),
                                recur
                            });
            newID = (String[])
                    this.sendRequest(request,
                            new CreateAppointmentResponseHandler());
        } catch (Exception e) {
            throw new SOAPException("Error " + e + "\n while creating calendar with the request: \n" + request);

        }
        return newID[0];
    }

    public FolderDetails[] getFolders()
            throws SOAPException {
        FolderDetails[] folderDetails;
        try {
            folderDetails = (FolderDetails[])
                    this.sendRequest("<GetFolderRequest xmlns=\"urn:zimbraMail\" />",
                            new GetFolderResponseHandler());
        } catch (SOAPException e) {
            throw new SOAPException("Error retrieving folders: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new SOAPException("Error retrieving folders: " + e.getMessage(), e);
        }
        if (folderDetails == null) return new FolderDetails[0];
        return folderDetails;
    }

    public String getTrashFolderId()
            throws SOAPException {
        FolderDetails[] folders = this.getFolders();
        String trashId = null;
        for (int i = 0; i < folders.length; i++) {
            if ("Trash".equalsIgnoreCase((String) folders[i].get(FolderDetails.K_NAME))) {
                trashId = (String) folders[i].get(FolderDetails.K_ID);
                break;
            }
        }
        if (trashId == null)
            throw new SOAPException("Thrash folder not found");
        return trashId;
    }

    private String requestInContext(String xmlRequest) {
        return
                "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
                "<soap:Envelope xmlns:soap=\"http://www.w3.org/2003/05/soap-envelope\">" +
                "<soap:Header>" +
                "<context xmlns=\"urn:zimbra\">" +
                "<authToken>" + this.authToken + "</authToken>" +
                "<sessionId id=\"" + this.sessionId + "\">" + this.sessionId + "</sessionId>" +
 //               "<nonotify/>" +
                "</context>" +
                "</soap:Header>" +
                "<soap:Body>" +
                xmlRequest +
                "</soap:Body>" +
                "</soap:Envelope>";
    }

    private HttpURLConnection getConnection()
            throws SOAPException {
        try {
            HttpURLConnection serverConn = (HttpURLConnection) this.serverURL.openConnection();
            serverConn.setRequestMethod("POST");
            serverConn.setDoOutput(true);
            serverConn.setDoInput(true);
            serverConn.setUseCaches(false);
	    serverConn.setRequestProperty("Content-Type","application/soap+xml");
            return serverConn;
        } catch (java.io.IOException e) {
            throw new SOAPException(e);
        }
    }

    private static Object nullToVoid(Object obj) {
        if (obj == null || "null".equals(obj)) return "";
        return obj;
    }
}
