package zimbra.soap.response;

import java.util.HashMap;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * parse handler for a AuthResponse response on SOAP protocol
 */
public class AuthResponseHandler extends DefaultHandler {
    /**
     * private member that a given time in the XML parsing
     * will contain the name of the last element found
     */
    private String crtElement;
    /**
     * private member that a given time in the XML parsing
     * will contain the valueof the last element found
     */
    private String crtValue;

    /**
     * HashMap used to construct an AuthResponseImpl
     * when the getAuthResponse is called
     */
    private HashMap authResponseFields = new HashMap();

    /**
     * variable showing if the start authRespose element
     * has been found
     */
    private boolean foundAuthResponseStart;
    /**
     * variable showing if the end authRespose element
     * has been reached
     */
    private boolean foundAuthResponseEnd;

    /**
     * checks if we should read the timezone
     */
    private boolean isTimeZone;

    /**
     * default constructor
     */
    public AuthResponseHandler() {

    }

    /**
     * method called by the xml parser when the begining
     * of the XML document is found
     *
     * @throws SAXException
     */

    public void startDocument()
            throws SAXException {
        this.foundAuthResponseStart = false;
        this.foundAuthResponseEnd = false;
        this.authResponseFields.clear();
    }

    /**
     * method called by the xml parser when the end
     * of the XML document is found
     *
     * @throws SAXException
     */
    public void endDocument()
            throws SAXException {
        //if all important elements of the authentication response have been found,
        //is ok
        if (this.authResponseFields.containsKey(AuthResponse.K_AUTH_TOKEN) &&
                this.authResponseFields.containsKey(AuthResponse.K_LIFETIME) &&
                this.authResponseFields.containsKey(AuthResponse.K_SESSION_ID)) {
            return;
        }

        throw new SAXException("Authentication Response broken!");
    }

    /**
     * handler of the event that occurs when the xml parser
     * found a new element in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @param attrs
     * @throws SAXException
     */
    public void startElement(String uri,
                             String localName,
                             String qName,
                             Attributes attrs)
            throws SAXException {

        isTimeZone = false;
        if (attrs != null && attrs.getValue(0).equalsIgnoreCase("zimbraPrefTimeZoneId"))
            isTimeZone = true;

        //element's name
        String eName = localName;
        if ("".equals(eName)) eName = qName;

        if ("AuthResponse".equals(eName)) {
            this.foundAuthResponseStart = true;
            return;
        }
        //if the start of AuthRespose hasn't been found
        //element is not important
        if (!this.foundAuthResponseStart) return;
        if ("authToken".equals(eName) ||
                "lifetime".equals(eName) ||
                "sessionId".equals(eName)) {
            this.crtElement = eName;
        }
        if ("sessionId".equals(eName)) {
            this.crtValue = attrs.getValue("id");
        }
    }

    /**
     * handler of the event that occurs when the xml parser
     * founds the end of element's declaration in the XML document
     *
     * @param uri
     * @param localName
     * @param qName
     * @throws SAXException
     */
    public void endElement(String uri,
                           String localName,
                           String qName)
            throws SAXException {

        //if the end of the body has previously been found,
        // nothing mathers anymore
        if (this.foundAuthResponseEnd) return;
        //element's name
        String eName = localName;
        if ("".equals(eName)) eName = qName;
        if ("AuthResponse".equals(eName)) {
            this.foundAuthResponseEnd = true;
            return;
        }
        if ("authToken".equals(eName)) {
            this.authResponseFields.put(AuthResponse.K_AUTH_TOKEN, crtValue);
            return;
        }
        if ("lifetime".equals(eName)) {
            this.authResponseFields.put(AuthResponse.K_LIFETIME, crtValue);
            return;
        }
        if ("sessionId".equals(eName)) {
            this.authResponseFields.put(AuthResponse.K_SESSION_ID, crtValue);
            return;
        }
        if ("pref".equals(eName) && isTimeZone) {
            this.authResponseFields.put(AuthResponse.K_TIME_ZONE, crtValue);
        }

    }

    /**
     * handler of the event that occurs when character are found
     * inx the xml document
     *
     * @param charBuff
     * @param start
     * @param length
     * @throws SAXException
     */
    public void characters(char[] charBuff, int start, int length)
            throws SAXException {
        if ("authToken".equals(crtElement) ||
                "lifetime".equals(crtElement) ||
                "sessionID".equals(crtElement)) {
            this.crtValue = new String(charBuff, start, length);
        }
    }

    /**
     * @return a new AuthResonse object with the propreties obtained
     *         by parsing the xml document
     */
    public AuthResponse getAuthResponse() {
        return new AuthResponse(authResponseFields);
    }

}
